// Add_Ons.cpp

// User written functions for combustion calculations

#include <math.h>
#include "Combustion_Library.h"


// Gas constants
#define Rgas		10.73150	// ft3-psia/lbmol-R
#define RgasSI		8.31441E3	// m3-Pa/kgmol-K
#define R			1.98586		// Btu/lbmol-R
#define Rcalmol		1.987		// cal/mole-K
#define MwAir		28.96
#define MwAr		39.948
#define MwN2		28.0134
#define MwCO2		44.01
#define MwO2		31.9994
#define MwH2O		18.016


#define yN2			0.7809		// mole fractions in air.
#define yO2			0.2095
#define yAr			0.0096

// Conversion (multiply by to get English units).
#define ConvJkg		4.29922547582305E-04	// J/kg to Btu/lbm
#define ConvM3kg		1.60184633739601E+01	// m3/kg to ft3/lbm
#define Convcalmole		1.79882					// cal/mole to Btu/lbm
#define ConvmmHg	51.715758				// psia to mmHg

// Reference temperature and pressure (for initial guesses for secant
// method).
#define RefT		536.67				// R
#define RefP		14.6959487755134	// psia

// Iteration constraints for the secant method.
#define MaxErr		1.0E-7
#define MaxIter		1000


// Cv constants for Nitrogen (equation C-4).
const double N2G[] = {
	 3353.40610,			// beta is N2G[0]
	-2.18203473713518E+05,
	 1.01573580096247E+04,
	-1.65504721657240E+02,
	 7.43175999190430E+02,
	-5.14605623546025E-03,
	 5.18347156760489E-06,
	-1.05922170493616E-09,
	 2.98389393363817E+02
};


// Returns the molecular weight of comubstion products for combustion
// with at least 100% of stoichiometric air.  Returns -1 if partial 
// combustion (<100% stoichiometric air).  Also computes mole fractions
// of each combustion product.  PercentAir is fractional (1.0 = 100%).
double Mw_Products( double HC_ratio, double PercentAir, double *CO2,
				    double *H2O, double *Air, double *Ar, double *N2 )
{
	// Mw is not known for partial combustion.
	if( PercentAir < 0.999 ) return -1;

	double SpentO2 = (HC_ratio + 4)/4;

	// Computing the mole fractions of the mixture.
	*CO2 = 1/((HC_ratio + 2)/2 
			+ SpentO2*(PercentAir/yO2 - 1.0));
	*H2O = (*CO2)*HC_ratio/2;
	*Ar = (*CO2)*SpentO2*yAr/yO2;
	*N2 = (*CO2)*SpentO2*yN2/yO2;
	*Air = (*CO2)*SpentO2*(PercentAir - 1.0)/yO2;
	
	return (*CO2)*MwCO2 + (*H2O)*MwH2O + (*Air)*MwAir 
			+ (*Ar)*MwAr + (*N2)*MwN2;
}



// Enthalpy and entropy functions for combustion products as a mixture
// of real gases.  These functions return enthalpy and entropy per
// pound combustion products.  NOT pound mole.  Send in a percent 
// stoichiometric air less than 100% (incomplete combustion) to get
// the properties of air (per pound mole).  Incomplete combustion cannot be
// modeled since the unburned fuel is unaccounted for here.
// 
// PercentStoi is fractional (1 = 100%).  PercentStoi is the ratio of moles of 
// dry air supplied / moles of dry air needed for complete combustion
double H_Products( double P, double T, double HydroCarb, double PercentStoi )
{
	// Return the property of air if incomplete combustion.
	if( PercentStoi < 0.999 ) return H_Air(P,T)/MwAir;
	
	double xH2O, xCO2, xAir, xAr, xN2, MwProducts;

	// Get mole fractions and molecular weight of products.
	MwProducts = Mw_Products( HydroCarb, PercentStoi, &xCO2, &xH2O, &xAir,
							  &xAr, &xN2 );

	// Properties are not defined at zero pressure.  Since air can be
	// present (absent) at zero mole fraction, this guards against its
	// property evaluation at zero or negative pressure.
	if( xAir < 0.000001 ) xAir = 0.000001;

	// Compute enthalpy (enthalpy (P,T)) - enthalphy ref (P0=14.696 psia, T0 = 536.67))
	// Numbers (P0, T0) are used in the reference to remind the user
	// know what you have picked as your reference state when you use this function. 
	// Steam ref determined by curve fit of H_Steam (p fixed 14.696).
	// H_Products must be above 675 R 
	return (
		xH2O*H_Steam(P,T) + xCO2*H_CarbDiox(P,T) 
		+ xAir*H_Air(P,T) + xAr*H_Argon(P,T)
		+ xN2*H_Nitrogen(P,T)
		- xH2O*(19505.414) - xCO2*H_CarbDiox(14.696,536.67) 
		- xAir*H_Air(14.696,536.67) - xAr*H_Argon(14.696,536.67)
		- xN2*H_Nitrogen(14.696,536.67)
		)/MwProducts;
}
// 
// PercentStoi is fractional (1 = 100%).  PercentStoi is the ratio of moles of 
// dry air supplied / moles of dry air needed for complete combustion

double S_Products( double P, double T, double HydroCarb, double PercentStoi )
{
	// Return the property of air if incomplete combustion.
	if( PercentStoi < 0.999 ) return S_Air(P,T);
	
	double xH2O, xCO2, xAir, xAr, xN2, MwProducts;

	// Get mole fractions and molecular weight of products.
	MwProducts = Mw_Products( HydroCarb, PercentStoi, &xCO2, &xH2O, &xAir,
							  &xAr, &xN2 );
	
	// Properties are not defined at zero pressure.  Since air can be
	// present (absent) at zero mole fraction, this guards against its
	// property evaluation at zero or negative pressure.
	if( xAir < 0.000001 ) xAir = 0.000001;

	// Compute entropy (entropy (P,T)) - entropy ref (P0=14.696 psia, T0 = 536.67))
	// Here the -R*SUM(xi*ln(xi)) terms cancel in S - Sref.
	// Steam ref determined by curve fit of S_Steam (p fixed 14.696).
	// S_Products must be above 675 R 

	return (
		xH2O*S_Steam(P,T) + xCO2*S_CarbDiox(P,T) 
		+ xAir*S_Air(P,T) + xAr*S_Argon(P,T)
		+ xN2*S_Nitrogen(P,T)
		- xH2O*(29.6476)- xCO2*S_CarbDiox(14.696,536.67) 
		- xAir*S_Air(14.696,536.67) - xAr*S_Argon(14.696,536.67)
		- xN2*S_Nitrogen(14.696,536.67)
		)/MwProducts;
}


// Iterative functions to return the temperature at a given enthalpy
// or entropy, pressure, and composition.
double TfromH_Products( double P, double H, double HydroCarb, 
						double PercentStoi )
{
	// Initial guess using ideal gas approximation with
	// Cp = G4 (G4 is the most dominant term in Cp(T)) for
	// nitrogen, which is the dominant component of any
	// combustion product mixture.
	double T = H/(N2G[4]*ConvJkg*MwN2 + R) + RefT;
	double ThisT = T * 1.01, OldT;
	double ThisH = H_Products(P,ThisT,HydroCarb,PercentStoi) - H, OldH;
	double delT, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldT = ThisT; ThisT = T; OldH = ThisH;
		delT = OldT - ThisT;
		ThisH = H_Products( P, ThisT, HydroCarb, PercentStoi ) - H;
		T = ThisT - ThisH*delT/(OldH - ThisH);
		AproxErr = (ThisT - T)/(T);
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return T;
}

double TfromS_Products( double P, double S, double HydroCarb, 
						double PercentStoi )
{
	// Initial guess using ideal gas approximation with
	// Cp = G4 (G4 is the most dominant term in Cp(T)) for
	// nitrogen, which is the dominant component of any
	// combustion product mixture.
	double T = RefT*exp((S + log(P/RefP))/(N2G[4]*ConvJkg*MwN2 + R));
	double ThisT = T * 1.01, OldT;
	double ThisS = S_Products(P,ThisT,HydroCarb,PercentStoi) - S, OldS;
	double delT, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldT = ThisT; ThisT = T; OldS = ThisS;
		delT = OldT - ThisT;
		ThisS = S_Products( P, ThisT, HydroCarb, PercentStoi ) - S;
		T = ThisT - ThisS*delT/(OldS - ThisS);
		AproxErr = (ThisT - T)/(T);
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return T;
}