
#include "Refrig.h"

// Define Gas Constants
#define RGas_kJ_kg_K 0.0815  // R in kJ/kg-K 

#define ConvkJkG    1.0 // kPa-m^3 -> kJ

//Define R134a Physical Property Constants

#define MwR134a		102.3		// Molecular Weight
#define TcR134a		374.23		// Critical Temperature (K) 
#define PcR134a		4060.3		// Critical Pressure (kPa)
#define VcR134a		0.00194		// m^3/kg


#define T0_R134a 273.15		//Lower T Limit for Cv integration (K)

// Define iteration parameters

#define MaxErr 1.0E-6
#define MaxIter 100


// PressureEOS_R134a.  P-V-T equation for R134a
//
// in this M-H EOS for R134a the following units are used
//		P = kPa
//		T = K
//		V = m^3/kg
//		rho_m = 1/V	= kg/m^3

double PressureEOS_R134a(double V,  double T, const double b[], double Mw)
{

	return 	(
	RGas_kJ_kg_K*T* pow((V-b[1]),-1)
	+ (b[2] + b[3]*T+b[4]*exp(-1*b[14]*T/TcR134a))*pow((V-b[1]),-2)
	+ (b[5] + b[6]*T+b[7]*exp(-1*b[14]*T/TcR134a))*pow((V-b[1]),-3)
	+ (b[8] + b[9]*T+b[10]*exp(-1*b[14]*T/TcR134a))*pow((V-b[1]),-4)
	+ (b[11] + b[12]*T+b[13]*exp(-1*b[14]*T/TcR134a))*pow((V-b[1]),-5)
	);
}


// Secant_R134a
///////////////// Secant method root finder ////////////////////////////////  
// Inputs
//	  P_Known	Value of function where root (V_Target) is to be found.
//	  *V_Target	Desired variable and first argument of evaluated function.
//	  			MUST have an initial guess assigned when Secant is called.
//	  T_Known	Second argument of evaluated function.
//	  MaxErr		Maximum error (stopping criteria)
//    MaxIter		Maximum iterations allowed (stopping criteria)
// Outputs
//    *V_Target		Solution is in same address as initial guess.
//	  Secant_R134a	Number of iterations.
////////////////////////////////////////////////////////////////////////////
int SecantPT_V_R134a( double P_Known, double *V_Target, double T_Known )
{
//  Variable declarations
	//Lower limit (0.0022) on the volume for EOS 
    if(*V_Target < 0.0022 ) *V_Target = 0.0022; 
	double ThisGuess = *V_Target * 1.01, OldGuess; 

	double ThisResult = Fnc_P_R134a( ThisGuess, T_Known ) - P_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
//    Print Statement if method NOT working	
//    cout<<"AT Iteration "<<iterations<<": "<<(*V_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *V_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_P_R134a( *V_Target, T_Known ) - P_Known;
		*V_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		//Lower limit (0.0022) on the volume for EOS 
		//if(*V_Target < 0.0022) *V_Target = 0.0022;
		if(*V_Target < 0.0022) *V_Target = 0.0022 + iterations * 0.0001;
		AproxErr = (ThisGuess - *V_Target)/(*V_Target);
//
//    Print Statement use if method NOT working
//    cout<<"AT Iteration "<<iterations<<": Volume"<<(*V_Target)<<"     P="<<ThisResult+P_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}




double HResidualEOS_R134a(double V, double T, const double b[], double Mw )
{


	double  f = -T/(V-b[1])*b[3]+
		T/(V-b[1])*b[4]*b[14]/TcR134a*exp(-b[14]*T/TcR134a)
		-1/2*T/pow((V-b[1]),2)*b[6]
		+1/2*T/pow((V-b[1]),2)*b[7]*b[14]/TcR134a*exp(-b[14]*T/TcR134a)
		-1/3*T/pow((V-b[1]),3)*b[9]
		+1/3*T/pow((V-b[1]),3)*b[10]*b[14]/TcR134a*exp(-b[14]*T/TcR134a)
		-1/4*T/pow((V-b[1]),4)*b[12]
		+1/4*T/pow((V-b[1]),4)*b[13]*b[14]/TcR134a*exp(-b[14]*T/TcR134a)
		-(-b[2]-b[3]*T-b[4]*exp(-b[14]*T/TcR134a))/(V-b[1])
		-1/2*(-b[5]-b[6]*T-b[7]*exp(-b[14]*T/TcR134a))/pow((V-b[1]),2)
		-1/3*(-b[8]-b[9]*T-b[10]*exp(-b[14]*T/TcR134a))/pow((V-b[1]),3)
		-1/4*(-b[11]-b[12]*T
		-b[13]*exp(-b[14]*T/TcR134a))/pow((V-b[1]),4);
		
  f += PressureEOS_R134a(V, T, b, Mw)*V;   //+(PV)

  return f*ConvkJkG; 

}

double HIdealEOS_R134a(double T, const double G[], double Mw)
{
	
 return G[1]*(T - T0_R134a)+
	   1.0/2.0*G[2]*(T*T - T0_R134a*T0_R134a)+ 
	  1.0/3.0*G[3]*(pow(T,3) - pow(T0_R134a, 3) ) +
	  1.0/4.0*G[4]*(pow(T,4) - pow(T0_R134a, 4) ) +
      1.0*G[5]*(-pow(T,-1) + pow(T0_R134a, -1));
}



double PureEnthalpy_R134a(double V, double T, double Mw, const double R134a_b[],
											const double R134a_G[], double Href )
{
	double RealPart, IdealPart;
    RealPart = HResidualEOS_R134a(V, T,R134a_b,Mw);
    IdealPart = HIdealEOS_R134a(T,R134a_G, Mw);
//	cout<<"Ideal Part :"<<IdealPart<<"\n";
	return IdealPart + RealPart - Href;
}



double SResidualEOS_R134a(double V, double T, const double b[], double Mw )
{
	double f = -(b[3]-b[4]*b[14]/TcR134a*exp(-b[14]*T/TcR134a))/(V-b[1])
		-1.0/2.0*(b[6]-b[7]*b[14]/TcR134a*exp(-b[14]*T/TcR134a))/pow((V-b[1]),2)
		-1.0/3.0*(b[9]-b[10]*b[14]/TcR134a*exp(-b[14]*T/TcR134a))/pow((V-b[1]),3)
		-1.0/4.0*(b[12]-b[13]*b[14]/TcR134a*exp(-b[14]*T/TcR134a))/pow((V-b[1]),4);


	f += RGas_kJ_kg_K*log(V-b[1]);        //+ Rln(V-b[1])
	return f;
	 
}

double SIdealEOS_R134a(double T, const double G[], double Mw)
{

double f = 1.0/3.0*G[4]*pow(T,3)+1.0/2.0*G[3]*pow(T,2)+G[2]*T+G[1]*log(T)-1.0/2.0*G[5]/pow(T,2)
	-(1.0/3.0*G[4]*pow(T0_R134a,3)+1.0/2.0*G[3]*pow(T0_R134a,2)
	+G[2]*T0_R134a+G[1]*log(T0_R134a)-1.0/2.0*G[5]/pow(T0_R134a,2));

return f;
}

double PureEntopy_R134a(double V, double T, double Mw, const double R134a_b[],
						                  const double R134a_G[],double Sref)
{
	double RealPart, IdealPart;
    RealPart = SResidualEOS_R134a(V, T,R134a_b,Mw);
    IdealPart = SIdealEOS_R134a(T,R134a_G, Mw);
//  print if error
//	cout<<"Ideal Part :"<<IdealPart<<"\n";
	return IdealPart + RealPart - Sref;
}




int SecantHP_T_R134a( double H_Known, double *T_Target, double P_Known )
{
//  Variable declarations
	double ThisGuess = *T_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_H_R134a(P_Known, ThisGuess) - H_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *T_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_H_R134a(P_Known, *T_Target) - H_Known;
		*T_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		AproxErr = (ThisGuess - *T_Target)/(*T_Target);
//
// Print Statement use if method NOT working
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"Approx Error"<<AproxErr<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}


int SecantSP_T_R134a( double S_Known, double *T_Target, double P_Known )
{
	{
//  Variable declarations 
//Note: we had to check T_Guess compared to Tsat
	if(*T_Target < Fnc_Tsat_R134a(P_Known) ) *T_Target = Fnc_Tsat_R134a(P_Known);

	double ThisGuess = *T_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_S_R134a(P_Known, ThisGuess) - S_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)-273.15<<" S= "<<ThisResult+S_Known<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *T_Target; OldResult = ThisResult;
	    delX = OldGuess - ThisGuess;
		ThisResult = Fnc_S_R134a(P_Known, *T_Target) - S_Known;
    	*T_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		AproxErr = (ThisGuess - *T_Target)/(*T_Target);
//
// Print Statement use if method NOT working
//	cout<<"AT"<<iterations<<"T= "<<(*T_Target) -273.15<<" S= "<<ThisResult+S_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}

}



int SecantHT_P_R134a( double H_Known, double *P_Target, double T_Known )
{
	{
//  Variable declarations
	double ThisGuess = *P_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_H_R134a(ThisGuess, T_Known) - H_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *P_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_H_R134a(*P_Target, T_Known ) - H_Known;
		*P_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		if (*P_Target < 0.57) *P_Target = 0.57;
		AproxErr = (ThisGuess - *P_Target)/(*P_Target);
//
// Print Statement use if method NOT working
// cout<<"AT Iter "<<iterations<<":P= "<<(*P_Target)<<" H="<<ThisResult+H_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}

}


int SecantST_P_R134a( double S_Known, double *P_Target, double T_Known )
{
	{
//  Variable declarations
	double ThisGuess = *P_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_S_R134a(ThisGuess, T_Known) - S_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *P_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_S_R134a(*P_Target, T_Known ) - S_Known;
		*P_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		if (*P_Target < 0.57) *P_Target = 0.57;
		AproxErr = (ThisGuess - *P_Target)/(*P_Target);
//
// Print Statement use if method NOT working
// cout<<"AT Iter "<<iterations<<":P= "<<(*P_Target)<<" S="<<ThisResult+S_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}

}



//saturated conditions
double PureSatPressure_R134a(double T)
{
  double A = 4.069889E+01, B = -2.362540E+03,C = -1.306883E+01, D = 7.616005E-03,
		   E = 2.342564E-01, F = 3.761111E+02;
	return pow(10, A+B/T+C*log10(T)+D*T+E*((F-T)/T)*log10(F-T) );

}



double SecantPsat_Tsat_R134a(double P_Known, double* T_Target)
{
  //Variable declarations
	if (*T_Target < 173.15) *T_Target = 173.15;
    if (*T_Target > 370.15) *T_Target = 370.15;

	double ThisGuess = *T_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_Psat_R134a(ThisGuess) - P_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *T_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_Psat_R134a(*T_Target) - P_Known;
		*T_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		if (*T_Target < 173.15) *T_Target = 173.15;
        if (*T_Target > 374.15) *T_Target = 374.15; 
		AproxErr = (ThisGuess - *T_Target)/(*T_Target);
//
// Print Statement use if method NOT working
// cout<<"AT Iteration "<<iterations<<":Tsat= "<<(*T_Target)<<"Psat="<<ThisResult+P_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}


double PureSatLDensity_R134a(double T)
{
	double Af = 5.281464E+02,Bf = 7.551834E+02,Cf = 1.028676E+03,
			Df = -9.491172E+02, Ef = 5.935660E+02;
	return Af+Bf*pow(1-T/TcR134a, 1.0/3.0)+Cf*pow(1- T/TcR134a, 2.0/3.0)+
			Df*pow(1- T/TcR134a, 1.0)+Ef*pow(1.0- T/TcR134a, 4.0/3.0);
}


double PureEnthalpySat_R134a(double T)
{

//		if (T < 173.15) return -100;
        if (T < 173.149999999) return -100;
        if (T > 374.15) return -100; 

//  Otherwise, Picewise Saturated Liquid Enthalpy 

	if (T <= 303.15)	
	{	
	double A = 10.78256394, B = 0.012086986, C = 3.1321E-05;
	double T0 = 173.15;
	return RGas_kJ_kg_K*(A*T+B*T*T/2+C*T*T*T/3)				
					-RGas_kJ_kg_K*(A*T0+B*T0*T0/2
					+C*T0*T0*T0/3) +77.3;
	}
	if (T <= 366.15)	
	{
	double A = 24.29115431, B = -0.140199878, C = 0.000386451;
	double T0 = 302.15;
	return RGas_kJ_kg_K*(A*T+B*T*T/2+C*T*T*T/3)				
					-RGas_kJ_kg_K*(A*T0+B*T0*T0/2
							+C*T0*T0*T0/3) +240.4;
	}
	else					
	// T <= 374.15)	
	{
	double A = 21.25986498, B = -4.757567787, C = 0.013036761;
	double T0 = 364.15;
	return RGas_kJ_kg_K*(A*T+B*T*T/2+C*T*T*T/3)				
					-RGas_kJ_kg_K*(A*T0+B*T0*T0/2
						+C*T0*T0*T0/3) +345.7;
	}
}


double PureEntropySat_R134a(double T)
{

 	   //	if (T < 173.15) return -100;
		if (T < 173.149999999) return -100;
        if (T > 374.15) return -100; 

//  Otherwise, Picewise Saturated Liquid Entropy 

	if (T <= 303.15)	
	{	
	double A = 10.78256394, B = 0.012086986, C = 3.1321E-05;
	double T0 = 173.15;
	return RGas_kJ_kg_K*(A*log(T)+B*T+C*T*T/2)				
					-RGas_kJ_kg_K*(A*log(T0)+B*T0
						+C*T0*T0/2) + 0.4448;
	}
	else
	if (T <= 366.15)	
	{
	double A = 24.29115431, B = -0.140199878, C = 0.000386451;
	double T0 = 302.15;
	return RGas_kJ_kg_K*(A*log(T)+B*T+C*T*T/2)				
					-RGas_kJ_kg_K*(A*log(T0)+B*T0
						+C*T0*T0/2) + 1.1396;
	}					
	else
	// (T <= 374.15)	
	{
	double A = 21.25986498, B = -4.757567787, C = 0.013036761;
	double T0 = 364.15;
	return RGas_kJ_kg_K*(A*log(T)+B*T+C*T*T/2)				
					-RGas_kJ_kg_K*(A*log(T0)+B*T0
						+C*T0*T0/2) + 1.4524;

	}
}



double SecantHsat_Tsat_L_R134a(double H_Known, double* T_Target)
{
  //Variable declarations
	if (*T_Target < 173.15) *T_Target = 173.15;
    if (*T_Target > 370.15) *T_Target = 370.15;

	double ThisGuess = *T_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_Hsat_L_R134a(ThisGuess) - H_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *T_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_Hsat_L_R134a(*T_Target) - H_Known;
		*T_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		if (*T_Target < 173.15) *T_Target = 173.15;
        if (*T_Target > 374.15) *T_Target = 374.15; 
		AproxErr = (ThisGuess - *T_Target)/(*T_Target);
//
// Print Statement use if method NOT working
// cout<<"AT Iteration "<<iterations<<":Tsat= "<<(*T_Target)<<"Hsat="<<ThisResult+H_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}




double SecantSsat_Tsat_L_R134a(double S_Known, double* T_Target)
{
  //Variable declarations
	if (*T_Target < 173.15) *T_Target = 173.15;
    if (*T_Target > 370.15) *T_Target = 370.15;

	double ThisGuess = *T_Target * 1.01, OldGuess; 
	double ThisResult = Fnc_Ssat_L_R134a(ThisGuess) - S_Known, OldResult;
	double delX, AproxErr;
	int iterations = 0;
//
// Print Statement if method NOT working	
// cout<<"AT Iteration "<<iterations<<": "<<(*T_Target)<<"\n";
//
//
//  The Secant Method
//	
	do {
		OldGuess = ThisGuess; ThisGuess = *T_Target; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = Fnc_Ssat_L_R134a(*T_Target) - S_Known;
		*T_Target = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		if (*T_Target < 173.15) *T_Target = 173.15;
        if (*T_Target > 374.15) *T_Target = 374.15; 
		AproxErr = (ThisGuess - *T_Target)/(*T_Target);
//
// Print Statement use if method NOT working
// cout<<"AT Iteration "<<iterations<<":Tsat= "<<(*T_Target)<<"Ssat="<<ThisResult+S_Known<<"\n";
//		
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	
	return iterations;
}


