// Cpv.cpp
// This code is based on a polynomial Cp equation for vapor phase 1 bar.
// Constants used here are found in Rodriguez-Toral, 1999 (originally from Prothero, 1969).
// The Cp equation used was fit over a single temperature interval.
//
// The Cp equation can be broken into several temperature intervals.  See for example, 
// Bathie, 1996.
// Program values here are in approximate agreement
// with values in Appendix A of Bathie, 1996
// 

#include<math.h>
#include "Combustion_Library.h"

#define MwC5H12		72.15
#define MwCO		38.00
#define MwNO		30.00
#define MwOH		17.00
#define MwH			 1.00
#define MwO			16.00
#define MwH2O		18.016 

//From Rodriguez-Toral's, 1999
#define HrefC5H12		35076.48184	//in cal/mole
#define SrefC5H12		-83.1739	//in cal/mole/K	
#define HrefCO			26417.0		//in cal/mole
#define SrefCO			-47.214		//in cal/mole/K	
#define HrefNO			-21580.0	//in cal/mole
#define SrefNO			-50.347		//in cal/mole/K	
#define HrefOH			-9432.0		//in cal/mole
#define SrefOH			-43.88		//in cal/mole/K	
#define HrefH			-52100.0	//in cal/mole
#define SrefH			-27.392		//in cal/mole/K	
#define HrefO			-59559.0	//in cal/mole
#define SrefO			-38.468		//in cal/mole/K	
#define HrefH2O_V		 57798.0	//in cal/mole
#define SrefH2O_V		-45.106		//in cal/mole/K	



#define Rcalmol		1.987		// cal/mole-K

// Reference temperature and volume (ideal gas)
#define T0			298.15				// K
#define V0			391.895357897		// ft3/lbmole
#define P0			 14.5038			// psia


// Conversion (multiply by to get English units).
#define ConvJkg		4.29922547582305E-04	// J/kg to Btu/lbm
#define ConvM3kg		1.60184633739601E+01	// m3/kg to ft3/lbm
#define Convcalmole		1.79882					// cal/mole to Btu/lbm
#define ConvmmHg	51.715758				// psia to mmHg

// Reference temperature and pressure (for initial guesses for secant
// method).
#define RefT		536.67				// R
#define RefP		14.6959487755134	// psia

// Iteration constraints for the secant method.
#define MaxErr		1.0E-7
#define MaxIter		1000


//Constants for the specific heat calculations (gaseous chemical species)
//using a polynomial equation.
//See Rodrigues-Toral's Thesis for detail

// Constants for n-Pentane.
const double nC5H12A[] = {
	 0,				// unused element.
	-0.2857971878E+01,
	 0.1288347906E+03,
	-0.9084176048E+02,
	 0.4823747357E+02,
	-0.2197105629E+02,
	 0.6792072625E+01,
    -0.9221203575
};

// Constants for Carbon Monoxide.
const double COA[] = {
	 0,				// unused element.
	 0.7812249E+01,
	-0.6668293E+01,
	 0.1728296E+02,
	-0.1728709E+02,
	 0.8860125E+01,
	-0.2314819E+01,
     0.2447785
};

// Constants for Nitrogen Oxide.
const double NOA[] = {
	 0,				// unused element.
	 0.8462334E+01,
	-0.1040669E+02,
	 0.2754876E+02,
	-0.3028119E+02,
	 0.1718511E+02,
	-0.4957260E+01,
     0.5755281
};

// Constants for Hydroxyl.
const double OHA[] = {
	 0,				// unused element.
	 0.7615100E+01,
	-0.1936000E+01,
	 0.8770000,
	 0.2615300E+01,
	-0.2690900E+01,
	 0.9778900,
    -0.1269500
};

// Constants for Hydrogen Atom.
const double HA[] = {
	 0,				// unused element.
	 0.4968000E+01,
	 0.0,
	 0.0,
	 0.0,
	 0.0,
	 0.0,
     0.0
};

// Constants for Oxygen Atom.
const double OA[] = {
	 0,				// unused element.
	 0.5974134E+01,
	-0.4241883E+01,
	 0.7931254E+01,
	-0.7944230E+01,
	 0.4403357E+01,
	-0.1271341E+01,
	 0.1491408
};

// Constants for Water Vapor.
const double H2O_VA[] = {
	 0,				// unused element.
	 0.7988860E+01,
	-0.1506271E+01,
	 0.6661376E+01,
	-0.4655970E+01,
	 0.1696464E+01,
	-0.3706212,
	 0.3992444E-01
};

double H_C5H12( double P, double T )
{
	return PureEnthalpyPoly( T, MwC5H12, nC5H12A, HrefC5H12);
}

double H_CO( double P, double T )
{
	return PureEnthalpyPoly( T, MwCO, COA, HrefCO);
}

double H_NO( double P, double T )
{
	return PureEnthalpyPoly( T, MwNO, NOA, HrefNO);
}

double H_OH( double P, double T )
{
	return PureEnthalpyPoly( T, MwOH, OHA, HrefOH);
}

double H_H( double P, double T )
{
	return PureEnthalpyPoly( T, MwH, HA, HrefH);
}

double H_O( double P, double T )
{
	return PureEnthalpyPoly( T, MwO, OA, HrefO);
}

double H_H2O_V( double P, double T )
{
	return PureEnthalpyPoly( T, MwH2O, H2O_VA, HrefH2O_V);
}

double S_C5H12( double P, double T )
{
	return PureEntropyPoly( P, T, MwC5H12, nC5H12A, SrefC5H12);
}

double S_CO( double P, double T )
{
	return PureEntropyPoly( P, T, MwCO, COA, SrefCO);
}

double S_NO( double P, double T )
{
	return PureEntropyPoly( P, T, MwNO, NOA, SrefNO);
}

double S_OH( double P, double T )
{
	return PureEntropyPoly( P, T, MwOH, OHA, SrefOH);
}

double S_H( double P, double T )
{
	return PureEntropyPoly( P, T, MwH, HA, SrefH);
}

double S_O( double P, double T )
{
	return PureEntropyPoly( P, T, MwO, OA, SrefO);
}

double S_H2O_V( double P, double T )
{
	return PureEntropyPoly( P, T, MwH2O, H2O_VA, SrefH2O_V);
}

double T_HP_C5H12( double H, double P) { 
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefC5H12)/29.2705;
	return T_XP( H, RefP, H_C5H12, InitGuess ); }

double T_HP_CO( double H, double P ) {
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefCO)/COA[1];
	return T_XP( H, RefP, H_CO, InitGuess ); }

double T_HP_NO( double H, double P ) {
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefNO)/NOA[1];
	return T_XP( H, RefP, H_NO, InitGuess ); }

double T_HP_OH( double H, double P ) {
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefOH)/OHA[1];
	return T_XP( H, RefP, H_OH, InitGuess ); }

double T_HP_H( double H, double P ) {
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefH)/HA[1];
	return T_XP( H, RefP, H_H, InitGuess ); }

double T_HP_O( double H, double P ) {
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefO)/OA[1];
	return T_XP( H, RefP, H_O, InitGuess ); }

double T_HP_H2O_V( double H, double P ) {
	double InitGuess;
	InitGuess = RefT + (H/Convcalmole+HrefH2O_V)/H2O_VA[1];
	return T_XP( H, RefP, H_H2O_V, InitGuess ); }


double T_SP_C5H12( double S, double P) { 
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefC5H12)
				       +Rcalmol*log(P/RefP))/29.2705);
	return T_XP( S, P, S_C5H12, InitGuess ); }

double T_SP_CO( double S, double P ) {
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefCO)
				       +Rcalmol*log(P/RefP))/COA[1]);
	return T_XP( S, P, S_CO, InitGuess ); }

double T_SP_NO( double S, double P ) {
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefNO)
				       +Rcalmol*log(P/RefP))/NOA[1]);
	return T_XP( S, P, S_NO, InitGuess ); }

double T_SP_OH( double S, double P ) {
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefOH)
				       +Rcalmol*log(P/RefP))/OHA[1]);
	return T_XP( S, P, S_OH, InitGuess ); }

double T_SP_H( double S, double P ) {
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefH)
				       +Rcalmol*log(P/RefP))/HA[1]);
	return T_XP( S, P, S_H, InitGuess ); }

double T_SP_O( double S, double P ) {
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefO)
				       +Rcalmol*log(P/RefP))/OA[1]);
	return T_XP( S, P, S_O, InitGuess ); }

double T_SP_H2O_V( double S, double P ) {
	double InitGuess;
	InitGuess = exp(log(RefT) 
		           + ((S*1.8/Convcalmole+SrefH2O_V)
				       +Rcalmol*log(P/RefP))/H2O_VA[1]);
	return T_XP( S, P, S_H2O_V, InitGuess ); }




// T_XP,PureEnthalpyPoly,PureEntropyPoly are needed to exceute Cpv.cpp stand alone.
// You must uncomment T_XP, PureEnthalpyPoly,PureEntropyPoly.


/*
// Enthalpy function for pure components.
// Using Polynomial coefficients for Cp
double PureEnthalpyPoly( double T, double Mw, 
					 const double A[], double Href)
{
	double EnthalpyinCalperMol, x, x0, c=1.0*1E-3;
	T /= 1.8;						// T to Kelvin

	x=c*T;
	x0=c*T0;
	EnthalpyinCalperMol = A[1]*(x-x0)/c
						 +A[2]/2.0*(pow(x,2)-pow(x0,2))/c
						 +A[3]/3.0*(pow(x,3)-pow(x0,3))/c
						 +A[4]/4.0*(pow(x,4)-pow(x0,4))/c
						 +A[5]/5.0*(pow(x,5)-pow(x0,5))/c
						 +A[6]/6.0*(pow(x,6)-pow(x0,6))/c
						 +A[7]/7.0*(pow(x,7)-pow(x0,7))/c;

	return (EnthalpyinCalperMol - Href)*Convcalmole;

}


// Entropy function for pure components.
// Using Polynomial coefficients for Cp
double PureEntropyPoly( double P, double T, double Mw, 
					 const double A[], double Sref)
{
	double EntropyinCalperMolK, x, x0;
	T /= 1.8;						// T to Kelvin

	x=1.0*1E-3*T;
	x0=1.0*1E-3*T0;
	EntropyinCalperMolK = A[1]*log(T/T0)
						 +A[2]/1.0*(x-x0)
						 +A[3]/2.0*(pow(x,2)-pow(x0,2))
						 +A[4]/3.0*(pow(x,3)-pow(x0,3))
						 +A[5]/4.0*(pow(x,4)-pow(x0,4))
						 +A[6]/5.0*(pow(x,5)-pow(x0,5))
						 +A[7]/6.0*(pow(x,6)-pow(x0,6))
						 -Rcalmol*log(P/P0);

	return (EntropyinCalperMolK - Sref)*Convcalmole/1.8;

}*/

/*
//Functions to calculate T given H or S and P

double T_XP( double X, double P, double (*XFunc)( double, double ), double InitGuess )
{
	
	double T = InitGuess;
	double ThisT = T * 1.01, OldT;
	double ThisX = (*XFunc)( P, ThisT ) - X, OldX;
	double delT, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldT = ThisT; ThisT = T; OldX= ThisX;
		delT = OldT - ThisT;
		ThisX = (*XFunc)( P, ThisT ) - X;
		T = ThisT - ThisX*delT/(OldX - ThisX);
		AproxErr = (ThisT - T)/(T);
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	if (iterations < MaxIter) return T;
	else return -iterations;

}
*/