// Reynolds_Calls.cpp

#include "Combustion_Library.h"
#include <math.h>

// Gas constants
#define Rgas		10.73150	// ft3-psia/lbmol-R
#define RgasSI		8.31441E3	// m3-Pa/kgmol-K
#define R			1.98586		// Btu/lbmol-R
#define Rcalmol		1.987		// cal/mole-K
#define MwAir		28.96
#define MwAr		39.948
#define MwN2		28.0134
#define MwCO2		44.01
#define MwO2		31.9994
#define MwH2O		18.016

#define MwNH3		17.03
#define MwC4H10		58.12
#define MwC2H6		30.07
#define MwC2H4		28.054
#define MwH2		2.0159
#define MwCH4		16.043
#define MwC8H18		114.22
#define MwO2		31.9994
#define MwC3H8		44.09

#define MwC5H12		72.15
#define MwCO		38.00
#define MwNO		30.00
#define MwOH		17.00
#define MwH			 1.00
#define MwO			16.00
#define yN2			0.7809		// mole fractions in air.
#define yO2			0.2095
#define yAr			0.0096


// Tc and Pc defined here are used for inital guesses NOT Reynolds EOS

#define TcH2O		647.286  // Critical Temperature for Water (K)
#define PcH2O		22.089   // Critical Pressure for Water (MPa)
#define omegaH2O	  0.040  // Omega for Water
#define rhocH2O		317.0    // Critical Density for Water (kg/m3)
#define TcN2		126.200  // Critical Temperature for Nitrogen (K)
#define PcN2		  3.4000   // Critical Pressure for Nitrogen (MPa)
#define omegaN2		  0.040   // Omega for Nitrogen

#define TcAr		150.8  
#define PcAr		  4.87 
#define omegaAr		  0.0  

#define TcCO2		304.2  
#define PcCO2		  7.38 
#define omegaCO2	  0.225  

#define TcNH3		406.80
#define PcNH3		11.28
#define omegaNH3	0.250  

#define TcC4H10		425.2  
#define PcC4H10		  3.80 
#define omegaC4H10	  0.193  

#define TcC2H6		305.4  
#define PcC2H6		  4.88 
#define omegaC2H6	  0.098  

#define TcC2H4		282.4  
#define PcC2H4		  5.04 
#define omegaC2H4	  0.085  

#define TcH2		33.2  
#define PcH2		  1.30 
#define omegaH2		  -0.22  

#define TcCH4		190.6  
#define PcCH4		  4.60 
#define omegaCH4	  0.008  

#define TcC8H18		568.8  
#define PcC8H18		  2.48 
#define omegaC8H18	  0.394  

#define TcO2		154.6  
#define PcO2		  5.05 
#define omegaO2		  0.021  

#define TcC3H8		369.8  
#define PcC3H8		  4.25 
#define omegaC3H8	  0.152  


// Reference enthalpy and entropy values to match the
// corresponding figures in standard tables

// for the Reynolds Tables

#define HrefAr			-4780.1
#define SrefAr			-25.4455
#define HrefN2			-5533.27
#define SrefN2			-29.7045
#define HrefH2Ol		-19733.8	
#define SrefH2Ol		-29.3438
#define HrefH2Og		-19732.2
#define SrefH2Og		-29.3427
#define HrefCO2			-8097.48
#define SrefCO2			-24.5384

#define HrefNH3			-12289.7
#define SrefNH3			-27.7589
#define HrefC4H10		-15083.95
#define SrefC4H10		-33.7187
#define HrefC2H6		-9860.34
#define SrefC2H6		-28.8369
#define HrefC2H4		-8549.6
#define SrefC2H4		-26.5455
#define HrefH2			-3884.12
#define SrefH2			-23.7618
#define HrefCH4			-6769.84
#define SrefCH4			-27.4267
#define HrefC8H18		-17849.4
#define SrefC8H18		-28.3646
#define HrefO2			-6390.48
#define SrefO2			-33.468	
#define HrefC3H8		-11406.5
#define SrefC3H8		-28.1245

#define HrefAir			-5704.61
#define SrefAir			-31.6521



//***********************************



// Conversion (multiply by to get English units).
#define ConvJkg		4.29922547582305E-04	// J/kg to Btu/lbm
#define ConvM3kg		1.60184633739601E+01	// m3/kg to ft3/lbm
#define Convcalmole		1.79882					// cal/mole to Btu/lbm
#define ConvmmHg	51.715758				// psia to mmHg

// Reference temperature and pressure (for initial guesses for secant
// method).
#define RefT		536.67				// R
#define RefP		14.6959487755134	// psia

// Iteration constraints for the secant method.
#define MaxErr		1.0E-7
#define MaxIter		1000

// P-rho-T equation constants for Water (equation Q-2 / P-6).
const double H2OA[][8]={{1000.0,4.8E-3,0.0,0.0,0.0,0.0,0.0,0.0},

		{0.0, 2.9492937E-2, -5.1985860E-3, 6.8335354E-3,
		-1.5641040E-4, -6.3972405E-3, -3.9661401E-3, -6.9048554E-4},

		{0.0, -1.3213917E-4, 7.7779182E-6, -2.6149751E-5,	
		-7.2546108E-7, 2.6409282E-5, 1.5453061E-5, 2.7407416E-6},

		{0.0,	2.7464632E-7, -3.3301902E-8, 6.5326396E-8, 
		-9.2734289E-9, -4.7740374E-8, -2.9142470E-8, -5.1028070E-9},

		{0.0, -3.6093828E-10, -1.6254622E-11,-2.6181978E-11,
		4.3125840E-12, 5.6323130E-11, 2.9568796E-11, 3.9636085E-12},

		{0.0, 3.4218431E-13, -1.7731074E-13, 0.0,
		0.0, 0.0, 0.0, 0.0},

		{0.0, -2.4450042E-16, 1.2748742E-16, 0.0,
		0.0, 0.0, 0.0, 0.0},

		{0.0, 1.5518535E-19, 1.3746153E-19, 0.0,
		0.0, 0.0, 0.0, 0.0},

		{0.0, 5.9728487E-24, 1.5597836E-22, 0.0,
		0.0, 0.0, 0.0, 0.0},

		{0.0, -4.1030848E-1, 3.3731180E-1, -1.3746618E-1,
		6.7874983E-3, 1.3687317E-1, 7.9847970E-2, 1.3041253E-2},

		{0.0, -4.1605860E-4, -2.0988866E-4, -7.3396848E-4,
		1.0401717E-5, 6.4581880E-4, 3.9917570E-4, 7.1531353E-5}

};

	//Ta is A[0,0], E is A[0,1]
// P-rho-T equation constants for Water (equation Q-2 / P-6).
const double rho_aH2O[] = { 
				0,		// not used
				634,
				1000,
				1000,
				1000,
				1000,
				1000,
				1000 };

// Saturation Pressure equation constants for Water (equation S-6).
const double H2OF[] = { 
				 0.01,		// a
				-7.4192420,
				 2.9721000E-1,
				-1.1552860E-1,
				 8.6856350E-3,
				 1.0940980E-3,
				-4.3999300E-3,
				 2.5206580E-3,
				-5.2186840E-4,
				338.15 };	// Tp

// Saturated Liquid Density equation constants for Water (equation D-5).
const double H2OD[] = { 
				 0,		// not used
				 3.6711257,
				-2.8512396E+1,
				 2.2265240E+2,
				-8.8243852E+2,
				 2.0002765E+3,
				-2.6122557E+3,
				 1.8297674E+3,
				-5.3350520E+2};	

// P-rho-T equation constants for Air (equation P-4).
const double AirA[33] = {
	 5.97105475117183E-06,		// gamma is AirA[0]
	 1.55623098409137E-01,
	 1.25288666202326E+01,
	-2.92541568638838E+02,
	 4.29432480725523E+03,
	-5.58450959675108E+05,
	 3.92054480883008E-04,
	-4.40985641881347E-02,
	 5.86387178724129E-04,
	 7.97411385439405E+04,
	 9.88045320906742E-09,
	 2.97999237261289E-04,
	-6.81783040959070E-02,
	 2.02551630992042E-07,	
	-1.62724281849497E-07,
	-1.06340143152999E-04,
	 3.51428501875049E-10,
	-1.70388092279449E-13,
	 5.91103444646786E-11,
	-1.05363473794348E-14,
	-7.32732651196979E+04,
	-5.42674649924748E+05,
	-4.48935466142735E-01,
	 2.81453138446295E+02,
	-8.83132042791851E-07,
	-1.32229814838386E-05,
	-2.16521865046609E-12,
	-1.47835008246593E-09,
	-6.93219849301501E-19,
	 6.06743598768355E-17,
	-3.20538718135891E-24,
	-4.73178337355130E-23,
	 3.83950822306912E-22
};

// P-rho-T equation constants for Argon (equation P-3).
const double ArA[] = {
	 3.5E-06,				// gamma
	 1.9825921E-01,
	-8.1733119E+01,
	 1.7777470E+03,
	-8.2406544E+05,
	 3.1666098E+07,
	-4.4202671E-05,
	 6.2161420E-02,
	 1.1443248E+00,
	 4.7797520E-07,
	-1.9645227E-04,
	-2.1572754E-10,
	 1.6544141E-07,
	-2.8142112E-11,
	 8.2532059E+01,
	-9.1538377E+03,
	-1.8340752E+06,
	-3.3858136E-03,
	 1.5532886E+00,
	-6.7479568E+01
};

// P-rho-T equation constants for Nitrogen (equation P-4).
const double N2A[] = {
	 7.13602531283233E-06,	// gamma
	 1.75889959256970E-01,
	 1.38197604384933E+01,
	-3.14918412133921E+02,
	 4.40300150239380E+03,
	-5.45358971644916E+05,
	 4.84413320182919E-04,
	-5.18964416491365E-02,
	 6.57265859197103E-04,
	 8.51299771713314E+04,
	 1.33459405162578E-08,
	 3.83381319826746E-04,
	-8.35421151028455E-02,
	 2.84874912286101E-07,
	-2.38296116270360E-07,
	-1.48321912935764E-04,
	 5.62605853190540E-10,
	-2.98201050924595E-13,
	 9.85319087685241E-11,
	-1.92002176056468E-14,
	-7.82250103373122E+04,
	-5.51801778744598E+05,
	-5.72781957607352E-01,
	 3.25760529488327E+02,
	-1.34659309828737E-06,
	-1.92036423064911E-05,
	-3.94564337674524E-12,
	-2.44388245328965E-09,
	-1.50970602460077E-18,
	 1.25854885346038E-16,
	-8.34271144923969E-24,
	-1.17299202018417E-22,
	 9.06544823455730E-22
};

// P-rho-T equation constants for Carbon Dioxide (equation P-3).
const double CO2A[] = {
	 5.0E-06,			// gamma
	 2.2488558E-01,
	-1.3717965E+02,
	-1.4430214E+04,
	-2.9630491E+06,
	-2.0606039E+08,
	 4.5554393E-05,
	 7.7042840E-02,
	 4.0602371E+01,
	 4.0029509E-07,
	-3.9436077E-04,
	 1.2115286E-10,
	 1.0783386E-07,
	 4.3962336E-11,
	-3.6505545E+04,
	 1.9490511E+07,
	-2.9186718E+09,
	 2.4358627E-02,
	-3.7546530E+01,
	 1.1898141E+04
};


// P-rho-T equation constants for Ammonia (equation Q-1 / P-6).
//						Ta	  tau_c	
const double NH3A[][7]={{500.0,1.2333498,0.0,0.0,0.0,0.0,0.0},

		{0.0, -6.453022304053E-03, -1.371992677050E-02, -8.100620315713E-03, 
		-4.880096421085E-03, -1.202877562682E-02, 6.806345929616E-03},

		{0.0, 8.080094367688E-06,	1.435692000561E-05,	-4.505297669943E-05, 
		-1.661889985705E-04, 3.790895022982E-05,	-4.073020833373E-05},

		{0.0, 1.032994880724E-09,	5.584395580933E-08,	4.920166508177E-07,	
		1.737835999473E-06,	-3.087491526377E-08, 7.148353041627E-08},

		{0.0, -8.948264632008E-12, -1.697777441391E-10, -1.236532371672E-09, 
		-7.812161168317E-09, 1.779548269140E-12,	-3.897461095850E-11},

		{0.0, -6.692285882015E-14, -1.753943775320E-15, 2.085533713355E-13, 
		2.134894661440E-11,	0.000000000000E+00,	0.000000000000E+00},

		{0.0, 2.473417459954E-16,	2.999839155475E-16,	4.509080578790E-15,	
		-3.798084988179E-14, 0.000000000000E+00, 0.000000000000E+00},

		{0.0, -3.065578854310E-19, 2.411655109855E-20, -9.323356799989E-18, 
		4.272409853059E-17,	0.000000000000E+00,	0.000000000000E+00},

		{0.0, 1.617910033375E-22, -5.074780704643E-22,  8.139470397409E-21, 
		-2.745871062656E-20,	0.000000000000E+00,	0.000000000000E+00},

		{0.0, -2.782168879368E-26, 2.988129173133E-25, -2.772597352058E-24, 
		7.668928677925E-24,	0.000000000000E+00,	0.000000000000E+00},

};

// P-rho-T equation constants for Butane (equation P-2).
const double C4H10A[] = {
	1.681913E-03,	//B0
	2.588747E+02,	//A0
	3.374153E+07,	//C0
	4.544053E+08,	//D0
	1.749633E+09,	//E0
	1.054551E-05,	//b
	6.081677E-01,	//a
	1.728889E+02,	//d
	1.846013E+05,	//c
	4.968935E-09,	//alpha
	8.700251E-06	//gamma
};

// P-rho-T equation constants for Ethane (equation P-2).
const double C2H6A[] = {
	1.716337E-03,	//B0	
	4.000173E+02,	//A0
	2.711853E+07,	//C0
	1.314084E+09,	//D0
	4.162885E+10,	//E0
	1.343479E-05,	//b
	1.385571,		//a	
	2.412543E+01,	//d	
	1.301434E+05,	//c	
	8.159501E-09,	//alpha
	1.293618E-05	//gamma
};

// P-rho-T equation constants for Ethylene (equation P-3).
const double C2H4A[] = {
	2.1E-05,			// gamma
	6.199009568E-01, 
   -5.480331377E+02, 
	5.939028162E+04, 
   -2.978456992E+07,
	1.410862498E+09, 
	2.099253547E-03,
   -3.023961281E-01,
	2.126582512E+02,
	4.064840279E-06, 
   -4.409410402E-03, 
	4.569939966E-10,
	6.188244460E-06,
	5.116189590E-10, 
	5.920592324E+04,
   -3.868585434E+07,
	6.383210497E+09,
   -3.452285470,
	2.067324621E+03,
   -2.179125512E+05
};
	
// P-rho-T equation constants for Hydrogen(para) (equation P-4).
const double H2A[33] = {
		  1.008854772E-03,		// gamma is H2A[0]
	1.150470519352900E+01,
	1.055427998826072E+03,
   -1.270685949968568E+04,
	7.287844527295619E+04,
   -7.448780703363973E+05,
	2.328994151810363E-01,
   -1.635308393739296E+01,
	3.730678064960389E+03,
	6.299667723184813E+05,
	1.210920358305697E-03,
	1.753651095884817,
   -1.367022988058101E+02,
   -6.869936641299885E-03,
	3.644494201750974E-02,
   -2.559784772600182,
   -4.038855202905836E-04,
	1.485396303520942E-06,
	4.243613981060742E-04,
   -2.307910113586888E-06,
   -6.082192173879582E+05,
   -1.961080967486886E+06,
   -5.786932854076408E+02,
	2.799129504191752E+04,
   -2.381566558300913E-01,
	8.918796032452872E-01,
   -6.985739539036644E-05,
   -7.339554179182899E-03,
   -5.597033440289980E-09,
	8.842130160884514E-08,
   -2.655507264539047E-12,
   -4.544474518140164E-12,
	9.818775257001922E-11
};

// P-rho-T equation constants for Methane (equation P-4).
const double CH4A[33] = {
	3.72992471469E-05,		// gamma is CH4A[0]
   -7.25929210183,
	4.13766054566E+02,
   -6.32167316855E+03,
	3.34015577724E+05,
   -1.68253379982E+07,
	1.87884851902E-02,
   -1.18673201223E+01,
	2.09062618015E+03,
   -4.07532656958E+05,
   -5.73917603241E-05,
	4.37711441593E-02,
   -4.38766500673,
	1.13524630779E-05,
   -5.07028240949E-05,
	2.28002199522E-02,  
	9.25611329590E-09,
	1.33865662546E-10,
   -1.65439044196E-07,
	1.81030980110E-10,
	5.45753645958E+05,
   -3.63192281933E+07,
	4.81463473761,
	1.56633022620E+05,
	7.89977010972E-05,
	1.39993881210E-02,
   -1.70656092212E-11,
   -4.55256623445E-05,
   -2.29314170748E-14,
	8.31548197665E-12,
	6.84673626259E-20,
   -4.70845544152E-17,
	5.21465091383E-16
};

// P-rho-T equation constants for Octane (equation P-2).
const double C8H18A[] = {
	2.661451E-03,	//B0
	1.682414E+02,	//A0
	6.334511E+07,	//C0
	2.791813E+09,	//D0
	6.796298E+09,	//E0
	3.163488E-06,	//b
	1.481799E-01,	//a
	1.162525E+02,	//d
	2.230527E+05,	//c
	5.634268E-09,	//alpha	
	6.568151E-06	//gamma
};

// P-rho-T equation constants for Oxygen (equation P-4).
const double O2A[] = {
	 5.46895508389297E-06,	// gamma
	-4.26396872798684E-01,
	 3.48334938784107E+01,	
	-5.77516910418738E+02,	
	 2.40961751553325E+04,	
	-1.23332307855543E+06,	
	 3.73585286319658E-04,	
	-1.70178244046465E-01,	
	-3.33226903068473E-04,	
	 8.61334799901291E+03,	
    -6.80394661057309E-07,
	 7.09583347162704E-04,	
    -5.73905688255053E-02,
	-1.92123080811409E-07,	
	 3.11764722329504E-08,	
	-8.09463854745591E-06,	
	-2.22562296356501E-11,	
	 9.18401045361994E-15,	
	 5.75758417511114E-12,	
	-2.10752269644774E-15,	
	 3.62884761272184E+03,	
	-1.23317754317110E+06,
	-5.03800414800672E-02,	
	 3.30686173177055E+02,
	-5.26259633964252E-08,
	 5.53075442383100E-06,
	-2.71042853363688E-13,
	-1.65732450675251E-09,
	-5.82711196409204E-20,
	 4.42953322148281E-17,
	-2.95529679136244E-25,
	-1.92361786708846E-23,
	 9.43758410350413E-23
};

// P-rho-T equation constants for Propane (equation P-2).
const double C3H8A[] = {
	1.366892E-03,	//B0
	2.579108E+02,	//A0
	3.401044E+07,	//C0
	1.076728E+09,	//D0
	3.375879E+10,	//E0
	1.096523E-05,	//b
	7.856721E-01,	//a	
	1.639769E+02,	//d
	1.661103E+05,	//c
	5.728034E-09,	//alpha
	9.157270E-06	//gamma
};

// P-rho-T equation constants for Refrigerant 12 (equation P-1).
// Equation P-1 is not implemented as of 2004
const double Rfrgrnt12A[] = {
	4.06366926E-04,
   -9.16210126E+0l,
	1.01049598E-0l,
   -5.74640225E-05,
	0.0,
	7.71136428E-02,
   -5.67539138E-05,
	0.0,
	4.08193371E-11,
   -1.52524293E+03,
	2.19982681,
	0.0,
   -1.66307226E-07,
	5.475
};

// P-rho-T equation constants for Refrigerant 22 (equation P-1).
// Equation P-1 is not implemented as of 2004
const double Rfrgrnt22A[] = {
	1.24855636E-04,
   -1.16981908E+02,
   -2.92952588E-02,
	2.41919261E-04,
   -2.43458381E-07,
	9.40022615E+11,
	1.16431240E-01,
	2.30319412E-04,
   -6.79667708E-07,
	6.30201766E-10,
   -2.07580650E+09,
   -1.18409710E+03,
	2.48896136,
	0.0,
   -1.20619716E-06,
	0.0,
	4.2,
	8781.3417
};

// Cv constant for Argon (equation C-0).
const double ArG[] = { 0.0, 312.192 };

// Cv constants for Nitrogen (equation C-4).
const double N2G[] = {
	 3353.40610,			// beta is N2G[0]
	-2.18203473713518E+05,
	 1.01573580096247E+04,
	-1.65504721657240E+02,
	 7.43175999190430E+02,
	-5.14605623546025E-03,
	 5.18347156760489E-06,
	-1.05922170493616E-09,
	 2.98389393363817E+02
};

// Cv constants for Carbon Dioxide (equation C-6).
const double CO2G[] = {
	 0,				// unused element.
	 8.726361E+03,
	 1.840040E+02,
	 1.914025E+00,
	-1.667825E-03,
	 7.305950E-07,
	-1.255290E-10
};

// Cv constants for Water (equation C-6).
const double H2OG[] = {
	 0,				// unused element.
	 4.600000E+04,
	 1.011249E+03,
	 8.389300E-01,
	-2.199890E-04,
	 2.466190E-07,
	-9.704700E-11
};


// Cv constants for Ammonia (equation C-2).
// G0 is the number of parameters
const double NH3G[] = {
	 9,				// number of parameters.
	 1.469259288E+03,
	 2.411085448E-01,
	-7.038236532E-03,
	 5.157906857E-05,
	-1.209815448E-07,
	 1.440829341E-10,
    -9.429402197E-14,
	 3.229595395E-17,
    -4.528318341E-21
};

// Cv constants for Butane (equation C-6).
const double C4H10G[] = {
	 0,				// unused element.
	 9.4817634E+03,
	 3.9633418E+02,
	 2.8576075,
	 4.8802965E-03,
	-6.4100047E-06,
	 2.0651236E-09
};

// Cv constants for Ethane (equation C-6).
const double C2H6G[] = {
	 0,				// unused element.
	 2.6209109E+04,
	 3.9731855E+02,
	 2.0372154,
	 6.3813897E-03,
	-7.2185581E-06,
	 2.2048025E-09
};

// Cv constants for Ethylene (equation C-2).
const double C2H4G[] = {
	 8,				// number of parameters.
	 1.0657565E+03,
	-2.5996503,
	 3.0370139E-03,
	 7.5775277E-05,
	-2.5613127E-07,
	 3.5699585E-10,
    -2.3618747E-13,
	 6.1095297E-17
};

// Cv constants for Hydrogen (para) (equation C-5).
const double H2G[] = {
	35.0,				// T1
	6.1934792E+03,
	2.9490437E+02,
   -1.5401979E+03,
   -4.9176101E+03,
	6.8957165E+04,
   -2.2282185E+05,
	3.7990059E+05,
   -3.7094216E+05,
    2.1326792E+05,
   -7.1519411E+04,
    1.2971743E+04,
   -9.8533014E+02,
    1.0434776E+04,
   -3.9144179E+02,
    5.8277696E+02,
    6.5409163E+02,
   -1.8728847E+02,
	400.0			// T2
};

// Cv constants for Methane (equation C-3).
const double CH4G[] = {
	 2009.152,				// beta
	 1.34740610E+03,
	 1.35512060E+02,
	-2.93910458E+01,
	 2.12774600,
	 2.44656600E+03
};

// Cv constants for Octane (equation C-6).
const double C8H18G[] = {
	 0,				// unused element.
	 4.0859678E+04,
	-3.2250398E+02,
	 6.6958265,
	-2.6759063E-03,
	 0.0,
	 0.0
};

// Cv constants for Oxygen (equation C-4).
const double O2G[] = {
	 2239.18105,			// beta
	-1.29442711174062E+06,
	 5.98231747005341E+04,
	-8.97850772730944E+02,
	 6.55236176900400E+02,
	-1.13131252131570E-02,
	 3.49810702442228E-06,
	 4.21065222886885E-09,
	 2.67997030050139E+02
};

// Cv constants for Propane (equation C-6).
const double C3H8G[] = {
	 0,				// unused element.
	 2.0582170E+05,
	-1.9109547E+03,
	 1.1622054E+01,
	-9.7951510E-03,
	 4.5167026E-06,
	-8.6345035E-10
};




// Basic EOS functions returning the pressure from molar volume
// and temperature.  
double P_Air( double V, double T ) 
	{ return PressureEOS_P4( V, T, AirA, MwAir ); }

double P_Argon( double V, double T ) 
	{ return PressureEOS_P3( V, T, ArA, MwAr ); }

double P_Nitrogen( double V, double T ) 
	{ return PressureEOS_P4( V, T, N2A, MwN2 ); }

double P_CarbDiox( double V, double T ) 
	{ return PressureEOS_P3( V, T, CO2A, MwCO2 ); }

double P_Water( double V, double T ) 
	{ return PressureEOS_P6( V, T, H2OA, rho_aH2O, TcH2O, MwH2O ); }


double P_Ammonia( double V, double T )

	{ return PressureEOS_P6Q1( V, T, NH3A, TcNH3, MwNH3 ); }

double P_Butane( double V, double T )

	{ return PressureEOS_P2( V, T, C4H10A, MwC4H10 ); }

double P_Ethane( double V, double T )

	{ return PressureEOS_P2( V, T, C2H6A, MwC2H6 ); }

double P_Ethylene( double V, double T )

	{ return PressureEOS_P3( V, T, C2H4A, MwC2H4 ); }

double P_Hydrogen( double V, double T )

	{ return PressureEOS_P4( V, T, H2A, MwH2 ); }

double P_Methane( double V, double T )

	{ return PressureEOS_P4( V, T, CH4A, MwCH4 ); }

double P_Octane( double V, double T )

	{ return PressureEOS_P2( V, T, C8H18A, MwC8H18 ); }

double P_Oxygen( double V, double T )

	{ return PressureEOS_P4( V, T, O2A, MwO2 ); }

double P_Propane( double V, double T )

	{ return PressureEOS_P2( V, T, C3H8A, MwC3H8 ); }



// EOS routines to get the molar volume from P and T by iteration
// for air, nitrogen, argon, carbon dioxide, and noniteratively 
// for steam (just a unit conversion).  This first routine is 
// a skeleton for the component functions.
double V_EOS( double P, double T, double (*PFunc)( double, double ), double InitGuess )
{
	// Initial guess for V using ideal gas law.
	double V = InitGuess;
	int iterations;
	
	iterations = Secant( P, &V, T, PFunc, MaxErr, MaxIter );		
	if (iterations < MaxIter) return V;
	else return -iterations;
}

double V_Air( double P, double T ) { return V_EOS( P, T, P_Air, Rgas*T/P ); }
double V_Argon( double P, double T ) { return V_EOS( P, T, P_Argon, Rgas*T/P ); }
double V_Nitrogen( double P, double T ) { return V_EOS( P, T, P_Nitrogen, Rgas*T/P ); }
double V_CarbDiox( double P, double T ) { return V_EOS( P, T, P_CarbDiox, Rgas*T/P ); }
double V_Water( double P, double T) { 
	double InitGuess;
	InitGuess = 1.0/PureSatLDensity(T, MwH2O, H2OD, TcH2O, rhocH2O,5);
	if (PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6) - 1E-1 <= P)
	return V_EOS( P, T, P_Water, InitGuess ); 
	else
	{if (T/P <= 1)
		InitGuess = 1.326*exp(1.946*T/P);
	else
	InitGuess = 10.731*T/P-3.7022;
	return V_EOS( P, T, P_Water, InitGuess );}; }

double V_Steam( double P, double T) { 
	double InitGuess;
	if (T/P <= 1)
		InitGuess = 1.326*exp(1.946*T/P);
	else
	InitGuess = 10.731*T/P-3.7022;
	if (PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6) + 1E-1>= P)
	return V_EOS( P, T, P_Water, InitGuess );
	else
	InitGuess = 1.0/PureSatLDensity(T, MwH2O, H2OD, TcH2O, rhocH2O,5);
	return V_EOS( P, T, P_Water, InitGuess ); }


double V_Ammonia( double P, double T ) 

			{ return V_EOS( P, T, P_Ammonia, Rgas*T/P ); }

double V_Butane( double P, double T )

			{ return V_EOS( P, T, P_Butane, Rgas*T/P ); }

double V_Ethane( double P, double T )

			{ return V_EOS( P, T, P_Ethane, Rgas*T/P ); }

double V_Ethylene( double P, double T )

			{ return V_EOS( P, T, P_Ethylene, Rgas*T/P ); }

double V_Hydrogen( double P, double T )

			{ return V_EOS( P, T, P_Hydrogen, Rgas*T/P ); }

double V_Methane( double P, double T )

			{ return V_EOS( P, T, P_Methane, Rgas*T/P ); }

double V_Octane( double P, double T )

			{ return V_EOS( P, T, P_Octane, Rgas*T/P ); }

double V_Oxygen( double P, double T )

			{ return V_EOS( P, T, P_Oxygen, Rgas*T/P ); }

double V_Propane( double P, double T )

			{ return V_EOS( P, T, P_Propane, Rgas*T/P ); }



// Pure component enthalpy and entropy routines.
double H_Argon( double P, double T )
{
	return PureEnthalpy( V_Argon(P,T), T, MwAr, ArA, ArG, HrefAr, 3, 0);
}

double H_Nitrogen( double P, double T )
{
	return PureEnthalpy( V_Nitrogen(P,T), T, MwN2, N2A, N2G, HrefN2, 4, 4);
}

double H_CarbDiox( double P, double T )
{
	return PureEnthalpy( V_CarbDiox(P,T), T, MwCO2, CO2A, CO2G, HrefCO2, 3, 6);
}

double H_Water( double P, double T )
{
	if (PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6) - 1E-1 <= P)
	return PureEnthalpyP6( V_Water(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, HrefH2Ol, TcH2O, 6);
	else 
	return PureEnthalpyP6( V_Steam(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, HrefH2Og, TcH2O, 6);
}

double  H_Steam( double P, double T )
{
	if (PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6) + 1E-1>= P)
	return PureEnthalpyP6( V_Steam(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, HrefH2Og, TcH2O, 6);
	else 
	return PureEnthalpyP6( V_Water(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, HrefH2Ol, TcH2O, 6);
}


double H_Ammonia( double P, double T )
{
	return PureEnthalpyP6Q1( V_Ammonia(P,T), T, MwNH3, NH3A, 
						     NH3G, HrefNH3, TcNH3, 2 );
}

double H_Butane( double P, double T )
{
	return PureEnthalpy( V_Butane(P,T), T, MwC4H10, C4H10A, 
		                 C4H10G, HrefC4H10, 2, 6);
}

double H_Ethane( double P, double T )
{
	return PureEnthalpy( V_Ethane(P,T), T, MwC2H6, C2H6A, 
		                 C2H6G, HrefC2H6, 2, 6);
}

double H_Ethylene( double P, double T )
{
	return PureEnthalpy( V_Ethylene(P,T), T, MwC2H4, C2H4A, 
		                 C2H4G, HrefC2H4, 3, 2);
}

double H_Hydrogen( double P, double T )
{
	return PureEnthalpy( V_Hydrogen(P,T), T, MwH2, H2A, 
		                 H2G, HrefH2, 4, 5);
}

double H_Methane( double P, double T )
{
	return PureEnthalpy( V_Methane(P,T), T, MwCH4, CH4A, 
		                 CH4G, HrefCH4, 4, 3);
}

double H_Octane( double P, double T )
{
	return PureEnthalpy( V_Octane(P,T), T, MwC8H18, C8H18A, 
		                 C8H18G, HrefC8H18, 2, 6);
}

double H_Oxygen( double P, double T )
{
	return PureEnthalpy( V_Oxygen(P,T), T, MwO2, O2A, 
		                 O2G, HrefO2, 4, 4);
}

double H_Propane( double P, double T )
{
	return PureEnthalpy( V_Propane(P,T), T, MwC3H8, C3H8A, 
		                 C3H8G, HrefC3H8, 2, 6);
}

//

double S_Argon( double P, double T )
{
	return PureEntropy( V_Argon(P,T), T, MwAr, ArA, ArG, SrefAr, 3, 0);
}

double S_Nitrogen( double P, double T )
{
	return PureEntropy( V_Nitrogen(P,T), T, MwN2, N2A, N2G, SrefN2, 4, 4);
}

double S_CarbDiox( double P, double T )
{
	return PureEntropy( V_CarbDiox(P,T), T, MwCO2, CO2A, CO2G, SrefCO2, 3, 6);
}

double S_Water( double P, double T )
{
	if (PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6) - 1E-1 <= P)
	return PureEntropyP6( V_Water(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, SrefH2Ol, TcH2O, 6);
	else
	return PureEntropyP6( V_Steam(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, SrefH2Og, TcH2O, 6);
}

double S_Steam( double P, double T )
{
	if (PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6) + 1E-1 >= P)
	return PureEntropyP6( V_Steam(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, SrefH2Og, TcH2O, 6);
	else 
	return PureEntropyP6( V_Water(P,T), T, MwH2O, H2OA, 
						   rho_aH2O, H2OG, SrefH2Ol, TcH2O, 6);
}



double S_Ammonia( double P, double T )
{
	return PureEntropyP6Q1( V_Ammonia(P,T), T, MwNH3, NH3A, 
						     NH3G, SrefNH3, TcNH3, 2 );
}

double S_Butane( double P, double T )
{
	return PureEntropy( V_Butane(P,T), T, MwC4H10, C4H10A, 
		                 C4H10G, SrefC4H10, 2, 6);
}

double S_Ethane( double P, double T )
{
	return PureEntropy( V_Ethane(P,T), T, MwC2H6, C2H6A, 
		                 C2H6G, SrefC2H6, 2, 6);
}

double S_Ethylene( double P, double T )
{
	return PureEntropy( V_Ethylene(P,T), T, MwC2H4, C2H4A, 
		                 C2H4G, SrefC2H4, 3, 2);
}

double S_Hydrogen( double P, double T )
{
	return PureEntropy( V_Hydrogen(P,T), T, MwH2, H2A, 
		                 H2G, SrefH2, 4, 5);
}

double S_Methane( double P, double T )
{
	return PureEntropy( V_Methane(P,T), T, MwCH4, CH4A, 
		                 CH4G, SrefCH4, 4, 3);
}

double S_Octane( double P, double T )
{
	return PureEntropy( V_Octane(P,T), T, MwC8H18, C8H18A, 
		                 C8H18G, SrefC8H18, 2, 6);
}

double S_Oxygen( double P, double T )
{
	return PureEntropy( V_Oxygen(P,T), T, MwO2, O2A, 
		                 O2G, SrefO2, 4, 4);
}

double S_Propane( double P, double T )
{
	return PureEntropy( V_Propane(P,T), T, MwC3H8, C3H8A, 
		                 C3H8G, SrefC3H8, 2, 6);
}


//


// Air entropy and enthalpy
double H_Air( double P, double T )
{
	double M[] = { MwAir, MwAr, MwN2, MwO2 },
		   y[] = { 0, yAr, yN2, yO2 };

	return MixedEnthalpy( V_Air(P,T), T, M, AirA, y, ArG, N2G, O2G, HrefAir );
}

double S_Air( double P, double T )
{
	double M[] = { MwAir, MwAr, MwN2, MwO2 },
		   y[] = { 0, yAr, yN2, yO2 };

	return MixedEntropy( V_Air(P,T), T, M, AirA, y, ArG, N2G, O2G, SrefAir );
}


//Functions to calculate T given H or S and P
double T_XP( double X, double P, double (*XFunc)( double, double ), double InitGuess )
{
	
	double T = InitGuess;
	double ThisT = T * 1.01, OldT;
	double ThisX = (*XFunc)( P, ThisT ) - X, OldX;
	double delT, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldT = ThisT; ThisT = T; OldX= ThisX;
		delT = OldT - ThisT;
		ThisX = (*XFunc)( P, ThisT ) - X;
		T = ThisT - ThisX*delT/(OldX - ThisX);
		AproxErr = (ThisT - T)/(T);
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	if (iterations < MaxIter) return T;
	else return -iterations;

}

double T_HP_Air( double H, double P ) { 
	double InitGuess;
	InitGuess = 560+ H/7.43;
	return T_XP( H, P, H_Air, InitGuess); }

double T_HP_Nitrogen( double H, double P ) { 
	double InitGuess;
	InitGuess = 556.1+ (H+HrefN2)/7.367;
	return T_XP( H, P, H_Nitrogen, InitGuess); }

double T_HP_Argon( double H, double P ) { 
	double InitGuess;
	InitGuess = 572.3+ (H+HrefAr)/5.24;
	return T_XP( H, P, H_Argon, InitGuess ); }

double T_HP_CarbDiox( double H, double P ) {
	double InitGuess;
	InitGuess = 610.8+ (H+HrefCO2)/11.64;
	return T_XP( H, P, H_CarbDiox, InitGuess ); }

double T_HP_Ammonia( double H, double P ){
	double InitGuess;
	InitGuess = (0.1157*P+569.84)+ (H+HrefNH3)/(0.0021*P+10.56);
	return T_XP( H, P, H_Ammonia, InitGuess ); }

double T_HP_Butane( double H, double P ){
	double InitGuess;
	InitGuess = (0.109*P+590.27)+ (H+HrefC4H10)/(0.0063*P+38.145);
	return T_XP( H, P, H_Butane, InitGuess ); }

double T_HP_Ethane( double H, double P ){
	double InitGuess;
	InitGuess = (0.0735*P+580.84)+ (H+HrefC2H6)/(0.0022*P+19.858);
	return T_XP( H, P, H_Ethane, InitGuess ); }

double T_HP_Ethylene( double H, double P ){
	double InitGuess;
	InitGuess = (0.0658*P+576.37)+ (H+HrefC2H4)/(0.0015*P+15.732);
	return T_XP( H, P, H_Ethylene, InitGuess ); }

double T_HP_Hydrogen( double H, double P ){
	double InitGuess;
	InitGuess = 533.72+ (H+HrefH2)/7.053;
	return T_XP( H, P, H_Hydrogen, InitGuess ); }

double T_HP_Methane( double H, double P ){
	double InitGuess;
	InitGuess = (0.039*P+552.67)+ (H+HrefCH4)/(0.0007*P+11.348);
	return T_XP( H, P, H_Methane, InitGuess ); }

double T_HP_Octane( double H, double P ){
	double InitGuess;
	InitGuess = (0.1271*P+638.9)+ (H+HrefC8H18)/(0.0136*P+79.874);
	return T_XP( H, P, H_Octane, InitGuess ); }

double T_HP_Oxygen( double H, double P ){
	double InitGuess;
	InitGuess = (0.0321*P+540.87)+ (H+HrefO2)/(0.0004*P+7.48);
	return T_XP( H, P, H_Oxygen, InitGuess ); }

double T_HP_Propane( double H, double P ){
	double InitGuess;
	InitGuess = (0.0888*P+579)+ (H+HrefC3H8)/(0.0037*P+28.564);
	return T_XP( H, P, H_Propane, InitGuess ); }

double T_HP_Water( double H, double P) { 
	double InitGuess;
	InitGuess = 1564 + (H+HrefH2Ol)/18.42;
	if (fabs(InitGuess - Tsat_Water(P)) <= 30.0)
		InitGuess = Tsat_Water(P);
	return T_XP( H, P, H_Water, InitGuess ); }

double T_HP_Steam( double H, double P) { 
	double InitGuess;
	InitGuess = (H+HrefH2Og)/(0.0034*P+8.536)+0.2202*P+569.9;
	if (fabs(InitGuess - Tsat_Water(P)) <= 30.0)
		InitGuess = Tsat_Water(P);
	return T_XP( H, P, H_Steam, InitGuess ); }


//

double T_SP_Air( double S, double P ) {
	double InitGuess;
	InitGuess = (300.99*log(P)-4437.7)+ S/0.0079;
	return T_XP( S, P, S_Air, InitGuess ); }

double T_SP_Nitrogen( double S, double P ){
	double InitGuess;
	InitGuess = (256.91*log(P)-174.21)+ (S+SrefN2)/0.0078;
	return T_XP( S, P, S_Nitrogen, InitGuess ); }

double T_SP_Argon( double S, double P ){
	double InitGuess;
	InitGuess = (352.81*log(P)-271.11)+ (S+SrefAr)/0.0055;
	return T_XP( S, P, S_Argon, InitGuess ); }

double T_SP_CarbDiox( double S, double P ) {
	double InitGuess;
	InitGuess = (180.15*log(P)+110.6)+ (S+SrefCO2)/0.0114;
	return T_XP( S, P, S_CarbDiox, InitGuess ); }

double T_SP_Ammonia( double S, double P ){
	double InitGuess;
	InitGuess = (184.51*log(P)-62.539)+ (S+SrefNH3)/0.0108;
	return T_XP( S, P, S_Ammonia, InitGuess ); }

double T_SP_Butane( double S, double P ){
	double InitGuess;
	InitGuess = (62.387*log(P)+395.58)+ (S+SrefC4H10)/0.0386;
	return T_XP( S, P, S_Butane, InitGuess ); }

double T_SP_Ethane( double S, double P ){
	double InitGuess;
	InitGuess = (106.57*log(P)+247.62)+ (S+SrefC2H6)/0.0201;
	return T_XP( S, P, S_Ethane, InitGuess ); }

double T_SP_Ethylene( double S, double P ){
	double InitGuess;
	InitGuess = (128.07*log(P)+186.66)+ (S+SrefC2H4)/0.0162;
	return T_XP( S, P, S_Ethylene, InitGuess ); }

double T_SP_Hydrogen( double S, double P ){
	double InitGuess;
	InitGuess = (263.57*log(P)-920.18)+ (S+SrefH2)/0.007;
	return T_XP( S, P, S_Hydrogen, InitGuess ); }

double T_SP_Methane( double S, double P ){
	double InitGuess;
	InitGuess = (165.83*log(P)-10.795)+ (S+SrefCH4)/0.0121;
	return T_XP( S, P, S_Methane, InitGuess ); }

double T_SP_Octane( double S, double P ){
	double InitGuess;
	InitGuess = (41.959*log(P)+442.28)+ (S+SrefC8H18)/0.0751;
	return T_XP( S, P, S_Octane, InitGuess ); }

double T_SP_Oxygen( double S, double P ){
	double InitGuess;
	InitGuess = (252.64*log(P)-137.61)+ (S+SrefO2)/0.0078;
	return T_XP( S, P, S_Oxygen, InitGuess ); }

double T_SP_Propane( double S, double P ){
	double InitGuess;
	InitGuess = (75.727*log(P)+353.48)+ (S+SrefC3H8)/0.0296;
	return T_XP( S, P, S_Propane, InitGuess ); }

double T_SP_Water( double S, double P) { 
	double InitGuess;
	InitGuess = (48.617*log(P)+1262.9)
	          + (S+SrefH2Ol)/(-0.0015*log(P)+0.0365);
	return T_XP( S, P, S_Water, InitGuess ); }

double T_SP_Steam( double S, double P) { 
	double InitGuess;
	InitGuess = exp((S+SrefH2Og)/(0.0036*P+8.46))*(214*log(P)-96.704);
	return T_XP( S, P, S_Steam, InitGuess ); }


//

//Functions to calculate P given H or S and T
double P_XT( double X, double T, double (*XFunc)( double, double ), double InitGuess )
{
	
	double P = InitGuess;
	double ThisP = P * 1.01, OldP;
	double ThisX = (*XFunc)( ThisP, T ) - X, OldX;
	double delP, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldP = ThisP; ThisP = P; OldX= ThisX;
		delP = OldP - ThisP;
		ThisX = (*XFunc)( ThisP, T ) - X;
		P = ThisP - ThisX*delP/(OldX - ThisX);
		AproxErr = (ThisP - P)/(P);
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	if (iterations < MaxIter) return P;
	else return -iterations;

}
/*
double P_HT_Air( double H, double T ) { 
	double InitGuess;
	InitGuess = (H-(7.7195*T-4240.2))/(0.221*log(T)-1.623);
	return P_XT( H, T, H_Air, InitGuess); }
*/
double P_HT_Air( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, 
		   B1, dB1, HR, Tr, TcAir, PcAir, 
		   omegaAir, HIdeal;
	TcAir = yN2*TcN2 + yAr*TcAr + yO2* TcO2;
    PcAir = yN2*PcN2 + yAr*PcAr + yO2* PcO2;
	omegaAir = yN2*omegaN2 + yAr*omegaAr + yO2*omegaO2;
	Tr=(T/(TcAir*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	
	HIdeal =	yAr*IdealPEnthalpy(T, MwAr, ArG, 0)
			  + yN2*IdealPEnthalpy(T, MwN2, N2G, 4)
			  + yO2*IdealPEnthalpy(T, MwO2, O2G, 4);

	HR=H - HIdeal +HrefAir;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcAir*1.8)*PcAir*145.038)/
		        (B0-Tr*dB0+omegaAir*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Air, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Air, SecGuess); }

double P_HT_Nitrogen( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcN2*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwN2, N2G, 4 ) + HrefN2;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcN2*1.8)*PcN2*145.038)/
		        (B0-Tr*dB0+omegaN2*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Nitrogen, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Nitrogen, SecGuess); }


double P_HT_Argon( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcAr*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwAr, ArG, 0 ) + HrefAr;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcAr*1.8)*PcAr*145.038)/
		        (B0-Tr*dB0+omegaAr*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Argon, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Argon, SecGuess); }

double P_HT_CarbDiox( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcCO2*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwCO2, CO2G, 6 ) + HrefCO2;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcCO2*1.8)*PcCO2*145.038)/
		        (B0-Tr*dB0+omegaCO2*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_CarbDiox, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_CarbDiox, SecGuess); }

double P_HT_Ammonia( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcNH3*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwNH3, NH3G, 2 ) + HrefNH3;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcNH3*1.8)*PcNH3*145.038)/
		        (B0-Tr*dB0+omegaNH3*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Ammonia, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Ammonia, SecGuess); }

double P_HT_Butane( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcC4H10*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwC4H10, C4H10G, 6 ) + HrefC4H10;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcC4H10*1.8)*PcC4H10*145.038)/
		        (B0-Tr*dB0+omegaC4H10*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Butane, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Butane, SecGuess); }

double P_HT_Ethane( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcC2H6*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwC2H6, C2H6G, 6 ) + HrefC2H6;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcC2H6*1.8)*PcC2H6*145.038)/
		        (B0-Tr*dB0+omegaC2H6*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Ethane, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Ethane, SecGuess); }

double P_HT_Ethylene( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcC2H4*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwC2H4, C2H4G, 2 ) + HrefC2H4;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcC2H4*1.8)*PcC2H4*145.038)/
		        (B0-Tr*dB0+omegaC2H4*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Ethylene, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Ethylene, SecGuess); }

double P_HT_Hydrogen( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcH2*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwH2, H2G, 5 ) + HrefH2;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcH2*1.8)*PcH2*145.038)/
		        (B0-Tr*dB0+omegaH2*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Hydrogen, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Hydrogen, SecGuess); }

double P_HT_Methane( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcCH4*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwCH4, CH4G, 3 ) + HrefCH4;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcCH4*1.8)*PcCH4*145.038)/
		        (B0-Tr*dB0+omegaCH4*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Methane, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Methane, SecGuess); }

double P_HT_Octane( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcC8H18*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwC8H18, C8H18G, 6 ) + HrefC8H18;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcC8H18*1.8)*PcC8H18*145.038)/
		        (B0-Tr*dB0+omegaC8H18*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Octane, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Octane, SecGuess); }

double P_HT_Oxygen( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcO2*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwO2, O2G, 4 ) + HrefO2;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcO2*1.8)*PcO2*145.038)/
		        (B0-Tr*dB0+omegaO2*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Oxygen, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Oxygen, SecGuess); }

double P_HT_Propane( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcC3H8*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwC3H8, C3H8G, 6 ) + HrefC3H8;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcC3H8*1.8)*PcC3H8*145.038)/
		        (B0-Tr*dB0+omegaC3H8*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;

	SecGuess = P_XT( H, T, H_Propane, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Propane, SecGuess); }
/*
double P_HT_Water( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcH2O*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwH2O, H2OG, 6 ) + HrefH2Ol;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcH2O*1.8)*PcH2O*145.038)/
		        (B0-Tr*dB0+omegaH2O*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;
	if (fabs(InitGuess - Psat_Water(T)) <= 10.0)
		InitGuess = Psat_Water(T);
	SecGuess = P_XT( H, T, H_Water, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Water, SecGuess); }
*/
double P_HT_Water( double H, double T ) { 
	double InitGuess;
	InitGuess=Psat_Water(T);
	return P_XT( H, T, H_Water, InitGuess); }

double P_HT_Steam( double H, double T ) { 
	double InitGuess, SecGuess, B0, dB0, B1, dB1, HR, Tr;
	Tr=(T/(TcH2O*1.8));
	B0=0.083-0.422/pow(Tr,1.6);
	dB0=0.675/pow(Tr,2.6);
	B1=0.139-0.172/pow(Tr,4.2);
	dB1=0.722/pow(Tr,5.2);
	HR=H - IdealPEnthalpy( T, MwH2O, H2OG, 6 ) + HrefH2Og;
	//See Smith, van Ness Eqn 6.58 for details
	InitGuess = (HR/(R*TcH2O*1.8)*PcH2O*145.038)/
		        (B0-Tr*dB0+omegaH2O*(B1-Tr*dB1));
	if (InitGuess < 0)
		InitGuess = -InitGuess;
	if (fabs(InitGuess - Psat_Water(T)) <= 10.0)
		InitGuess = Psat_Water(T);
	SecGuess = P_XT( H, T, H_Steam, InitGuess);
	//Perturbate the first solution
	SecGuess = SecGuess*1.1;
	return P_XT( H, T, H_Steam, SecGuess); }

// Calculate P when S and T is given

double P_ST_Air( double S, double T ) { 
	double InitGuess, B, C, SIdealx, VGuess;
	B=ConvM3kg*MwAir;
	C=RgasSI/MwAir*ConvJkg*MwAir/1.8;
	SIdealx =	yAr*IdealPEntropyx(T, MwAr, ArG, 0)
			  + yN2*IdealPEntropyx(T, MwN2, N2G, 4)
			  + yO2*IdealPEntropyx(T, MwO2, O2G, 4);
	VGuess = B * exp ((S-SIdealx+SrefAir)/C);
	InitGuess = P_Air(VGuess,T);
	return P_XT( S, T, S_Air, InitGuess); }

double P_ST_Nitrogen( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwN2;
	C=RgasSI/MwN2*ConvJkg*MwN2/1.8;
	VGuess = B * exp ((S-IdealPEntropyx(T, MwN2, N2G, 4)+SrefN2)/C);
	InitGuess = P_Nitrogen(VGuess,T);
	return P_XT( S, T, S_Nitrogen, InitGuess); }


double P_ST_Argon( double S, double T ) {
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwAr;
	C=RgasSI/MwAr*ConvJkg*MwAr/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwAr, ArG, 0 ) + SrefAr)/C);
	InitGuess = P_Argon(VGuess,T);
	return P_XT( S, T, S_Argon, InitGuess); }

double P_ST_CarbDiox( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwCO2;
	C=RgasSI/MwCO2*ConvJkg*MwCO2/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwCO2, CO2G, 6 ) + SrefCO2)/C);
	InitGuess = P_Argon(VGuess,T);
	return P_XT( S, T, S_CarbDiox, InitGuess); }

double P_ST_Ammonia( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwNH3;
	C=RgasSI/MwNH3*ConvJkg*MwNH3/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwNH3, NH3G, 2 ) + SrefNH3)/C);
	InitGuess = P_Ammonia(VGuess,T);
	return P_XT( S, T, S_Ammonia, InitGuess); }

double P_ST_Butane( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwC4H10;
	C=RgasSI/MwC4H10*ConvJkg*MwC4H10/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwC4H10, C4H10G, 6 )
		             + SrefC4H10)/C);
	InitGuess = P_Butane(VGuess,T);
	return P_XT( S, T, S_Butane, InitGuess); }

double P_ST_Ethane( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwC2H6;
	C=RgasSI/MwC2H6*ConvJkg*MwC2H6/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwC2H6, C2H6G, 6 ) 
		             + SrefC2H6)/C);
	InitGuess = P_Ethane(VGuess,T);
	return P_XT( S, T, S_Ethane, InitGuess); }

double P_ST_Ethylene( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwC2H4;
	C=RgasSI/MwC2H4*ConvJkg*MwC2H4/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwC2H4, C2H4G, 2 ) 
		             + SrefC2H4)/C);
	InitGuess = P_Ethylene(VGuess,T);
	return P_XT( S, T, S_Ethylene, InitGuess); }

double P_ST_Hydrogen( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwH2;
	C=RgasSI/MwH2*ConvJkg*MwH2/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwH2, H2G, 5 ) 
		             + SrefH2)/C);
	InitGuess = P_Hydrogen(VGuess,T);
	return P_XT( S, T, S_Hydrogen, InitGuess); }

double P_ST_Methane( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwCH4;
	C=RgasSI/MwCH4*ConvJkg*MwCH4/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwCH4, CH4G, 3 ) + SrefCH4)/C);
	InitGuess = P_Methane(VGuess,T);
	return P_XT( S, T, S_Methane, InitGuess); }

double P_ST_Octane( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwC8H18;
	C=RgasSI/MwC8H18*ConvJkg*MwC8H18/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwC8H18, C8H18G, 6 ) 
					 + SrefC8H18)/C);
	InitGuess = P_Octane(VGuess,T);
	return P_XT( S, T, S_Octane, InitGuess); }

double P_ST_Oxygen( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwO2;
	C=RgasSI/MwO2*ConvJkg*MwO2/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwO2, O2G, 4 ) + SrefO2)/C);
	InitGuess = P_Oxygen(VGuess,T);
	return P_XT( S, T, S_Oxygen, InitGuess); }

double P_ST_Propane( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwC3H8;
	C=RgasSI/MwC3H8*ConvJkg*MwC3H8/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwC3H8, C3H8G, 6 ) 
					 + SrefC3H8)/C);
	InitGuess = P_Propane(VGuess,T);
	return P_XT( S, T, S_Propane, InitGuess); }

double P_ST_Water( double S, double T ) { 
	double InitGuess;
	InitGuess = Psat_Water(T);
	return P_XT( S, T, S_Water, InitGuess); }

double P_ST_Steam( double S, double T ) { 
	double InitGuess, B, C, VGuess;
	B=ConvM3kg*MwH2O;
	C=RgasSI/MwH2O*ConvJkg*MwH2O/1.8;
	VGuess = B * exp ((S-IdealPEntropyx( T, MwH2O, H2OG, 6 ) + SrefH2Og)/C);
	InitGuess = P_Water(VGuess,T);
	if (fabs(InitGuess - Psat_Water(T)) <= 10.0)
		InitGuess = Psat_Water(T);
	return P_XT( S, T, S_Steam, InitGuess); }


//

//Saturation Properties for Water
double Psat_Water( double T )
{ return PureSatPressure(T, MwH2O, H2OF, TcH2O, PcH2O, 6);};

double Tsat_Water( double P )
{
	double A=18.3036, B=3816.44, C=-46.13;
	double T = 1.8*(B/(A-log(P*ConvmmHg))-C);
	double ThisT = T * 1.01, OldT;
	double ThisP = Psat_Water( ThisT ) - P, OldP;
	double delT, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldT = ThisT; ThisT = T; OldP= ThisP;
		delT = OldT - ThisT;
		ThisP = Psat_Water( ThisT ) - P;
		T = ThisT - ThisP*delT/(OldP - ThisP);
		AproxErr = (ThisT - T)/(T);
		iterations++;
	} while( fabs(AproxErr) > MaxErr && iterations < MaxIter );
	if (iterations < MaxIter) return T;
	else return -iterations;
};

double rhosat_Water( double T ) {
	return SLDensity_D5( T, H2OD, TcH2O, rhocH2O, MwH2O );};
