// Reynolds_Skeletons.cpp

// Implementation of thermo functions described in "Thermodynamic
// Properties in SI," W.C. Reynolds, 1979, Stanford University.
// These functions were converted to English units, with quantities
// in lb-moles instead of kilograms.  The reference state is the 
// ideal gas in the standard state (1atm) at 298.15K (77F).

// Corrections and new implementations by DBO early 2002
// Corrections and new implementations by CK 2004 
#include <math.h>
#include "Combustion_Library.h"


// Gas constants
#define RgasSI		8.31441E3	// m3-Pa/kgmol-K
#define Rcalmol		1.987		// cal/mole-K
#define Rgas		10.73150	// ft3-psia/lbmol-R
#define R			1.98586		// Btu/lbmol-R

// Reference temperature and volume (ideal gas)
#define T0			298.15				// K
#define V0			391.895357897		// ft3/lbmole
#define P0			 14.5038			// psia


// Conversions (multiply by to get English units).
#define ConvPa			1.45037737730209E-04	// Pa to psia
#define ConvM3kg		1.60184633739601E+01	// m3/kg to ft3/lbm
#define ConvJkg			4.29922547582305E-04	// J/kg to Btu/lbm
#define Convcalmole		1.79882					// cal/mole to Btu/lbm


// Iteration constraints.
#define MaxErr		1.0E-6
#define MaxIter		100


// P-V-T equation P-4.
double PressureEOS_P4( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	return (
		rho*RgasSI/Mw*T + rho*rho*(A[1]*T + A[2]*sqrt(T) + A[3] + A[4]/T 
		+ A[5]/Tsq) + pow(rho,3)*(A[6]*T + A[7] + A[8]/T + A[9]/Tsq) 
		+ pow(rho,4)*(A[10]*T + A[11] + A[12]/T) + pow(rho,5)*A[13] 
		+ pow(rho,6)*(A[14]/T + A[15]/Tsq) + pow(rho,7)*A[16]/T 
		+ pow(rho,8)*(A[17]/T + A[18]/Tsq) + pow(rho,9)*A[19]/Tsq 
		+ (pow(rho,3)*(A[20]/Tsq + A[21]/Tcu) + pow(rho,5)*(A[22]/Tsq 
		+ A[23]/Tfr) + pow(rho,7)*(A[24]/Tsq + A[25]/Tcu) 
		+ pow(rho,9)*(A[26]/Tsq + A[27]/Tfr) + pow(rho,11)*(A[28]/Tsq 
		+ A[29]/Tcu) + pow(rho,13)*(A[30]/Tsq + A[31]/Tcu 
		+ A[32]/Tfr)) * exp(-A[0]*rho*rho)
		) * ConvPa;
}


// P-V-T equation P-3.
double PressureEOS_P3( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);
	
	return (
		rho*RgasSI/Mw*T + rho*rho*(A[1]*T + A[2] + A[3]/T + A[4]/Tsq
		+ A[5]/Tcu) + pow(rho,3)*(A[6]*T + A[7] + A[8]/T) 
		+ pow(rho,4)*(A[9]*T + A[10]) + pow(rho,5)*(A[11]*T + A[12])
		+ pow(rho,6)*A[13] + (pow(rho,3)*(A[14]/Tsq + A[15]/Tcu 
		+ A[16]/Tfr) + pow(rho,5)*(A[17]/Tsq + A[18]/Tcu 
		+ A[19]/Tfr)) * exp(-A[0]*rho*rho)
		) * ConvPa;
}

// P-V-T equation P-2.
//B0=A[0]; A0=A[1];C0=A[2];D0=A[3];E0=A[4];
//b=A[5];a=A[6];d=A[7];c=A[8];alpha=A[9];gamma=A[10];
double PressureEOS_P2( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);
	double B0=A[0], A0=A[1], C0=A[2], D0=A[3], E0=A[4],
	b=A[5],	a=A[6], d=A[7],
	c=A[8], alpha=A[9], gamma=A[10];
	
	return (
		rho*RgasSI/Mw*T
		+(B0*RgasSI/Mw*T-A0-C0/(Tsq)
		+D0/(Tcu)-E0/(Tfr))*pow(rho,2)+(b*RgasSI/Mw*T-a-d/T)*pow(rho,3)
		+alpha*(a+d/T)*pow(rho,6)
		+c*pow(rho,3)*(1+gamma*pow(rho,2))*exp(-gamma*pow(rho,2))/(Tsq)
		) * ConvPa;
}

// equation Q-1 used with P-6.
double Fnc_Q1( double rho, double T, double Tcrit, 
			  const double AA[][7])
{
	int i,j;
	double tau, tau_c, Q=0.0;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin

	tau = AA[0][0]/T; tau_c = AA[0][1];
	for (i=1;i<=9;++i)
		for (j=1;j<=6;++j)
			Q = Q + AA[i][j]*pow(rho,i-1)*pow((tau-tau_c),j-1);
	
	return Q;


}

// Derivative of equation Q-1 wrt. rho used with P-6.
// (partial Q1/ partial rho) at constant T
double Fnc_delQ1delrho( double rho, double T, double Tcrit, 
			  const double AA[][7])
{
	int i,j;
	double tau, tau_c, dQdr=0.0;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin

	tau = AA[0][0]/T; tau_c = AA[0][1];
	for (i=2;i<=9;++i)
		for (j=1;j<=6;++j)
			dQdr = dQdr + (i-1)*AA[i][j]*pow(rho,i-2)*pow((tau-tau_c),j-1);
	
	return dQdr;


}


// Integral Part I (Enthaply and Entropy Calculations)
//(using equation Q-1 used with P-6.)
// integral from 0 to rho of (partial Q1/partial T) at constant rho d (rho)
double Fnc_intdQ1dT( double rho, double T, double Tcrit, 
			  const double AA[][7])
{
	int i,j;
	double tau, tau_c, dQdT=0.0;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin

	tau = AA[0][0]/T; tau_c = AA[0][1];
	for (i=1;i<=9;++i)
		for (j=2;j<=6;++j)
			dQdT = dQdT - (AA[i][j]*pow(rho,i)/static_cast<double>(i))*tau/T
						 *(pow((tau-tau_c),j-2)*(static_cast<double>(j)-1.0));
	
	return dQdT;


}


// Integral Part II (Enthaply and Entropy Calculations)             
//(using Derivative of equation Q-1 wrt. rho used with P-6.)
// integral from 0 to rho of rho *(partial^2 Q1/(partial rho)(partial T)) d (rho)


double Fnc_intdQ1drhodT( double rho, double T, double Tcrit, 
			  const double AA[][7])
{
	int i,j;
	double tau, tau_c, dQdrdT=0.0;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin

	tau = AA[0][0]/T; tau_c = AA[0][1];
	for (i=2;i<=9;++i)
		for (j=2;j<=6;++j)
			dQdrdT = dQdrdT - 
			(AA[i][j]*(i-1)*pow(rho,i)/static_cast<double>(i))*tau/T
			*(pow((tau-tau_c),j-2)*(static_cast<double>(j)-1.0));
	
	return dQdrdT;
}

// Integral Part III (Entropy Calculations)             
//(using equation Q-1 wrt.)
// integral from 0 to rho of Q1 d (rho)

double Fnc_intQ1( double rho, double T, double Tcrit, 
			  const double AA[][7])
{
	int i,j;
	double tau, tau_c, intQ=0.0;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin

	tau = AA[0][0]/T; tau_c = AA[0][1];
	for (i=1;i<=9;++i)
		for (j=1;j<=6;++j)
			intQ = intQ + AA[i][j]*pow(rho,i)/static_cast<double>(i)
			                     *pow((tau-tau_c),j-1);
	
	return intQ;

};

// Integral Part IV (Entropy Calculations)             
//(using Derivative of equation Q-1 wrt. rho used with P-6.)
// integral from 0 to rho of rho * (partial Q1/ partial rho) at constant T d (rho)
double Fnc_intdQ1drho( double rho, double T, double Tcrit, 
			  const double AA[][7])
{

	int i,j;
	double tau, tau_c, intdQdr=0.0;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin

	tau = AA[0][0]/T; tau_c = AA[0][1];
	for (i=2;i<=9;++i)
		for (j=1;j<=6;++j)
//			intdQdr = intdQdr + AA[i][j]*pow(rho,i-1)
//						       *pow((tau-tau_c),j-1);
			intdQdr = intdQdr + 
			AA[i][j]*(i-1)*pow(rho,i)/static_cast<double>(i)
			*pow((tau-tau_c),j-1);	
	return intdQdr;

}


// equation Q-2 used with P-6.
double Fnc_Q( double rho, double T, double Tcrit, 
			  const double AA[][8], const double rho_a[])
{
	int i;
	double tau, tau_c, tau_a[8], Tsq, Tcu , Tfr;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	tau = AA[0][0]/T; tau_c = AA[0][0]/Tcrit;
	for (i=2;i<=7;++i) tau_a[i] = 2.5;tau_a[1]= tau_c;
	
	return (
		(tau-tau_c)*((AA[1][1]
		             +AA[2][1]*(rho-rho_a[1])
					 +AA[3][1]*pow((rho-rho_a[1]),2)
					 +AA[4][1]*pow((rho-rho_a[1]),3)
					 +AA[5][1]*pow((rho-rho_a[1]),4)
					 +AA[6][1]*pow((rho-rho_a[1]),5)
					 +AA[7][1]*pow((rho-rho_a[1]),6)
					 +AA[8][1]*pow((rho-rho_a[1]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][1]+AA[10][1]*rho))/(tau-tau_a[1])
		
					 +AA[1][2]
					 +AA[2][2]*(rho-rho_a[2])
					 +AA[3][2]*pow((rho-rho_a[2]),2)
					 +AA[4][2]*pow((rho-rho_a[2]),3)
					 +AA[5][2]*pow((rho-rho_a[2]),4)
					 +AA[6][2]*pow((rho-rho_a[2]),5)
					 +AA[7][2]*pow((rho-rho_a[2]),6)
					 +AA[8][2]*pow((rho-rho_a[2]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][2]+AA[10][2]*rho)

	+(tau-tau_a[3])*(AA[1][3]
					 +AA[2][3]*(rho-rho_a[3])
					 +AA[3][3]*pow((rho-rho_a[3]),2)
					 +AA[4][3]*pow((rho-rho_a[3]),3)
					 +AA[5][3]*pow((rho-rho_a[3]),4)
					 +AA[6][3]*pow((rho-rho_a[3]),5)
					 +AA[7][3]*pow((rho-rho_a[3]),6)
					 +AA[8][3]*pow((rho-rho_a[3]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][3]+AA[10][3]*rho))

	+pow((tau-tau_a[4]),2)*(AA[1][4]
					 +AA[2][4]*(rho-rho_a[4])
					 +AA[3][4]*pow((rho-rho_a[4]),2)
					 +AA[4][4]*pow((rho-rho_a[4]),3)
					 +AA[5][4]*pow((rho-rho_a[4]),4)
					 +AA[6][4]*pow((rho-rho_a[4]),5)
					 +AA[7][4]*pow((rho-rho_a[4]),6)
					 +AA[8][4]*pow((rho-rho_a[4]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][4]+AA[10][4]*rho))
					 
	+pow((tau-tau_a[5]),3)*(AA[1][5]
					 +AA[2][5]*(rho-rho_a[5])
					 +AA[3][5]*pow((rho-rho_a[5]),2)
					 +AA[4][5]*pow((rho-rho_a[5]),3)
					 +AA[5][5]*pow((rho-rho_a[5]),4)
					 +AA[6][5]*pow((rho-rho_a[5]),5)
					 +AA[7][5]*pow((rho-rho_a[5]),6)
					 +AA[8][5]*pow((rho-rho_a[5]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][5]+AA[10][5]*rho))
					 
	+pow((tau-tau_a[6]),4)*(AA[1][6]
					 +AA[2][6]*(rho-rho_a[6])
					 +AA[3][6]*pow((rho-rho_a[6]),2)
					 +AA[4][6]*pow((rho-rho_a[6]),3)
					 +AA[5][6]*pow((rho-rho_a[6]),4)
					 +AA[6][6]*pow((rho-rho_a[6]),5)
					 +AA[7][6]*pow((rho-rho_a[6]),6)
					 +AA[8][6]*pow((rho-rho_a[6]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][6]+AA[10][6]*rho))
					 
	+pow((tau-tau_a[7]),5)*(AA[1][7]
					 +AA[2][7]*(rho-rho_a[7])
					 +AA[3][7]*pow((rho-rho_a[7]),2)
					 +AA[4][7]*pow((rho-rho_a[7]),3)
					 +AA[5][7]*pow((rho-rho_a[7]),4)
					 +AA[6][7]*pow((rho-rho_a[7]),5)
					 +AA[7][7]*pow((rho-rho_a[7]),6)
					 +AA[8][7]*pow((rho-rho_a[7]),7)
					 +exp(-AA[0][1]*rho)*(AA[9][7]+AA[10][7]*rho)))
    );


}

// Derivative of equation Q-2 wrt. rho used with P-6.
// (partial Q2 / partial rho) at constant T
double Fnc_delQdelrho( double rho, double T, double Tcrit, 
			  const double AA[][8], const double rho_a[])
{

	int i;
	double tau, tau_c, tau_a[8], Tsq, Tcu , Tfr;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	tau = AA[0][0]/T; tau_c = AA[0][0]/Tcrit;
	for (i=2;i<=7;++i) tau_a[i] = 2.5;tau_a[1]= tau_c;

	return (
		(tau-tau_c)*((
		             +AA[2][1]
					 +AA[3][1]*2*(rho-rho_a[1])
					 +AA[4][1]*3*pow((rho-rho_a[1]),2)
					 +AA[5][1]*4*pow((rho-rho_a[1]),3)
					 +AA[6][1]*5*pow((rho-rho_a[1]),4)
					 +AA[7][1]*6*pow((rho-rho_a[1]),5)
					 +AA[8][1]*7*pow((rho-rho_a[1]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][1]+AA[10][1]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][1]))/(tau-tau_a[1])
							 
					 +AA[2][2]
					 +AA[3][2]*2*(rho-rho_a[2])
					 +AA[4][2]*3*pow((rho-rho_a[2]),2)
					 +AA[5][2]*4*pow((rho-rho_a[2]),3)
					 +AA[6][2]*5*pow((rho-rho_a[2]),4)
					 +AA[7][2]*6*pow((rho-rho_a[2]),5)
					 +AA[8][2]*7*pow((rho-rho_a[2]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][2]+AA[10][2]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][2])

	+(tau-tau_a[3])*(
					 +AA[2][3]
					 +AA[3][3]*2*(rho-rho_a[3])
					 +AA[4][3]*3*pow((rho-rho_a[3]),2)
					 +AA[5][3]*4*pow((rho-rho_a[3]),3)
					 +AA[6][3]*5*pow((rho-rho_a[3]),4)
					 +AA[7][3]*6*pow((rho-rho_a[3]),5)
					 +AA[8][3]*7*pow((rho-rho_a[3]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][3]+AA[10][3]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][3]))

	+pow((tau-tau_a[4]),2)*(
					 +AA[2][4]
					 +AA[3][4]*2*(rho-rho_a[4])
					 +AA[4][4]*3*pow((rho-rho_a[4]),2)
					 +AA[5][4]*4*pow((rho-rho_a[4]),3)
					 +AA[6][4]*5*pow((rho-rho_a[4]),4)
					 +AA[7][4]*6*pow((rho-rho_a[4]),5)
					 +AA[8][4]*7*pow((rho-rho_a[4]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][4]+AA[10][4]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][4]))
					 
	+pow((tau-tau_a[5]),3)*(
					 +AA[2][5]
					 +AA[3][5]*2*(rho-rho_a[5])
					 +AA[4][5]*3*pow((rho-rho_a[5]),2)
					 +AA[5][5]*4*pow((rho-rho_a[5]),3)
					 +AA[6][5]*5*pow((rho-rho_a[5]),4)
					 +AA[7][5]*6*pow((rho-rho_a[5]),5)
					 +AA[8][5]*7*pow((rho-rho_a[5]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][5]+AA[10][5]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][5]))
					 
	+pow((tau-tau_a[6]),4)*(
					 +AA[2][6]
					 +AA[3][6]*2*(rho-rho_a[6])
					 +AA[4][6]*3*pow((rho-rho_a[6]),2)
					 +AA[5][6]*4*pow((rho-rho_a[6]),3)
					 +AA[6][6]*5*pow((rho-rho_a[6]),4)
					 +AA[7][6]*6*pow((rho-rho_a[6]),5)
					 +AA[8][6]*7*pow((rho-rho_a[6]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][6]+AA[10][6]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][6]))
					 
	+pow((tau-tau_a[7]),5)*(
					 +AA[2][7]
					 +AA[3][7]*2*(rho-rho_a[7])
					 +AA[4][7]*3*pow((rho-rho_a[7]),2)
					 +AA[5][7]*4*pow((rho-rho_a[7]),3)
					 +AA[6][7]*5*pow((rho-rho_a[7]),4)
					 +AA[7][7]*6*pow((rho-rho_a[7]),5)
					 +AA[8][7]*7*pow((rho-rho_a[7]),6)
					 +(-AA[0][1]*exp(-AA[0][1]*rho)*(AA[9][7]+AA[10][7]*rho)
					 +exp(-AA[0][1]*rho)*AA[10][7])))
    );

  
}

// Integral Part I (Enthaply and Entropy Calculations)
//(using equation Q-2 used with P-6.)
// integral from 0 to rho of (partial Q2/partial T) at constant rho d (rho)

double Fnc_intdQdT( double rho, double T, double Tcrit, 
			  const double AA[][8], const double rho_a[])
{
	int i;
	double tau, tau_c, tau_a[8], Tsq, Tcu , Tfr;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	tau = AA[0][0]/T; tau_c = AA[0][0]/Tcrit;
	for (i=2;i<=7;++i) tau_a[i] = 2.5;tau_a[1]= tau_c;
	
	return (
		(-AA[0][0]/(Tsq*(AA[0][0]/T-tau_a[1]))
		+(AA[0][0]/T-tau_c)*AA[0][0]/(pow((AA[0][0]/T-tau_a[1]),2)*Tsq))
					*(AA[1][1]*rho
		             +AA[2][1]/2.0*(pow((rho-rho_a[1]),2)
									-pow((-rho_a[1]),2))
					 +AA[3][1]/3.0*(pow((rho-rho_a[1]),3)
									-pow((-rho_a[1]),3))
					 +AA[4][1]/4.0*(pow((rho-rho_a[1]),4)
									-pow((-rho_a[1]),4))
					 +AA[5][1]/5.0*(pow((rho-rho_a[1]),5)
									-pow((-rho_a[1]),5))
					 +AA[6][1]/6.0*(pow((rho-rho_a[1]),6)
									-pow((-rho_a[1]),6))
					 +AA[7][1]/7.0*(pow((rho-rho_a[1]),7)
									-pow((-rho_a[1]),7))
					 +AA[8][1]/8.0*(pow((rho-rho_a[1]),8)
									-pow((-rho_a[1]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][1]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][1]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][1]
					   -AA[9][1]*AA[0][1]-AA[10][1])/(pow(AA[0][1],2)))
		
		-AA[0][0]/(Tsq)
				   *(AA[1][2]*rho
		             +AA[2][2]/2.0*(pow((rho-rho_a[2]),2)
									-pow((-rho_a[2]),2))
					 +AA[3][2]/3.0*(pow((rho-rho_a[2]),3)
									-pow((-rho_a[2]),3))
					 +AA[4][2]/4.0*(pow((rho-rho_a[2]),4)
									-pow((-rho_a[2]),4))
					 +AA[5][2]/5.0*(pow((rho-rho_a[2]),5)
									-pow((-rho_a[2]),5))
					 +AA[6][2]/6.0*(pow((rho-rho_a[2]),6)
									-pow((-rho_a[2]),6))
					 +AA[7][2]/7.0*(pow((rho-rho_a[2]),7)
									-pow((-rho_a[2]),7))
					 +AA[8][2]/8.0*(pow((rho-rho_a[2]),8)
									-pow((-rho_a[2]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][2]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][2]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][2]
					   -AA[9][2]*AA[0][1]-AA[10][2])/(pow(AA[0][1],2)))


	 +(-AA[0][0]*(AA[0][0]/T-tau_a[3])/(Tsq)
	 -(AA[0][0]/T-tau_c)*AA[0][0]/(Tsq))
					*(AA[1][3]*rho
		             +AA[2][3]/2.0*(pow((rho-rho_a[3]),2)
									-pow((-rho_a[3]),2))
					 +AA[3][3]/3.0*(pow((rho-rho_a[3]),3)
									-pow((-rho_a[3]),3))
					 +AA[4][3]/4.0*(pow((rho-rho_a[3]),4)
									-pow((-rho_a[3]),4))
					 +AA[5][3]/5.0*(pow((rho-rho_a[3]),5)
									-pow((-rho_a[3]),5))
					 +AA[6][3]/6.0*(pow((rho-rho_a[3]),6)
									-pow((-rho_a[3]),6))
					 +AA[7][3]/7.0*(pow((rho-rho_a[3]),7)
									-pow((-rho_a[3]),7))
					 +AA[8][3]/8.0*(pow((rho-rho_a[3]),8)
									-pow((-rho_a[3]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][3]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][3]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][3]
					   -AA[9][3]*AA[0][1]-AA[10][3])/(pow(AA[0][1],2)))

	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[4]),2)/(Tsq)
	-2*(AA[0][0]/T-tau_c)*(AA[0][0]/T-tau_a[4])*AA[0][0]/(Tsq))
					*(AA[1][4]*rho
		             +AA[2][4]/2.0*(pow((rho-rho_a[4]),2)
									-pow((-rho_a[4]),2))
					 +AA[3][4]/3.0*(pow((rho-rho_a[4]),3)
									-pow((-rho_a[4]),3))
					 +AA[4][4]/4.0*(pow((rho-rho_a[4]),4)
									-pow((-rho_a[4]),4))
					 +AA[5][4]/5.0*(pow((rho-rho_a[4]),5)
									-pow((-rho_a[4]),5))
					 +AA[6][4]/6.0*(pow((rho-rho_a[4]),6)
									-pow((-rho_a[4]),6))
					 +AA[7][4]/7.0*(pow((rho-rho_a[4]),7)
									-pow((-rho_a[4]),7))
					 +AA[8][4]/8.0*(pow((rho-rho_a[4]),8)
									-pow((-rho_a[4]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][4]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][4]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][4]
					   -AA[9][4]*AA[0][1]-AA[10][4])/(pow(AA[0][1],2)))
					 
	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[5]),3)/(Tsq)
	-3*(AA[0][0]/T-tau_c)*pow((AA[0][0]/T-tau_a[5]),2)*AA[0][0]/(Tsq))
					*(AA[1][5]*rho
		             +AA[2][5]/2.0*(pow((rho-rho_a[5]),2)
									-pow((-rho_a[5]),2))
					 +AA[3][5]/3.0*(pow((rho-rho_a[5]),3)
									-pow((-rho_a[5]),3))
					 +AA[4][5]/4.0*(pow((rho-rho_a[5]),4)
									-pow((-rho_a[5]),4))
					 +AA[5][5]/5.0*(pow((rho-rho_a[5]),5)
									-pow((-rho_a[5]),5))
					 +AA[6][5]/6.0*(pow((rho-rho_a[5]),6)
									-pow((-rho_a[5]),6))
					 +AA[7][5]/7.0*(pow((rho-rho_a[5]),7)
									-pow((-rho_a[5]),7))
					 +AA[8][5]/8.0*(pow((rho-rho_a[5]),8)
									-pow((-rho_a[5]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][5]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][5]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][5]
					   -AA[9][5]*AA[0][1]-AA[10][5])/(pow(AA[0][1],2)))
					 
	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[6]),4)/(Tsq)
	-4*(AA[0][0]/T-tau_c)*pow((AA[0][0]/T-tau_a[6]),3)*AA[0][0]/(Tsq))
					*(AA[1][6]*rho
		             +AA[2][6]/2.0*(pow((rho-rho_a[6]),2)
									-pow((-rho_a[6]),2))
					 +AA[3][6]/3.0*(pow((rho-rho_a[6]),3)
									-pow((-rho_a[6]),3))
					 +AA[4][6]/4.0*(pow((rho-rho_a[6]),4)
									-pow((-rho_a[6]),4))
					 +AA[5][6]/5.0*(pow((rho-rho_a[6]),5)
									-pow((-rho_a[6]),5))
					 +AA[6][6]/6.0*(pow((rho-rho_a[6]),6)
									-pow((-rho_a[6]),6))
					 +AA[7][6]/7.0*(pow((rho-rho_a[6]),7)
									-pow((-rho_a[6]),7))
					 +AA[8][6]/8.0*(pow((rho-rho_a[6]),8)
									-pow((-rho_a[6]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][6]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][6]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][6]
					   -AA[9][6]*AA[0][1]-AA[10][6])/(pow(AA[0][1],2)))
					 
	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[7]),5)/(Tsq)
	-5*(AA[0][0]/T-tau_c)*pow((AA[0][0]/T-tau_a[7]),4)*AA[0][0]/(Tsq))
					*(AA[1][7]*rho
		             +AA[2][7]/2.0*(pow((rho-rho_a[7]),2)
									-pow((-rho_a[7]),2))
					 +AA[3][7]/3.0*(pow((rho-rho_a[7]),3)
									-pow((-rho_a[7]),3))
					 +AA[4][7]/4.0*(pow((rho-rho_a[7]),4)
									-pow((-rho_a[7]),4))
					 +AA[5][7]/5.0*(pow((rho-rho_a[7]),5)
									-pow((-rho_a[7]),5))
					 +AA[6][7]/6.0*(pow((rho-rho_a[7]),6)
									-pow((-rho_a[7]),6))
					 +AA[7][7]/7.0*(pow((rho-rho_a[7]),7)
									-pow((-rho_a[7]),7))
					 +AA[8][7]/8.0*(pow((rho-rho_a[7]),8)
									-pow((-rho_a[7]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][7]*AA[0][1]
					   +exp(-AA[0][1]*rho)*AA[10][7]*AA[0][1]*rho
					   +exp(-AA[0][1]*rho)*AA[10][7]
					   -AA[9][7]*AA[0][1]-AA[10][7])/(pow(AA[0][1],2)))

  );


}


// Integral Part II (Enthaply and Entropy Calculations)             
//(using Derivative of equation Q-2 wrt. rho used with P-6.)
// integral from 0 to rho of rho * (partial^2 Q2 / (partial rho)(partial T)) d (rho)
double Fnc_intdQdrhodT( double rho, double T, double Tcrit, 
			  const double AA[][8], const double rho_a[])
{

	int i;
	double tau, tau_c, tau_a[8], Tsq, Tcu , Tfr;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	tau = AA[0][0]/T; tau_c = AA[0][0]/Tcrit;
	for (i=2;i<=7;++i) tau_a[i] = 2.5;tau_a[1]= tau_c;

	return (
		(-AA[0][0]/(Tsq*(AA[0][0]/T-tau_a[1]))
		+(AA[0][0]/T-tau_c)*AA[0][0]/(pow((AA[0][0]/T-tau_a[1]),2)*Tsq))
				   *(+1.0/2.0*AA[2][1]*pow(rho,2)
					 +2.0/3.0*AA[3][1]*pow(rho,3)
					         -AA[3][1]*rho_a[1]*pow(rho,2)
					 +AA[4][1]*(3.0/4.0*pow(rho,4)
							-2*rho_a[1]*pow(rho,3)
							+3.0/2.0*pow(rho_a[1],2)*pow(rho,2))
					 +AA[5][1]*(4.0/5.0*pow(rho,5)
							-3*rho_a[1]*pow(rho,4)
							+4*pow(rho_a[1],2)*pow(rho,3)
							-2*pow(rho_a[1],3)*pow(rho,2))
					 +AA[6][1]*(5.0/6.0*pow(rho,6)
							-4*rho_a[1]*pow(rho,5)
							+15.0/2.0*pow(rho_a[1],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[1],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[1],4)*pow(rho,2))
					 +AA[7][1]*(6.0/7.0*pow(rho,7)
							-5*rho_a[1]*pow(rho,6)
							+12*pow(rho_a[1],2)*pow(rho,5)
							-15*pow(rho_a[1],3)*pow(rho,4)
							+10*pow(rho_a[1],4)*pow(rho,3)
							-3*pow(rho_a[1],5)*pow(rho,2))
					 +AA[8][1]*(7.0/8.0*pow(rho,8)
							-6*rho_a[1]*pow(rho,7)
							+35.0/2.0*pow(rho_a[1],2)*pow(rho,6)
							-28*pow(rho_a[1],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[1],4)*pow(rho,4)
							-14*pow(rho_a[1],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[1],6)*pow(rho,2))
					 +(AA[9][1]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][1]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][1]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][1]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][1]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][1]-AA[10][1])/(pow(AA[0][1],2)))
				
		-AA[0][0]/(Tsq)			 
				   *(+1.0/2.0*AA[2][2]*pow(rho,2)
					 +2.0/3.0*AA[3][2]*pow(rho,3)
					         -AA[3][2]*rho_a[2]*pow(rho,2)
					 +AA[4][2]*(3.0/4.0*pow(rho,4)
							-2*rho_a[2]*pow(rho,3)
							+3.0/2.0*pow(rho_a[2],2)*pow(rho,2))
					 +AA[5][2]*(4.0/5.0*pow(rho,5)
							-3*rho_a[2]*pow(rho,4)
							+4*pow(rho_a[2],2)*pow(rho,3)
							-2*pow(rho_a[2],3)*pow(rho,2))
					 +AA[6][2]*(5.0/6.0*pow(rho,6)
							-4*rho_a[2]*pow(rho,5)
							+15.0/2.0*pow(rho_a[2],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[2],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[2],4)*pow(rho,2))
					 +AA[7][2]*(6.0/7.0*pow(rho,7)
							-5*rho_a[2]*pow(rho,6)
							+12*pow(rho_a[2],2)*pow(rho,5)
							-15*pow(rho_a[2],3)*pow(rho,4)
							+10*pow(rho_a[2],4)*pow(rho,3)
							-3*pow(rho_a[2],5)*pow(rho,2))
					 +AA[8][2]*(7.0/8.0*pow(rho,8)
							-6*rho_a[2]*pow(rho,7)
							+35.0/2.0*pow(rho_a[2],2)*pow(rho,6)
							-28*pow(rho_a[2],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[2],4)*pow(rho,4)
							-14*pow(rho_a[2],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[2],6)*pow(rho,2))
					 +(AA[9][2]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][2]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][2]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][2]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][2]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][2]-AA[10][2])/(pow(AA[0][1],2)))
				
	+(-AA[0][0]*(AA[0][0]/T-tau_a[3])/(Tsq)
	 -(AA[0][0]/T-tau_c)*AA[0][0]/(Tsq))
					*(+1.0/2.0*AA[2][3]*pow(rho,2)
					 +2.0/3.0*AA[3][3]*pow(rho,3)
					         -AA[3][3]*rho_a[3]*pow(rho,2)
					 +AA[4][3]*(3.0/4.0*pow(rho,4)
							-2*rho_a[3]*pow(rho,3)
							+3.0/2.0*pow(rho_a[3],2)*pow(rho,2))
					 +AA[5][3]*(4.0/5.0*pow(rho,5)
							-3*rho_a[3]*pow(rho,4)
							+4*pow(rho_a[3],2)*pow(rho,3)
							-2*pow(rho_a[3],3)*pow(rho,2))
					 +AA[6][3]*(5.0/6.0*pow(rho,6)
							-4*rho_a[3]*pow(rho,5)
							+15.0/2.0*pow(rho_a[3],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[3],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[3],4)*pow(rho,2))
					 +AA[7][3]*(6.0/7.0*pow(rho,7)
							-5*rho_a[3]*pow(rho,6)
							+12*pow(rho_a[3],2)*pow(rho,5)
							-15*pow(rho_a[3],3)*pow(rho,4)
							+10*pow(rho_a[3],4)*pow(rho,3)
							-3*pow(rho_a[3],5)*pow(rho,2))
					 +AA[8][3]*(7.0/8.0*pow(rho,8)
							-6*rho_a[3]*pow(rho,7)
							+35.0/2.0*pow(rho_a[3],2)*pow(rho,6)
							-28*pow(rho_a[3],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[3],4)*pow(rho,4)
							-14*pow(rho_a[3],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[3],6)*pow(rho,2))
					 +(AA[9][3]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][3]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][3]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][3]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][3]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][3]-AA[10][3])/(pow(AA[0][1],2)))

	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[4]),2)/(Tsq)
	-2*(AA[0][0]/T-tau_c)*(AA[0][0]/T-tau_a[4])*AA[0][0]/(Tsq))
					*(+1.0/2.0*AA[2][4]*pow(rho,2)
					 +2.0/3.0*AA[3][4]*pow(rho,3)
					         -AA[3][4]*rho_a[4]*pow(rho,2)
					 +AA[4][4]*(3.0/4.0*pow(rho,4)
							-2*rho_a[4]*pow(rho,3)
							+3.0/2.0*pow(rho_a[4],2)*pow(rho,2))
					 +AA[5][4]*(4.0/5.0*pow(rho,5)
							-3*rho_a[4]*pow(rho,4)
							+4*pow(rho_a[4],2)*pow(rho,3)
							-2*pow(rho_a[4],3)*pow(rho,2))
					 +AA[6][4]*(5.0/6.0*pow(rho,6)
							-4*rho_a[4]*pow(rho,5)
							+15.0/2.0*pow(rho_a[4],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[4],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[4],4)*pow(rho,2))
					 +AA[7][4]*(6.0/7.0*pow(rho,7)
							-5*rho_a[4]*pow(rho,6)
							+12*pow(rho_a[4],2)*pow(rho,5)
							-15*pow(rho_a[4],3)*pow(rho,4)
							+10*pow(rho_a[4],4)*pow(rho,3)
							-3*pow(rho_a[4],5)*pow(rho,2))
					 +AA[8][4]*(7.0/8.0*pow(rho,8)
							-6*rho_a[4]*pow(rho,7)
							+35.0/2.0*pow(rho_a[4],2)*pow(rho,6)
							-28*pow(rho_a[4],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[4],4)*pow(rho,4)
							-14*pow(rho_a[4],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[4],6)*pow(rho,2))
					 +(AA[9][4]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][4]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][4]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][4]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][4]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][4]-AA[10][4])/(pow(AA[0][1],2)))
					 
	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[5]),3)/(Tsq)
	-3*(AA[0][0]/T-tau_c)*pow((AA[0][0]/T-tau_a[5]),2)*AA[0][0]/(Tsq))
					*(+1.0/2.0*AA[2][5]*pow(rho,2)
					 +2.0/3.0*AA[3][5]*pow(rho,3)
					         -AA[3][5]*rho_a[5]*pow(rho,2)
					 +AA[4][5]*(3.0/4.0*pow(rho,4)
							-2*rho_a[5]*pow(rho,3)
							+3.0/2.0*pow(rho_a[5],2)*pow(rho,2))
					 +AA[5][5]*(4.0/5.0*pow(rho,5)
							-3*rho_a[5]*pow(rho,4)
							+4*pow(rho_a[5],2)*pow(rho,3)
							-2*pow(rho_a[5],3)*pow(rho,2))
					 +AA[6][5]*(5.0/6.0*pow(rho,6)
							-4*rho_a[5]*pow(rho,5)
							+15.0/2.0*pow(rho_a[5],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[5],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[5],4)*pow(rho,2))
					 +AA[7][5]*(6.0/7.0*pow(rho,7)
							-5*rho_a[5]*pow(rho,6)
							+12*pow(rho_a[5],2)*pow(rho,5)
							-15*pow(rho_a[5],3)*pow(rho,4)
							+10*pow(rho_a[5],4)*pow(rho,3)
							-3*pow(rho_a[5],5)*pow(rho,2))
					 +AA[8][5]*(7.0/8.0*pow(rho,8)
							-6*rho_a[5]*pow(rho,7)
							+35.0/2.0*pow(rho_a[5],2)*pow(rho,6)
							-28*pow(rho_a[5],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[5],4)*pow(rho,4)
							-14*pow(rho_a[5],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[5],6)*pow(rho,2))
					 +(AA[9][5]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][5]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][5]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][5]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][5]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][5]-AA[10][5])/(pow(AA[0][1],2)))
					 
	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[6]),4)/(Tsq)
	-4*(AA[0][0]/T-tau_c)*pow((AA[0][0]/T-tau_a[6]),3)*AA[0][0]/(Tsq))
					*(+1.0/2.0*AA[2][6]*pow(rho,2)
					 +2.0/3.0*AA[3][6]*pow(rho,3)
					         -AA[3][6]*rho_a[6]*pow(rho,2)
					 +AA[4][6]*(3.0/4.0*pow(rho,4)
							-2*rho_a[6]*pow(rho,3)
							+3.0/2.0*pow(rho_a[6],2)*pow(rho,2))
					 +AA[5][6]*(4.0/5.0*pow(rho,5)
							-3*rho_a[6]*pow(rho,4)
							+4*pow(rho_a[6],2)*pow(rho,3)
							-2*pow(rho_a[6],3)*pow(rho,2))
					 +AA[6][6]*(5.0/6.0*pow(rho,6)
							-4*rho_a[6]*pow(rho,5)
							+15.0/2.0*pow(rho_a[6],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[6],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[6],4)*pow(rho,2))
					 +AA[7][6]*(6.0/7.0*pow(rho,7)
							-5*rho_a[6]*pow(rho,6)
							+12*pow(rho_a[6],2)*pow(rho,5)
							-15*pow(rho_a[6],3)*pow(rho,4)
							+10*pow(rho_a[6],4)*pow(rho,3)
							-3*pow(rho_a[6],5)*pow(rho,2))
					 +AA[8][6]*(7.0/8.0*pow(rho,8)
							-6*rho_a[6]*pow(rho,7)
							+35.0/2.0*pow(rho_a[6],2)*pow(rho,6)
							-28*pow(rho_a[6],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[6],4)*pow(rho,4)
							-14*pow(rho_a[6],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[6],6)*pow(rho,2))
					 +(AA[9][6]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][6]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][6]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][6]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][6]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][6]-AA[10][6])/(pow(AA[0][1],2)))
					 
	+(-AA[0][0]*pow((AA[0][0]/T-tau_a[7]),5)/(Tsq)
	-5*(AA[0][0]/T-tau_c)*pow((AA[0][0]/T-tau_a[7]),4)*AA[0][0]/(Tsq))
					*(+1.0/2.0*AA[2][7]*pow(rho,2)
					 +2.0/3.0*AA[3][7]*pow(rho,3)
					         -AA[3][7]*rho_a[7]*pow(rho,2)
					 +AA[4][7]*(3.0/4.0*pow(rho,4)
							-2*rho_a[7]*pow(rho,3)
							+3.0/2.0*pow(rho_a[7],2)*pow(rho,2))
					 +AA[5][7]*(4.0/5.0*pow(rho,5)
							-3*rho_a[7]*pow(rho,4)
							+4*pow(rho_a[7],2)*pow(rho,3)
							-2*pow(rho_a[7],3)*pow(rho,2))
					 +AA[6][7]*(5.0/6.0*pow(rho,6)
							-4*rho_a[7]*pow(rho,5)
							+15.0/2.0*pow(rho_a[7],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[7],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[7],4)*pow(rho,2))
					 +AA[7][7]*(6.0/7.0*pow(rho,7)
							-5*rho_a[7]*pow(rho,6)
							+12*pow(rho_a[7],2)*pow(rho,5)
							-15*pow(rho_a[7],3)*pow(rho,4)
							+10*pow(rho_a[7],4)*pow(rho,3)
							-3*pow(rho_a[7],5)*pow(rho,2))
					 +AA[8][7]*(7.0/8.0*pow(rho,8)
							-6*rho_a[7]*pow(rho,7)
							+35.0/2.0*pow(rho_a[7],2)*pow(rho,6)
							-28*pow(rho_a[7],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[7],4)*pow(rho,4)
							-14*pow(rho_a[7],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[7],6)*pow(rho,2))
					 +(AA[9][7]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][7]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][7]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][7]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][7]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][7]-AA[10][7])/(pow(AA[0][1],2)))
    );

}

// Integral Part III (Entropy Calculations)             
//(using equation Q-2 wrt.)
// integral from 0 to rho of Q2 d (rho)
double Fnc_intQ( double rho, double T, double Tcrit, 
			  const double AA[][8], const double rho_a[])
{
	int i;
	double tau, tau_c, tau_a[8], Tsq, Tcu , Tfr;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	tau = AA[0][0]/T; tau_c = AA[0][0]/Tcrit;
	for (i=2;i<=7;++i) tau_a[i] = 2.5;tau_a[1]= tau_c;
	
	return (
		(tau-tau_c)*((AA[1][1]*rho
		             +AA[2][1]/2.0*(pow((rho-rho_a[1]),2)
									-pow((-rho_a[1]),2))
					 +AA[3][1]/3.0*(pow((rho-rho_a[1]),3)
									-pow((-rho_a[1]),3))
					 +AA[4][1]/4.0*(pow((rho-rho_a[1]),4)
									-pow((-rho_a[1]),4))
					 +AA[5][1]/5.0*(pow((rho-rho_a[1]),5)
									-pow((-rho_a[1]),5))
					 +AA[6][1]/6.0*(pow((rho-rho_a[1]),6)
									-pow((-rho_a[1]),6))
					 +AA[7][1]/7.0*(pow((rho-rho_a[1]),7)
									-pow((-rho_a[1]),7))
					 +AA[8][1]/8.0*(pow((rho-rho_a[1]),8)
									-pow((-rho_a[1]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][1]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][1]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][1]
					 -AA[9][1]*AA[0][1]-AA[10][1])/pow(AA[0][1],2))/(tau-tau_a[1])
		
					 +(AA[1][2]*rho
		             +AA[2][2]/2.0*(pow((rho-rho_a[2]),2)
									-pow((-rho_a[2]),2))
					 +AA[3][2]/3.0*(pow((rho-rho_a[2]),3)
									-pow((-rho_a[2]),3))
					 +AA[4][2]/4.0*(pow((rho-rho_a[2]),4)
									-pow((-rho_a[2]),4))
					 +AA[5][2]/5.0*(pow((rho-rho_a[2]),5)
									-pow((-rho_a[2]),5))
					 +AA[6][2]/6.0*(pow((rho-rho_a[2]),6)
									-pow((-rho_a[2]),6))
					 +AA[7][2]/7.0*(pow((rho-rho_a[2]),7)
									-pow((-rho_a[2]),7))
					 +AA[8][2]/8.0*(pow((rho-rho_a[2]),8)
									-pow((-rho_a[2]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][2]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][2]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][2]
					 -AA[9][2]*AA[0][1]-AA[10][2])/pow(AA[0][1],2))

	+(tau-tau_a[3])*(AA[1][3]*rho
		             +AA[2][3]/2.0*(pow((rho-rho_a[3]),2)
									-pow((-rho_a[3]),2))
					 +AA[3][3]/3.0*(pow((rho-rho_a[3]),3)
									-pow((-rho_a[3]),3))
					 +AA[4][3]/4.0*(pow((rho-rho_a[3]),4)
									-pow((-rho_a[3]),4))
					 +AA[5][3]/5.0*(pow((rho-rho_a[3]),5)
									-pow((-rho_a[3]),5))
					 +AA[6][3]/6.0*(pow((rho-rho_a[3]),6)
									-pow((-rho_a[3]),6))
					 +AA[7][3]/7.0*(pow((rho-rho_a[3]),7)
									-pow((-rho_a[3]),7))
					 +AA[8][3]/8.0*(pow((rho-rho_a[3]),8)
									-pow((-rho_a[3]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][3]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][3]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][3]
					 -AA[9][3]*AA[0][1]-AA[10][3])/pow(AA[0][1],2))

	+pow((tau-tau_a[4]),2)*(AA[1][4]*rho
		             +AA[2][4]/2.0*(pow((rho-rho_a[4]),2)
									-pow((-rho_a[4]),2))
					 +AA[3][4]/3.0*(pow((rho-rho_a[4]),3)
									-pow((-rho_a[4]),3))
					 +AA[4][4]/4.0*(pow((rho-rho_a[4]),4)
									-pow((-rho_a[4]),4))
					 +AA[5][4]/5.0*(pow((rho-rho_a[4]),5)
									-pow((-rho_a[4]),5))
					 +AA[6][4]/6.0*(pow((rho-rho_a[4]),6)
									-pow((-rho_a[4]),6))
					 +AA[7][4]/7.0*(pow((rho-rho_a[4]),7)
									-pow((-rho_a[4]),7))
					 +AA[8][4]/8.0*(pow((rho-rho_a[4]),8)
									-pow((-rho_a[4]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][4]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][4]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][4]
					 -AA[9][4]*AA[0][1]-AA[10][4])/pow(AA[0][1],2))
					 
	+pow((tau-tau_a[5]),3)*(AA[1][5]*rho
		             +AA[2][5]/2.0*(pow((rho-rho_a[5]),2)
									-pow((-rho_a[5]),2))
					 +AA[3][5]/3.0*(pow((rho-rho_a[5]),3)
									-pow((-rho_a[5]),3))
					 +AA[4][5]/4.0*(pow((rho-rho_a[5]),4)
									-pow((-rho_a[5]),4))
					 +AA[5][5]/5.0*(pow((rho-rho_a[5]),5)
									-pow((-rho_a[5]),5))
					 +AA[6][5]/6.0*(pow((rho-rho_a[5]),6)
									-pow((-rho_a[5]),6))
					 +AA[7][5]/7.0*(pow((rho-rho_a[5]),7)
									-pow((-rho_a[5]),7))
					 +AA[8][5]/8.0*(pow((rho-rho_a[5]),8)
									-pow((-rho_a[5]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][5]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][5]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][5]
					 -AA[9][5]*AA[0][1]-AA[10][5])/pow(AA[0][1],2))
					 
	+pow((tau-tau_a[6]),4)*(AA[1][6]*rho
		             +AA[2][6]/2.0*(pow((rho-rho_a[6]),2)
									-pow((-rho_a[6]),2))
					 +AA[3][6]/3.0*(pow((rho-rho_a[6]),3)
									-pow((-rho_a[6]),3))
					 +AA[4][6]/4.0*(pow((rho-rho_a[6]),4)
									-pow((-rho_a[6]),4))
					 +AA[5][6]/5.0*(pow((rho-rho_a[6]),5)
									-pow((-rho_a[6]),5))
					 +AA[6][6]/6.0*(pow((rho-rho_a[6]),6)
									-pow((-rho_a[6]),6))
					 +AA[7][6]/7.0*(pow((rho-rho_a[6]),7)
									-pow((-rho_a[6]),7))
					 +AA[8][6]/8.0*(pow((rho-rho_a[6]),8)
									-pow((-rho_a[6]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][6]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][6]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][6]
					 -AA[9][6]*AA[0][1]-AA[10][6])/pow(AA[0][1],2))
					 
	+pow((tau-tau_a[7]),5)*(AA[1][7]*rho
		             +AA[2][7]/2.0*(pow((rho-rho_a[7]),2)
									-pow((-rho_a[7]),2))
					 +AA[3][7]/3.0*(pow((rho-rho_a[7]),3)
									-pow((-rho_a[7]),3))
					 +AA[4][7]/4.0*(pow((rho-rho_a[7]),4)
									-pow((-rho_a[7]),4))
					 +AA[5][7]/5.0*(pow((rho-rho_a[7]),5)
									-pow((-rho_a[7]),5))
					 +AA[6][7]/6.0*(pow((rho-rho_a[7]),6)
									-pow((-rho_a[7]),6))
					 +AA[7][7]/7.0*(pow((rho-rho_a[7]),7)
									-pow((-rho_a[7]),7))
					 +AA[8][7]/8.0*(pow((rho-rho_a[7]),8)
									-pow((-rho_a[7]),8))
					 -(exp(-AA[0][1]*rho)*AA[9][7]*AA[0][1]
					 +exp(-AA[0][1]*rho)*AA[10][7]*AA[0][1]*rho
					 +exp(-AA[0][1]*rho)*AA[10][7]
					 -AA[9][7]*AA[0][1]-AA[10][7])/pow(AA[0][1],2)))
    );


};

// Integral Part IV (Entropy Calculations)             
//(using Derivative of equation Q-2 wrt. rho used with P-6.)
// integral from 0 to rho of (partial Q2/ partial rho) at constant T d (rho)
double Fnc_intdQdrho( double rho, double T, double Tcrit, 
			  const double AA[][8], const double rho_a[])
{

	int i;
	double tau, tau_c, tau_a[8], Tsq, Tcu , Tfr;
//Note that Temperature is in K 
//	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	tau = AA[0][0]/T; tau_c = AA[0][0]/Tcrit;
	for (i=2;i<=7;++i) tau_a[i] = 2.5;tau_a[1]= tau_c;

	return (
		(tau-tau_c)/(tau-tau_a[1])
				   *(+1.0/2.0*AA[2][1]*pow(rho,2)
					 +2.0/3.0*AA[3][1]*pow(rho,3)
					         -AA[3][1]*rho_a[1]*pow(rho,2)
					 +AA[4][1]*(3.0/4.0*pow(rho,4)
							-2*rho_a[1]*pow(rho,3)
							+3.0/2.0*pow(rho_a[1],2)*pow(rho,2))
					 +AA[5][1]*(4.0/5.0*pow(rho,5)
							-3*rho_a[1]*pow(rho,4)
							+4*pow(rho_a[1],2)*pow(rho,3)
							-2*pow(rho_a[1],3)*pow(rho,2))
					 +AA[6][1]*(5.0/6.0*pow(rho,6)
							-4*rho_a[1]*pow(rho,5)
							+15.0/2.0*pow(rho_a[1],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[1],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[1],4)*pow(rho,2))
					 +AA[7][1]*(6.0/7.0*pow(rho,7)
							-5*rho_a[1]*pow(rho,6)
							+12*pow(rho_a[1],2)*pow(rho,5)
							-15*pow(rho_a[1],3)*pow(rho,4)
							+10*pow(rho_a[1],4)*pow(rho,3)
							-3*pow(rho_a[1],5)*pow(rho,2))
					 +AA[8][1]*(7.0/8.0*pow(rho,8)
							-6*rho_a[1]*pow(rho,7)
							+35.0/2.0*pow(rho_a[1],2)*pow(rho,6)
							-28*pow(rho_a[1],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[1],4)*pow(rho,4)
							-14*pow(rho_a[1],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[1],6)*pow(rho,2))
					 +(AA[9][1]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][1]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][1]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][1]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][1]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][1]-AA[10][1])/(pow(AA[0][1],2)))
				
		+(tau-tau_c)		 
				   *(+1.0/2.0*AA[2][2]*pow(rho,2)
					 +2.0/3.0*AA[3][2]*pow(rho,3)
					         -AA[3][2]*rho_a[2]*pow(rho,2)
					 +AA[4][2]*(3.0/4.0*pow(rho,4)
							-2*rho_a[2]*pow(rho,3)
							+3.0/2.0*pow(rho_a[2],2)*pow(rho,2))
					 +AA[5][2]*(4.0/5.0*pow(rho,5)
							-3*rho_a[2]*pow(rho,4)
							+4*pow(rho_a[2],2)*pow(rho,3)
							-2*pow(rho_a[2],3)*pow(rho,2))
					 +AA[6][2]*(5.0/6.0*pow(rho,6)
							-4*rho_a[2]*pow(rho,5)
							+15.0/2.0*pow(rho_a[2],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[2],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[2],4)*pow(rho,2))
					 +AA[7][2]*(6.0/7.0*pow(rho,7)
							-5*rho_a[2]*pow(rho,6)
							+12*pow(rho_a[2],2)*pow(rho,5)
							-15*pow(rho_a[2],3)*pow(rho,4)
							+10*pow(rho_a[2],4)*pow(rho,3)
							-3*pow(rho_a[2],5)*pow(rho,2))
					 +AA[8][2]*(7.0/8.0*pow(rho,8)
							-6*rho_a[2]*pow(rho,7)
							+35.0/2.0*pow(rho_a[2],2)*pow(rho,6)
							-28*pow(rho_a[2],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[2],4)*pow(rho,4)
							-14*pow(rho_a[2],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[2],6)*pow(rho,2))
					 +(AA[9][2]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][2]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][2]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][2]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][2]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][2]-AA[10][2])/(pow(AA[0][1],2)))
				
		+(tau-tau_c)*(tau-tau_a[3])
					*(+1.0/2.0*AA[2][3]*pow(rho,2)
					 +2.0/3.0*AA[3][3]*pow(rho,3)
					         -AA[3][3]*rho_a[3]*pow(rho,2)
					 +AA[4][3]*(3.0/4.0*pow(rho,4)
							-2*rho_a[3]*pow(rho,3)
							+3.0/2.0*pow(rho_a[3],2)*pow(rho,2))
					 +AA[5][3]*(4.0/5.0*pow(rho,5)
							-3*rho_a[3]*pow(rho,4)
							+4*pow(rho_a[3],2)*pow(rho,3)
							-2*pow(rho_a[3],3)*pow(rho,2))
					 +AA[6][3]*(5.0/6.0*pow(rho,6)
							-4*rho_a[3]*pow(rho,5)
							+15.0/2.0*pow(rho_a[3],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[3],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[3],4)*pow(rho,2))
					 +AA[7][3]*(6.0/7.0*pow(rho,7)
							-5*rho_a[3]*pow(rho,6)
							+12*pow(rho_a[3],2)*pow(rho,5)
							-15*pow(rho_a[3],3)*pow(rho,4)
							+10*pow(rho_a[3],4)*pow(rho,3)
							-3*pow(rho_a[3],5)*pow(rho,2))
					 +AA[8][3]*(7.0/8.0*pow(rho,8)
							-6*rho_a[3]*pow(rho,7)
							+35.0/2.0*pow(rho_a[3],2)*pow(rho,6)
							-28*pow(rho_a[3],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[3],4)*pow(rho,4)
							-14*pow(rho_a[3],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[3],6)*pow(rho,2))
					 +(AA[9][3]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][3]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][3]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][3]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][3]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][3]-AA[10][3])/(pow(AA[0][1],2)))

		+(tau-tau_c)*pow((tau-tau_a[4]),2)
					*(+1.0/2.0*AA[2][4]*pow(rho,2)
					 +2.0/3.0*AA[3][4]*pow(rho,3)
					         -AA[3][4]*rho_a[4]*pow(rho,2)
					 +AA[4][4]*(3.0/4.0*pow(rho,4)
							-2*rho_a[4]*pow(rho,3)
							+3.0/2.0*pow(rho_a[4],2)*pow(rho,2))
					 +AA[5][4]*(4.0/5.0*pow(rho,5)
							-3*rho_a[4]*pow(rho,4)
							+4*pow(rho_a[4],2)*pow(rho,3)
							-2*pow(rho_a[4],3)*pow(rho,2))
					 +AA[6][4]*(5.0/6.0*pow(rho,6)
							-4*rho_a[4]*pow(rho,5)
							+15.0/2.0*pow(rho_a[4],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[4],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[4],4)*pow(rho,2))
					 +AA[7][4]*(6.0/7.0*pow(rho,7)
							-5*rho_a[4]*pow(rho,6)
							+12*pow(rho_a[4],2)*pow(rho,5)
							-15*pow(rho_a[4],3)*pow(rho,4)
							+10*pow(rho_a[4],4)*pow(rho,3)
							-3*pow(rho_a[4],5)*pow(rho,2))
					 +AA[8][4]*(7.0/8.0*pow(rho,8)
							-6*rho_a[4]*pow(rho,7)
							+35.0/2.0*pow(rho_a[4],2)*pow(rho,6)
							-28*pow(rho_a[4],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[4],4)*pow(rho,4)
							-14*pow(rho_a[4],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[4],6)*pow(rho,2))
					 +(AA[9][4]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][4]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][4]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][4]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][4]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][4]-AA[10][4])/(pow(AA[0][1],2)))
					 
		+(tau-tau_c)*pow((tau-tau_a[5]),3)
					*(+1.0/2.0*AA[2][5]*pow(rho,2)
					 +2.0/3.0*AA[3][5]*pow(rho,3)
					         -AA[3][5]*rho_a[5]*pow(rho,2)
					 +AA[4][5]*(3.0/4.0*pow(rho,4)
							-2*rho_a[5]*pow(rho,3)
							+3.0/2.0*pow(rho_a[5],2)*pow(rho,2))
					 +AA[5][5]*(4.0/5.0*pow(rho,5)
							-3*rho_a[5]*pow(rho,4)
							+4*pow(rho_a[5],2)*pow(rho,3)
							-2*pow(rho_a[5],3)*pow(rho,2))
					 +AA[6][5]*(5.0/6.0*pow(rho,6)
							-4*rho_a[5]*pow(rho,5)
							+15.0/2.0*pow(rho_a[5],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[5],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[5],4)*pow(rho,2))
					 +AA[7][5]*(6.0/7.0*pow(rho,7)
							-5*rho_a[5]*pow(rho,6)
							+12*pow(rho_a[5],2)*pow(rho,5)
							-15*pow(rho_a[5],3)*pow(rho,4)
							+10*pow(rho_a[5],4)*pow(rho,3)
							-3*pow(rho_a[5],5)*pow(rho,2))
					 +AA[8][5]*(7.0/8.0*pow(rho,8)
							-6*rho_a[5]*pow(rho,7)
							+35.0/2.0*pow(rho_a[5],2)*pow(rho,6)
							-28*pow(rho_a[5],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[5],4)*pow(rho,4)
							-14*pow(rho_a[5],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[5],6)*pow(rho,2))
					 +(AA[9][5]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][5]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][5]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][5]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][5]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][5]-AA[10][5])/(pow(AA[0][1],2)))
					 
		+(tau-tau_c)*pow((tau-tau_a[6]),4)
					*(+1.0/2.0*AA[2][6]*pow(rho,2)
					 +2.0/3.0*AA[3][6]*pow(rho,3)
					         -AA[3][6]*rho_a[6]*pow(rho,2)
					 +AA[4][6]*(3.0/4.0*pow(rho,4)
							-2*rho_a[6]*pow(rho,3)
							+3.0/2.0*pow(rho_a[6],2)*pow(rho,2))
					 +AA[5][6]*(4.0/5.0*pow(rho,5)
							-3*rho_a[6]*pow(rho,4)
							+4*pow(rho_a[6],2)*pow(rho,3)
							-2*pow(rho_a[6],3)*pow(rho,2))
					 +AA[6][6]*(5.0/6.0*pow(rho,6)
							-4*rho_a[6]*pow(rho,5)
							+15.0/2.0*pow(rho_a[6],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[6],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[6],4)*pow(rho,2))
					 +AA[7][6]*(6.0/7.0*pow(rho,7)
							-5*rho_a[6]*pow(rho,6)
							+12*pow(rho_a[6],2)*pow(rho,5)
							-15*pow(rho_a[6],3)*pow(rho,4)
							+10*pow(rho_a[6],4)*pow(rho,3)
							-3*pow(rho_a[6],5)*pow(rho,2))
					 +AA[8][6]*(7.0/8.0*pow(rho,8)
							-6*rho_a[6]*pow(rho,7)
							+35.0/2.0*pow(rho_a[6],2)*pow(rho,6)
							-28*pow(rho_a[6],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[6],4)*pow(rho,4)
							-14*pow(rho_a[6],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[6],6)*pow(rho,2))
					 +(AA[9][6]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][6]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][6]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][6]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][6]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][6]-AA[10][6])/(pow(AA[0][1],2)))
					 
		+(tau-tau_c)*pow((tau-tau_a[7]),5)
					*(+1.0/2.0*AA[2][7]*pow(rho,2)
					 +2.0/3.0*AA[3][7]*pow(rho,3)
					         -AA[3][7]*rho_a[7]*pow(rho,2)
					 +AA[4][7]*(3.0/4.0*pow(rho,4)
							-2*rho_a[7]*pow(rho,3)
							+3.0/2.0*pow(rho_a[7],2)*pow(rho,2))
					 +AA[5][7]*(4.0/5.0*pow(rho,5)
							-3*rho_a[7]*pow(rho,4)
							+4*pow(rho_a[7],2)*pow(rho,3)
							-2*pow(rho_a[7],3)*pow(rho,2))
					 +AA[6][7]*(5.0/6.0*pow(rho,6)
							-4*rho_a[7]*pow(rho,5)
							+15.0/2.0*pow(rho_a[7],2)*pow(rho,4)
							-20.0/3.0*pow(rho_a[7],3)*pow(rho,3)
							+5.0/2.0*pow(rho_a[7],4)*pow(rho,2))
					 +AA[7][7]*(6.0/7.0*pow(rho,7)
							-5*rho_a[7]*pow(rho,6)
							+12*pow(rho_a[7],2)*pow(rho,5)
							-15*pow(rho_a[7],3)*pow(rho,4)
							+10*pow(rho_a[7],4)*pow(rho,3)
							-3*pow(rho_a[7],5)*pow(rho,2))
					 +AA[8][7]*(7.0/8.0*pow(rho,8)
							-6*rho_a[7]*pow(rho,7)
							+35.0/2.0*pow(rho_a[7],2)*pow(rho,6)
							-28*pow(rho_a[7],3)*pow(rho,5)
							+105.0/4.0*pow(rho_a[7],4)*pow(rho,4)
							-14*pow(rho_a[7],5)*pow(rho,3)
							+7.0/2.0*pow(rho_a[7],6)*pow(rho,2))
					 +(AA[9][7]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*rho
					 +AA[9][7]*exp(-AA[0][1]*rho)*AA[0][1]
					 +AA[10][7]*exp(-AA[0][1]*rho)*pow(AA[0][1],2)*pow(rho,2)
					 +AA[10][7]*exp(-AA[0][1]*rho)*AA[0][1]*rho
					 +AA[10][7]*exp(-AA[0][1]*rho)
					 -AA[0][1]*AA[9][7]-AA[10][7])/(pow(AA[0][1],2)))
    );

}

// P-V-T equation P-6.
double PressureEOS_P6( double V, double T, const double AA[][8], 
					  const double rho_a[], double Tcrit, double Mw )
{

	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Q, delQdelrho;

	double Tsq, Tcu , Tfr;
	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	Q = Fnc_Q(rho,T,Tcrit,AA,rho_a);
	delQdelrho=Fnc_delQdelrho(rho,T,Tcrit,AA,rho_a);


	return (
		rho*RgasSI/Mw*T *(1 + rho*Q
		                  + pow(rho,2)*delQdelrho)
		) * ConvPa;
	
}

// P-V-T equation P-6 with Q-1.
double PressureEOS_P6Q1( double V, double T, const double AA[][7],
						 double Tcrit, double Mw )
{

	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Q, delQdelrho;

	double Tsq, Tcu , Tfr;
	T /= 1.8;						// T to Kelvin
	Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	Q = Fnc_Q1(rho,T,Tcrit,AA);
	delQdelrho=Fnc_delQ1delrho(rho,T,Tcrit,AA);


	return (
		rho*RgasSI/Mw*T *(1 + rho*Q
		                  + pow(rho,2)*delQdelrho)
		) * ConvPa;
	
}


// End EOS
//============================

////////////      Saturation Functions			///////////
// Inputs:												 //
//		T	temperature (R)							 //
//		P	pressure (psia)								 //
///////////////////////////////////////////////////////////

double SatPressure_S6( double T, const double F[], double Tcrit, double Pcrit )
{
	double a, Tp;
	T /= 1.8;						// T to Kelvin
	a=F[0]; Tp=F[9];

	return (
		exp((Tcrit/T-1)*(F[1]
						+F[2]*a*(T-Tp)
						+F[3]*pow((a*(T-Tp)),2)
						+F[4]*pow((a*(T-Tp)),3)
						+F[5]*pow((a*(T-Tp)),4)
						+F[6]*pow((a*(T-Tp)),5)
						+F[7]*pow((a*(T-Tp)),6)
						+F[8]*pow((a*(T-Tp)),7)))*Pcrit*1E6		
		) * ConvPa;

}

double SLDensity_D5( double T, const double D[], double Tcrit, double rhocrit, double Mw )
{

	T /= 1.8;						// T to Kelvin


	return (
	rhocrit*(1+D[1]*pow((1-T/Tcrit),(1.0/3.0))
			  +D[2]*pow((1-T/Tcrit),(2.0/3.0))
			  +D[3]*(1-T/Tcrit)
			  +D[4]*pow((1-T/Tcrit),(4.0/3.0))
			  +D[5]*pow((1-T/Tcrit),(5.0/3.0))
			  +D[6]*pow((1-T/Tcrit),2)
			  +D[7]*pow((1-T/Tcrit),(7.0/3.0))
			  +D[8]*pow((1-T/Tcrit),(8.0/3.0)))		
		) / (ConvM3kg*Mw);
}


//////////////////// End saturation functions ///////////////////


// The enthalpy is computed from the ideal case (the
// integral of (Cv + R) from T0 to T at zero density) plus an 
// integral that accounts for the density increase in a 
// real gas (the residual enthalpy).  The following 
// functions are for the real integral.  H means enthalpy.  
// P4 is the EOS (in Reynolds) that was used to model the 
// real gas. 

double HResidualP4( double V, double T, const double A[], double Mw )
{

/* Calculates the real part of the enthalpy using the formula
HR=int(0,rho; 1/rho^2*(P-T*((delP/delT)|rho=const))*d(rho) + P/rho - RT
(See Smith, van Ness, Abbott p.213 eqn. 6.58) 
 */

	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);

	return 
		
		((1.0/2.0*A[2]*sqrt(T)+A[3]+2*A[4]/T+3*A[5]/Tsq)*rho
		+(A[7]+2*A[8]/T+3*A[9]/Tsq)*pow(rho,2)/2
		+(A[11]+2*A[12]/T)*pow(rho,3)/3
		+A[13]*pow(rho,4)/4
		+(2*A[14]/T+3*A[15]/Tsq)*pow(rho,5)/5
		+2*A[16]/T*pow(rho,6)/6
		+(2*A[17]/T+3*A[18]/Tsq)*pow(rho,7)/7
		+3*A[19]/Tsq*pow(rho,8)/8
		-1.0/2.0*(exp(-A[0]*pow(rho,2))-1)/A[0]*3*A[20]/Tsq
		-2*A[21]*(exp(-A[0]*pow(rho,2))-1)/(Tcu*A[0])
		-3.0/2.0*A[22]*(pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+exp(-A[0]*pow(rho,2))-1)/(Tsq*pow(A[0],2))
		-5.0/2.0*A[23]*(pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+exp(-A[0]*pow(rho,2))-1)/(Tfr*pow(A[0],2))
		-3.0/2.0*A[24]*(pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+2*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+2*exp(-A[0]*pow(rho,2))-2)/(Tsq*pow(A[0],3))
		-2*A[25]*(pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+2*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+2*exp(-A[0]*pow(rho,2))-2)/(Tcu*pow(A[0],3))
		-3.0/2.0*A[26]*(pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+3*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+6*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+6*exp(-A[0]*pow(rho,2))-6)/(Tsq*pow(A[0],4))
		-5.0/2.0*A[27]*(pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+3*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+6*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+6*exp(-A[0]*pow(rho,2))-6)/(Tfr*pow(A[0],4))
		-3.0/2.0*A[28]*(pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)+4*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+12*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+24*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+24*exp(-A[0]*pow(rho,2))-24)/(Tsq*pow(A[0],5))
		-2*A[29]*(pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)+4*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+12*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+24*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+24*exp(-A[0]*pow(rho,2))-24)/(Tcu*pow(A[0],5))
		-3.0/2.0*A[30]*(pow(rho,10)*exp(-A[0]*pow(rho,2))*pow(A[0],5)+5*pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)+20*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+60*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+120*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+120*exp(-A[0]*pow(rho,2))-120)/(Tsq*pow(A[0],6))
		-2*A[31]*(pow(rho,10)*exp(-A[0]*pow(rho,2))*pow(A[0],5)+5*pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)+20*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+60*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+120*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+120*exp(-A[0]*pow(rho,2))-120)/(Tcu*pow(A[0],6))
		-5.0/2.0*A[32]*(pow(rho,10)*exp(-A[0]*pow(rho,2))*pow(A[0],5)+5*pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)+20*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)+60*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)+120*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+120*exp(-A[0]*pow(rho,2))-120)/(Tfr*pow(A[0],6))

		+(rho*RgasSI/Mw*T + rho*rho*(A[1]*T + A[2]*sqrt(T) + A[3] + A[4]/T 
		+ A[5]/Tsq) + pow(rho,3)*(A[6]*T + A[7] + A[8]/T + A[9]/Tsq) 
		+ pow(rho,4)*(A[10]*T + A[11] + A[12]/T) + pow(rho,5)*A[13] 
		+ pow(rho,6)*(A[14]/T + A[15]/Tsq) + pow(rho,7)*A[16]/T 
		+ pow(rho,8)*(A[17]/T + A[18]/Tsq) + pow(rho,9)*A[19]/Tsq 
		+ (pow(rho,3)*(A[20]/Tsq + A[21]/Tcu) + pow(rho,5)*(A[22]/Tsq 
		+ A[23]/Tfr) + pow(rho,7)*(A[24]/Tsq + A[25]/Tcu) 
		+ pow(rho,9)*(A[26]/Tsq + A[27]/Tfr) + pow(rho,11)*(A[28]/Tsq 
		+ A[29]/Tcu) + pow(rho,13)*(A[30]/Tsq + A[31]/Tcu 
		+ A[32]/Tfr)) * exp(-A[0]*rho*rho))/rho 

		- RgasSI/Mw*T

		)*ConvJkg*Mw;





}



// Enthalpy real part again.  P3 is another EOS.
double HResidualP3( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);



	return (

		(A[2]+2*A[3]/T+3*A[4]/Tsq+4*A[5]/Tcu)*rho
		+(A[7]+2*A[8]/T)*pow(rho,2)/2
		+A[10]*pow(rho,3)/3
		+A[12]*pow(rho,4)/4
		+A[13]*pow(rho,5)/5
		+(-1.0/2.0*(exp(-A[0]*pow(rho,2))-1)/A[0])*(3*A[14]/Tsq+4*A[15]/Tcu+5*A[16]/Tfr)
		+(-1.0/2.0*(pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]+exp(-A[0]*pow(rho,2))-1)/(pow(A[0],2)))
		 *(3*A[17]/Tsq+4*A[18]/Tcu+5*A[19]/Tfr)


		+(rho*RgasSI/Mw*T + rho*rho*(A[1]*T + A[2] + A[3]/T + A[4]/Tsq
		+ A[5]/Tcu) + pow(rho,3)*(A[6]*T + A[7] + A[8]/T) 
		+ pow(rho,4)*(A[9]*T + A[10]) + pow(rho,5)*(A[11]*T + A[12])
		+ pow(rho,6)*A[13] + (pow(rho,3)*(A[14]/Tsq + A[15]/Tcu 
		+ A[16]/Tfr) + pow(rho,5)*(A[17]/Tsq + A[18]/Tcu 
		+ A[19]/Tfr)) * exp(-A[0]*rho*rho))/rho 

		- RgasSI/Mw*T

		)*ConvJkg*Mw;
}

// Enthalpy real part again.  P2 is another EOS.
double HResidualP2( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4);
	double B0=A[0], A0=A[1], C0=A[2], D0=A[3], E0=A[4],
	b=A[5],	a=A[6], d=A[7],
	c=A[8], alpha=A[9], gamma=A[10];


	return (

		-A0*rho
		-3*C0*rho/(Tsq)
		+4*D0*rho/(Tcu)-5*E0*rho/(Tfr)
		+1.0/2.0*(-a-2*d/T)*pow(rho,2)
		+1.0/5.0*alpha*(a+2*d/T)*pow(rho,5)
		-3.0/2.0*c*(pow(rho,2)*exp(-gamma*pow(rho,2))*gamma
		       +exp(-gamma*pow(rho,2))-1)/(gamma*Tsq)
        -3.0/2.0*c*(exp(-gamma*pow(rho,2))-1)/(gamma*Tsq) 
	
		+(rho*RgasSI/Mw*T
		+(B0*RgasSI/Mw*T-A0-C0/(Tsq)
		+D0/(Tcu)-E0/(Tfr))*pow(rho,2)+(b*RgasSI/Mw*T-a-d/T)*pow(rho,3)
		+alpha*(a+d/T)*pow(rho,6)
		+c*pow(rho,3)*(1+gamma*pow(rho,2))*exp(-gamma*pow(rho,2))/(Tsq)
		)/rho 
		
		- RgasSI/Mw*T

		)*ConvJkg*Mw;
}


// Enthalpy real part again.  P6 is another EOS.
double HResidualP6( double V, double T, double Tcrit,
				    const double AA[][8], const double rho_a[], 
					double Mw )
{

	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Pr1,Pr2,Pr3, Tsq;

	T /= 1.8;						// T to Kelvin
	Tsq = T*T;

	Pr1=Fnc_intdQdT(rho, T, Tcrit, AA, rho_a);
	Pr2=Fnc_intdQdrhodT(rho, T, Tcrit, AA, rho_a);
	Pr3= PressureEOS_P6(V, T*1.8, AA, rho_a, Tcrit, Mw )/ConvPa/rho;
	

	return (		
		- RgasSI/Mw*Tsq*(Pr1+Pr2) + Pr3		
		- RgasSI/Mw*T
		)*ConvJkg*Mw;

}

// Enthalpy real part again.  P6 with Q-1.
double HResidualP6Q1( double V, double T, double Tcrit,
				    const double AA[][7], 
					double Mw )
{

	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Pr1,Pr2,Pr3, Tsq;

	T /= 1.8;						// T to Kelvin
	Tsq = T*T;

	Pr1=Fnc_intdQ1dT(rho, T, Tcrit, AA);
	Pr2=Fnc_intdQ1drhodT(rho, T, Tcrit, AA);
	Pr3= PressureEOS_P6Q1(V, T*1.8, AA, Tcrit, Mw )/ConvPa/rho;
	

	return (		
		- RgasSI/Mw*Tsq*(Pr1+Pr2) + Pr3		
		- RgasSI/Mw*T
		)*ConvJkg*Mw;

}

// The following five functions are Cv integrals for the ideal 
// part of the enthalpy equation.  C0 is a Cv equation from 
// Reynolds.  G[] is one element.
double HIdealC0( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (G[1]*ConvJkg*Mw + R*1.8)*(T-T0);

}

// C2 is another Cv equation.  Number of elements in G[] 
// is component dependent. N = G[0] 
double HIdealC2( double T, const double G[], double Mw)
{
	double intCvdT=0;
	int i, N=static_cast<int>(G[0]) ;
	T /= 1.8;						// T to Kelvin

	for (i=1;i<=N;++i) intCvdT += G[i]*(pow(T,i)-pow(T0,i))/i;
	return (intCvdT)*ConvJkg*Mw + R*(T-T0)*1.8;

}

// C3 is another Cv equation.  G[] is 6 elements.  The last
// element ( G[0] ) is beta.
double HIdealC3( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		G[1]*T
		+3.0/4.0*G[2]*pow(T,(4.0/3.0))
		+3.0/5.0*G[3]*pow(T,(5.0/3.0))
		+1.0/2.0*G[4]*pow(T,2)
		-G[1]*T0
		-3.0/4.0*G[2]*pow(T0,(4.0/3.0))
		-3.0/5.0*G[3]*pow(T0,(5.0/3.0))
		-1.0/2.0*G[4]*pow(T0,2) 
		+ G[5]*G[0]*(1/(exp(G[0]/T) - 1) 
		- 1/(exp(G[0]/T0) - 1))
		)*ConvJkg*Mw + R*(T-T0)*1.8;

}

// C4 is another Cv equation.  G[] is 9 elements.  The first
// element ( G[0] ) is beta.
double HIdealC4( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		-G[1]/2*(1/T/T-1/T0/T0) - G[2]*(1/T-1/T0) + G[3]*log(T/T0) 
		+ G[4]*(T-T0) + G[5]/2*(T*T-T0*T0) + G[6]/3*(pow(T,3)-pow(T0,3)) 
		+ G[7]/4*(pow(T,4)-pow(T0,4)) + G[8]*G[0]*(1/(exp(G[0]/T) - 1) 
		- 1/(exp(G[0]/T0) - 1))
		)*ConvJkg*Mw + R*(T-T0)*1.8;

}

// C5 is another Cv equation.  G[] is 19 elements. G[0] is T1
// G[18] is T2
// The constants 812.603 and 4.06623 are found emprically for Hydrogen
// to correct the error in region changes
double HIdealC5( double T, const double G[], double Mw )
{
	double T1=G[0], T2=G[18], HIdeal;
	T /= 1.8;						// T to Kelvin

if (T<=T1) HIdeal = HIdealC0( T, G, Mw ) - 812.603;

if ((T>T1) && (T<=T2))
	HIdeal = (
		-362880*G[10]*T+3628800*G[11]*T
		+20*G[6]*pow(log(T/T1),3)*T
		-120*G[7]*pow(log(T/T1),3)*T
		+30*G[7]*pow(log(T/T1),4)*T
		+42*G[8]*pow(log(T/T1),5)*T
		+G[2]*T*log(T/T1)
		-24*G[5]*T*log(T/T1)
		-4*G[5]*pow(log(T/T1),3)*T
		-990*G[12]*pow(log(T/T1),8)*T
		-3*G[4]*pow(log(T/T1),2)*T
		+6*G[4]*T*log(T/T1)
		-2520*G[8]*pow(log(T/T1),2)*T
		+G[5]*pow(log(T/T1),4)*T
		+G[3]*pow(log(T/T1),2)*T
		+12*G[5]*pow(log(T/T1),2)*T
		+G[7]*pow(log(T/T1),6)*T
		-720*G[7]*T*log(T/T1)
		+G[6]*pow(log(T/T1),5)*T
		+840*G[8]*pow(log(T/T1),3)*T
		+G[4]*pow(log(T/T1),3)*T
		-5*G[6]*pow(log(T/T1),4)*T
		+5040*G[8]*T*log(T/T1)
		+G[8]*pow(log(T/T1),7)*T
		-40320*G[9]*T*log(T/T1)
		+20160*G[9]*pow(log(T/T1),2)*T
		-6720*G[9]*pow(log(T/T1),3)*T
		+1680*G[9]*pow(log(T/T1),4)*T
		-336*G[9]*pow(log(T/T1),5)*T
		+56*G[9]*pow(log(T/T1),6)*T
		-7*G[8]*pow(log(T/T1),6)*T
		-8*G[9]*pow(log(T/T1),7)*T
		+G[9]*pow(log(T/T1),8)*T
		-6*G[7]*pow(log(T/T1),5)*T
		+120*G[6]*T*log(T/T1)
		+360*G[7]*pow(log(T/T1),2)*T
		+39916800*G[12]*T*log(T/T1)
		-19958400*G[12]*pow(log(T/T1),2)*T
		+6652800*G[12]*pow(log(T/T1),3)*T
		-1663200*G[12]*pow(log(T/T1),4)*T
		+332640*G[12]*pow(log(T/T1),5)*T
		-55440*G[12]*pow(log(T/T1),6)*T
		+7920*G[12]*pow(log(T/T1),7)*T
		+G[1]*T-110*G[12]*pow(log(T0/T1),9)*T0
		+720*G[7]*T
		-120*G[6]*T+G[11]*pow(log(T/T1),10)*T
		-2*G[3]*T*log(T/T1)
		-60*G[6]*pow(log(T/T1),2)*T
		+362880*G[10]*T*log(T/T1)
		-181440*G[10]*pow(log(T/T1),2)*T
		+60480*G[10]*pow(log(T/T1),3)*T
		-15120*G[10]*pow(log(T/T1),4)*T
		+3024*G[10]*pow(log(T/T1),5)*T
		-504*G[10]*pow(log(T/T1),6)*T
		+72*G[10]*pow(log(T/T1),7)*T
		-9*G[10]*pow(log(T/T1),8)*T
		+G[10]*pow(log(T/T1),9)*T
		+40320*G[9]*T
		+24*G[5]*T-6*G[4]*T-G[2]*T
		-39916800*G[12]*T
		-210*G[8]*pow(log(T/T1),4)*T
		+2*G[3]*T
		-90*G[11]*pow(log(T0/T1),8)*T0
		+10*G[11]*pow(log(T0/T1),9)*T0
		-G[11]*pow(log(T0/T1),10)*T0
		+2*G[3]*T0*log(T0/T1)
		+60*G[6]*pow(log(T0/T1),2)*T0
		-362880*G[10]*T0*log(T0/T1)
		+181440*G[10]*pow(log(T0/T1),2)*T0
		-60480*G[10]*pow(log(T0/T1),3)*T0
		+15120*G[10]*pow(log(T0/T1),4)*T0
		-3024*G[10]*pow(log(T0/T1),5)*T0
		+504*G[10]*pow(log(T0/T1),6)*T0
		-72*G[10]*pow(log(T0/T1),7)*T0
		-6652800*G[12]*pow(log(T0/T1),3)*T0
		+9*G[10]*pow(log(T0/T1),8)*T0
		-G[10]*pow(log(T0/T1),9)*T0
		+210*G[8]*pow(log(T0/T1),4)*T0
		-20*G[6]*pow(log(T0/T1),3)*T0
		+120*G[7]*pow(log(T0/T1),3)*T0
		-30*G[7]*pow(log(T0/T1),4)*T0
		-42*G[8]*pow(log(T0/T1),5)*T0
		-G[2]*T0*log(T0/T1)
		+24*G[5]*T0*log(T0/T1)
		+4*G[5]*pow(log(T0/T1),3)*T0
		+3*G[4]*pow(log(T0/T1),2)*T0
		-6*G[4]*T0*log(T0/T1)
		+2520*G[8]*pow(log(T0/T1),2)*T0
		-G[5]*pow(log(T0/T1),4)*T0
		-G[3]*pow(log(T0/T1),2)*T0
		-12*G[5]*pow(log(T0/T1),2)*T0
		-G[7]*pow(log(T0/T1),6)*T0
		+720*G[7]*T0*log(T0/T1)
		-G[6]*pow(log(T0/T1),5)*T0
		-840*G[8]*pow(log(T0/T1),3)*T0
		-G[4]*pow(log(T0/T1),3)*T0
		+5*G[6]*pow(log(T0/T1),4)*T0
		-5040*G[8]*T0*log(T0/T1)
		-G[8]*pow(log(T0/T1),7)*T0
		+40320*G[9]*T0*log(T0/T1)
		-20160*G[9]*pow(log(T0/T1),2)*T0
		+6720*G[9]*pow(log(T0/T1),3)*T0
		-1680*G[9]*pow(log(T0/T1),4)*T0
		+336*G[9]*pow(log(T0/T1),5)*T0
		-56*G[9]*pow(log(T0/T1),6)*T0
		+7*G[8]*pow(log(T0/T1),6)*T0
		+8*G[9]*pow(log(T0/T1),7)*T0
		-G[9]*pow(log(T0/T1),8)*T0
		+6*G[7]*pow(log(T0/T1),5)*T0
		-120*G[6]*T0*log(T0/T1)
		-360*G[7]*pow(log(T0/T1),2)*T0
		-39916800*G[12]*T0*log(T0/T1)
		-11*G[12]*pow(log(T/T1),10)*T
		+G[12]*pow(log(T/T1),11)*T
		-3628800*G[11]*T*log(T/T1)
		+1814400*G[11]*pow(log(T/T1),2)*T
		-604800*G[11]*pow(log(T/T1),3)*T
		+151200*G[11]*pow(log(T/T1),4)*T
		-30240*G[11]*pow(log(T/T1),5)*T
		+5040*G[11]*pow(log(T/T1),6)*T
		-720*G[11]*pow(log(T/T1),7)*T
		+90*G[11]*pow(log(T/T1),8)*T
		-10*G[11]*pow(log(T/T1),9)*T
		+55440*G[12]*pow(log(T0/T1),6)*T0
		-7920*G[12]*pow(log(T0/T1),7)*T0
		+11*G[12]*pow(log(T0/T1),10)*T0
		-G[12]*pow(log(T0/T1),11)*T0
		+3628800*G[11]*T0*log(T0/T1)
		-1814400*G[11]*pow(log(T0/T1),2)*T0
		+604800*G[11]*pow(log(T0/T1),3)*T0
		-151200*G[11]*pow(log(T0/T1),4)*T0
		+30240*G[11]*pow(log(T0/T1),5)*T0
		-5040*G[11]*pow(log(T0/T1),6)*T0
		+720*G[11]*pow(log(T0/T1),7)*T0
		+19958400*G[12]*pow(log(T0/T1),2)*T0
		-332640*G[12]*pow(log(T0/T1),5)*T0
		-5040*G[8]*T+990*G[12]*pow(log(T0/T1),8)*T0
		+110*G[12]*pow(log(T/T1),9)*T
		+1663200*G[12]*pow(log(T0/T1),4)*T0
		+5040*G[8]*T0+39916800*G[12]*T0
		+G[2]*T0+6*G[4]*T0-24*G[5]*T0
		-40320*G[9]*T0-G[1]*T0
		-3628800*G[11]*T0
		+362880*G[10]*T0+120*G[6]*T0
		-720*G[7]*T0-2*G[3]*T0

		)*ConvJkg*Mw + R*(T-T0)*1.8;


if (T>T2) HIdeal = (

		-G[17]*pow(log(T0/T2),4)*T0-6*G[16]*T0*log(T0/T2)-G[14]*T0*log(T0/T2)
		+24*G[17]*T0*log(T0/T2)-12*G[17]*pow(log(T0/T2),2)*T0
		+3*G[16]*pow(log(T0/T2),2)*T0-24*G[17]*T*log(T/T2)
		+G[17]*pow(log(T/T2),4)*T+G[14]*T*log(T/T2)
		-4*G[17]*pow(log(T/T2),3)*T+4*G[17]*pow(log(T0/T2),3)*T0
		+2*G[15]*T0*log(T0/T2)-G[15]*pow(log(T0/T2),2)*T0
		+G[15]*pow(log(T/T2),2)*T-2*G[15]*T*log(T/T2)
		+G[16]*pow(log(T/T2),3)*T-G[16]*pow(log(T0/T2),3)*T0
		+12*G[17]*pow(log(T/T2),2)*T-3*G[16]*pow(log(T/T2),2)*T
		+6*G[16]*T*log(T/T2)+G[14]*T0+2*G[15]*T-G[14]*T+G[13]*T-6*G[16]*T
		+24*G[17]*T-G[13]*T0-2*G[15]*T0+6*G[16]*T0-24*G[17]*T0

		)*ConvJkg*Mw + R*(T-T0)*1.8 + 4.06623;
	
	return HIdeal;

}

// C6 is another Cv equation.  G[] is 7 elements, but G[0]
// is not used.
double HIdealC6( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		G[1]*log(T/T0) + G[2]*(T-T0) + G[3]/2*(T*T-T0*T0) 
		+ G[4]/3*(pow(T,3)-pow(T0,3)) + G[5]/4*(pow(T,4)-pow(T0,4)) 
		+ G[6]/5*(pow(T,5)-pow(T0,5))
		)*ConvJkg*Mw + R*(T-T0)*1.8;
}


// The entropy is computed from the ideal case (the
// integral of Cv/T from T0 to T at zero density) plus an 
// integral that accounts for the density increase in a 
// real gas (the residual entropy).  The following 
// functions are for the real integral.  S means entropy.  
// P4 is the EOS (in Reynolds) that was used to model the 
// real gas. 
double SResidualP4( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3

	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4), Tfv = pow(T,5);

	return (

		(-A[1]-1.0/2.0*A[2]/(sqrt(T))+A[4]/(Tsq)+2*A[5]/(Tcu))*rho
		-pow(rho,2)/2*(A[6]-A[8]/(Tsq)-2*A[9]/(Tcu))
		-pow(rho,3)/3*(A[10]-A[12]/(Tsq))
		-pow(rho,5)/5*(-A[14]/(Tsq)-2*A[15]/(Tcu))
		+pow(rho,6)/6*A[16]/(Tsq)
		-pow(rho,7)/7*(-A[17]/(Tsq)-2*A[18]/(Tcu))
		+2*pow(rho,8)/8*A[19]/(Tcu)

		+(2*A[20]/(Tcu)+3*A[21]/(Tfr))*(-1.0/2.0*(exp(-A[0]*pow(rho,2))-1)/A[0])
		+(2*A[22]/(Tcu)+4*A[23]/(Tfv))*(-1.0/2.0*(pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]
										  +exp(-A[0]*pow(rho,2))-1)/(pow(A[0],2)))
		+(2*A[24]/(Tcu)+3*A[25]/(Tfr))*(-1.0/2.0*(pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)
										+2*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]
										+2*exp(-A[0]*pow(rho,2))-2)/(pow(A[0],3)))
		+(2*A[26]/(Tcu)+4*A[27]/(Tfv))*(-1.0/2.0*(pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)
											+3*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)
											+6*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]
											+6*exp(-A[0]*pow(rho,2))-6)/(pow(A[0],4)))
		+(2*A[28]/(Tcu)+3*A[29]/(Tfr))*(-1.0/2.0*(pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)
											+4*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)
											+12*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)
											+24*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]
											+24*exp(-A[0]*pow(rho,2))-24)/(pow(A[0],5)))
		+(2*A[30]/(Tcu)+3*A[31]/(Tfr)+4*A[32]/(Tfv))*(-1.0/2.0*(pow(rho,10)*exp(-A[0]*pow(rho,2))*pow(A[0],5)
														+5*pow(rho,8)*exp(-A[0]*pow(rho,2))*pow(A[0],4)
														+20*pow(rho,6)*exp(-A[0]*pow(rho,2))*pow(A[0],3)
														+60*pow(rho,4)*exp(-A[0]*pow(rho,2))*pow(A[0],2)
														+120*pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]
														+120*exp(-A[0]*pow(rho,2))-120)/(pow(A[0],6)))



		)*ConvJkg*Mw/1.8;
}

// Entropy real part (residual) again using P3 EOS.
double SResidualP3( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3

	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4), Tfv = pow(T,5);

	return (
		(-A[1]+A[3]/(Tsq)+2*A[4]/(Tcu)+3*A[5]/(Tfr))*rho
		-pow(rho,2)/2*(A[6]-A[8]/(Tsq))
		-pow(rho,3)/3*A[9]
		-pow(rho,4)/4*A[11]
		+(2*A[14]/(Tcu)+3*A[15]/(Tfr)+4*A[16]/(Tfv))*(-1.0/2.0*(exp(-A[0]*pow(rho,2))-1)/A[0])
		+(2*A[17]/(Tcu)+3*A[18]/(Tfr)+4*A[19]/(Tfv))*(-1.0/2.0*(pow(rho,2)*exp(-A[0]*pow(rho,2))*A[0]
													 +exp(-A[0]*pow(rho,2))-1)/(pow(A[0],2)))

		)*ConvJkg*Mw/1.8;
}

// Entropy real part (residual) again using P2 EOS.
double SResidualP2( double V, double T, const double A[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Tsq = T*T, Tcu = pow(T,3), Tfr = pow(T,4), Tfv = pow(T,5);
	double B0=A[0], A0=A[1], C0=A[2], D0=A[3], E0=A[4],
	b=A[5],	a=A[6], d=A[7],
	c=A[8], alpha=A[9], gamma=A[10];


	return (

		1.0/5.0*alpha*d*pow(rho,5)/(Tsq)
		+1.0/2.0*(-b*RgasSI/Mw-d/(Tsq))*pow(rho,2)
		-B0*RgasSI/Mw*rho-2*C0*rho/(Tcu)
		+3*D0*rho/(Tfr)
		-4*E0*rho/(Tfv)
		-c*(1+exp(-gamma*pow(rho,2))
		                *exp(gamma*pow(rho,2))
						*gamma*pow(rho,2)
		+exp(-gamma*pow(rho,2))*exp(gamma*pow(rho,2))
		-2*exp(gamma*pow(rho,2)))/(gamma*Tcu*exp(gamma*pow(rho,2)))

		)*ConvJkg*Mw/1.8;
}

// Entropy real part again.  P6 is another EOS.
double SResidualP6( double V, double T, double Tcrit,
				    const double AA[][8], const double rho_a[], 
					double Mw )
{

	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Pr1,Pr2,Pr3, Pr4;

	T /= 1.8;						// T to Kelvin

	Pr1= Fnc_intQ(rho, T, Tcrit, AA, rho_a);
	Pr2= Fnc_intdQdT(rho, T, Tcrit, AA, rho_a);
	Pr3= Fnc_intdQdrho(rho, T, Tcrit, AA, rho_a);
	Pr4= Fnc_intdQdrhodT(rho, T, Tcrit, AA, rho_a);
	
	return (		
		- (RgasSI/Mw*Pr1 + RgasSI/Mw*T*Pr2 + RgasSI/Mw*Pr3 + RgasSI/Mw*T*Pr4)		
		   )*ConvJkg*Mw/1.8;

}

// Entropy real part again.  P6 with Q-1
double SResidualP6Q1( double V, double T, double Tcrit,
				    const double AA[][7], double Mw )
{

	double rho = (1/V)*ConvM3kg*Mw;	// V from ft3/lbmol to rho in kg/m3
	double Pr1,Pr2,Pr3,Pr4;

	T /= 1.8;						// T to Kelvin

	Pr1= Fnc_intQ1(rho, T, Tcrit, AA);
	Pr2= Fnc_intdQ1dT(rho, T, Tcrit, AA);
	Pr3= Fnc_intdQ1drho(rho, T, Tcrit, AA);
	Pr4= Fnc_intdQ1drhodT(rho, T, Tcrit, AA);
	
	return (		
		- (RgasSI/Mw*Pr1 + RgasSI/Mw*T*Pr2 + RgasSI/Mw*Pr3 + RgasSI/Mw*T*Pr4)		
		   )*ConvJkg*Mw/1.8;

}


// The following six functions are Cv integrals for the ideal 
// part of the entropy equation plus the ln(V) term.  See the 
// comments in the ideal enthalpy function section for G[].  C0 
// is a Cv equation from Reynolds.
double SIdealC0( double V, double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (G[1]*log(T/T0)-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;
}

// C2 is another Cv equation.  Number of elements in G[] 
// is component dependent. N = G[0]
double SIdealC2( double V, double T, const double G[], double Mw )
{
	double intCvTdT=0;
	int i,N=static_cast<int>(G[0]);
	T /= 1.8;						// T to Kelvin
	intCvTdT=G[1]*log(T)-G[1]*log(T0);
	for (i=2;i<=N;++i) intCvTdT += G[i]*(pow(T,(i-1))-pow(T0,(i-1)))/(i-1);

	return (
		intCvTdT
		-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;

}

// C3 is another Cv equation.  G[] is 6 elements.  The last
// element ( G[0] ) is beta.
double SIdealC3( double V, double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin

	return (
		G[1]*log(T)-G[1]*log(T0)
		+3*G[2]*pow(T,(1.0/3.0))-3*G[2]*pow(T0,(1.0/3.0))
		+3.0/2.0*G[3]*pow(T,(2.0/3.0))-3.0/2.0*G[3]*pow(T0,(2.0/3.0))
		+G[4]*(T-T0)

		+G[5]*(-log(exp(G[0]/T)-1)*T*T0*exp(G[0]/T)*exp(G[0]/T0)
		+log(exp(G[0]/T)-1)*T*T0*exp(G[0]/T)
		+log(exp(G[0]/T)-1)*T*T0*exp(G[0]/T0)-log(exp(G[0]/T)-1)*T*T0
		+G[0]*exp(G[0]/T)*T0*exp(G[0]/T0)-G[0]*exp(G[0]/T)*T0
		+log(exp(G[0]/T0)-1)*T*T0*exp(G[0]/T)*exp(G[0]/T0)
		-log(exp(G[0]/T0)-1)*T*T0*exp(G[0]/T)
		-log(exp(G[0]/T0)-1)*T*T0*exp(G[0]/T0)+log(exp(G[0]/T0)-1)*T*T0
		-G[0]*exp(G[0]/T0)*T*exp(G[0]/T)
		+G[0]*exp(G[0]/T0)*T)/(T*(exp(G[0]/T)-1)*T0*(exp(G[0]/T0)-1))

		-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;


}

// C4 is another Cv equation.
double SIdealC4( double V, double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		-G[1]/3*(1/pow(T,3)-1/pow(T0,3)) - G[2]*(1/T/T-1/T0/T0) 
		- G[3]*(1/T-1/T0) + G[4]*log(T/T0) + G[5]*(T-T0) 
		+ G[6]/2*(T*T-T0*T0) + G[7]/3*(pow(T,3)-pow(T0,3)) 
		- G[8]*(log((exp(G[0]/T) - 1)/(exp(G[0]/T0) - 1))
		- G[0]*( 1/T/(1 - exp(-G[0]/T)) - 1/T0/(1 - exp(-G[0]/T0)) ))
		-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;
}

// C5 is another Cv equation.  G[] is 19 elements. G[0] is T1
// G[18] is T2
double SIdealC5( double V, double T, const double G[], double Mw )
{
	double T1=G[0], T2=G[18], SIdeal;
	T /= 1.8;						// T to Kelvin

if (T<=T1) SIdeal = SIdealC0( V, T, G, Mw );
if ((T>T1) && (T<=T2))
	SIdeal = (

		G[1]*log(T)+1.0/2.0*G[2]*pow(log(T/T1),2)
		+1.0/3.0*G[3]*pow(log(T/T1),3)
		+1.0/4.0*G[4]*pow(log(T/T1),4)+1.0/5.0*G[5]*pow(log(T/T1),5)
		+1.0/6.0*G[6]*pow(log(T/T1),6)+1.0/7.0*G[7]*pow(log(T/T1),7)
		+1.0/8.0*G[8]*pow(log(T/T1),8)+1.0/9.0*G[9]*pow(log(T/T1),9)
		+1.0/10.0*G[10]*pow(log(T/T1),10)+1.0/11.0*G[11]*pow(log(T/T1),11)
		+1.0/12.0*G[12]*pow(log(T/T1),12)
		-G[1]*log(T0)-1.0/2.0*G[2]*pow(log(T0/T1),2)
		-1.0/3.0*G[3]*pow(log(T0/T1),3)-1.0/4.0*G[4]*pow(log(T0/T1),4)
		-1.0/5.0*G[5]*pow(log(T0/T1),5)-1.0/6.0*G[6]*pow(log(T0/T1),6)
		-1.0/7.0*G[7]*pow(log(T0/T1),7)-1.0/8.0*G[8]*pow(log(T0/T1),8)
		-1.0/9.0*G[9]*pow(log(T0/T1),9)-1.0/10.0*G[10]*pow(log(T0/T1),10)
		-1.0/11.0*G[11]*pow(log(T0/T1),11)-1.0/12.0*G[12]*pow(log(T0/T1),12)
		
		-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;

if (T>T2) SIdeal = (

		G[13]*log(T)
		+1.0/2.0*G[14]*pow(log(T/T2),2)
		+1.0/3.0*G[15]*pow(log(T/T2),3)
		+1.0/4.0*G[16]*pow(log(T/T2),4)
		+1.0/5.0*G[17]*pow(log(T/T2),5)
		-G[13]*log(T0)
		-1.0/2.0*G[14]*pow(log(T0/T2),2)
		-1.0/3.0*G[15]*pow(log(T0/T2),3)
		-1.0/4.0*G[16]*pow(log(T0/T2),4)
		-1.0/5.0*G[17]*pow(log(T0/T2),5)
	
		-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;

	return SIdeal;
}

// C6 is another Cv equation.
double SIdealC6( double V, double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		-G[1]*(1/T-1/T0) + G[2]*log(T/T0) + G[3]*(T-T0) 
		+ G[4]/2*(T*T-T0*T0) + G[5]/3*(pow(T,3)-pow(T0,3)) 
		+ G[6]/4*(pow(T,4)-pow(T0,4))
		-RgasSI/Mw*log((1/V)*ConvM3kg*Mw))*ConvJkg*Mw/1.8;
}


// The following six functions are Cv integrals for the ideal 
// part of the entropy equation excluding the ln(V) term. They will 
// be used to obtain initial guesses for P_ST functions

double SIdealC0x( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (G[1]*log(T/T0))*ConvJkg*Mw/1.8;
}

double SIdealC2x( double T, const double G[], double Mw )
{
	double intCvTdT=0;
	int i,N=static_cast<int>(G[0]);
	T /= 1.8;						// T to Kelvin
	intCvTdT=G[1]*log(T)-G[1]*log(T0);
	for (i=2;i<=N;++i) intCvTdT += G[i]*(pow(T,(i-1))-pow(T0,(i-1)))/(i-1);

	return (
		intCvTdT)*ConvJkg*Mw/1.8;

}

double SIdealC3x( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin

	return (
		G[1]*log(T)-G[1]*log(T0)
		+3*G[2]*pow(T,(1.0/3.0))-3*G[2]*pow(T0,(1.0/3.0))
		+3.0/2.0*G[3]*pow(T,(2.0/3.0))-3.0/2.0*G[3]*pow(T0,(2.0/3.0))
		+G[4]*(T-T0)

		+G[5]*(-log(exp(G[0]/T)-1)*T*T0*exp(G[0]/T)*exp(G[0]/T0)
		+log(exp(G[0]/T)-1)*T*T0*exp(G[0]/T)
		+log(exp(G[0]/T)-1)*T*T0*exp(G[0]/T0)-log(exp(G[0]/T)-1)*T*T0
		+G[0]*exp(G[0]/T)*T0*exp(G[0]/T0)-G[0]*exp(G[0]/T)*T0
		+log(exp(G[0]/T0)-1)*T*T0*exp(G[0]/T)*exp(G[0]/T0)
		-log(exp(G[0]/T0)-1)*T*T0*exp(G[0]/T)
		-log(exp(G[0]/T0)-1)*T*T0*exp(G[0]/T0)+log(exp(G[0]/T0)-1)*T*T0
		-G[0]*exp(G[0]/T0)*T*exp(G[0]/T)
		+G[0]*exp(G[0]/T0)*T)/(T*(exp(G[0]/T)-1)*T0*(exp(G[0]/T0)-1))

		)*ConvJkg*Mw/1.8;


}

double SIdealC4x( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		-G[1]/3*(1/pow(T,3)-1/pow(T0,3)) - G[2]*(1/T/T-1/T0/T0) 
		- G[3]*(1/T-1/T0) + G[4]*log(T/T0) + G[5]*(T-T0) 
		+ G[6]/2*(T*T-T0*T0) + G[7]/3*(pow(T,3)-pow(T0,3)) 
		- G[8]*(log((exp(G[0]/T) - 1)/(exp(G[0]/T0) - 1))
		- G[0]*( 1/T/(1 - exp(-G[0]/T)) - 1/T0/(1 - exp(-G[0]/T0)) ))
		)*ConvJkg*Mw/1.8;
}

double SIdealC5x( double T, const double G[], double Mw )
{
	double T1=G[0], T2=G[18], SIdeal;
	T /= 1.8;						// T to Kelvin

if (T<=T1) SIdeal = SIdealC0x( T, G, Mw );
if ((T>T1) && (T<=T2))
	SIdeal = (

		G[1]*log(T)+1.0/2.0*G[2]*pow(log(T/T1),2)
		+1.0/3.0*G[3]*pow(log(T/T1),3)
		+1.0/4.0*G[4]*pow(log(T/T1),4)+1.0/5.0*G[5]*pow(log(T/T1),5)
		+1.0/6.0*G[6]*pow(log(T/T1),6)+1.0/7.0*G[7]*pow(log(T/T1),7)
		+1.0/8.0*G[8]*pow(log(T/T1),8)+1.0/9.0*G[9]*pow(log(T/T1),9)
		+1.0/10.0*G[10]*pow(log(T/T1),10)+1.0/11.0*G[11]*pow(log(T/T1),11)
		+1.0/12.0*G[12]*pow(log(T/T1),12)
		-G[1]*log(T0)-1.0/2.0*G[2]*pow(log(T0/T1),2)
		-1.0/3.0*G[3]*pow(log(T0/T1),3)-1.0/4.0*G[4]*pow(log(T0/T1),4)
		-1.0/5.0*G[5]*pow(log(T0/T1),5)-1.0/6.0*G[6]*pow(log(T0/T1),6)
		-1.0/7.0*G[7]*pow(log(T0/T1),7)-1.0/8.0*G[8]*pow(log(T0/T1),8)
		-1.0/9.0*G[9]*pow(log(T0/T1),9)-1.0/10.0*G[10]*pow(log(T0/T1),10)
		-1.0/11.0*G[11]*pow(log(T0/T1),11)-1.0/12.0*G[12]*pow(log(T0/T1),12)
		
		)*ConvJkg*Mw/1.8;

if (T>T2) SIdeal = (

		G[13]*log(T)
		+1.0/2.0*G[14]*pow(log(T/T2),2)
		+1.0/3.0*G[15]*pow(log(T/T2),3)
		+1.0/4.0*G[16]*pow(log(T/T2),4)
		+1.0/5.0*G[17]*pow(log(T/T2),5)
		-G[13]*log(T0)
		-1.0/2.0*G[14]*pow(log(T0/T2),2)
		-1.0/3.0*G[15]*pow(log(T0/T2),3)
		-1.0/4.0*G[16]*pow(log(T0/T2),4)
		-1.0/5.0*G[17]*pow(log(T0/T2),5)
	
		)*ConvJkg*Mw/1.8;

	return SIdeal;
}

double SIdealC6x( double T, const double G[], double Mw )
{
	T /= 1.8;						// T to Kelvin
	return (
		-G[1]*(1/T-1/T0) + G[2]*log(T/T0) + G[3]*(T-T0) 
		+ G[4]/2*(T*T-T0*T0) + G[5]/3*(pow(T,3)-pow(T0,3)) 
		+ G[6]/4*(pow(T,4)-pow(T0,4))
		)*ConvJkg*Mw/1.8;
}

#line 1000
// Enthalpy and Entropy skeleton functions for pure components.
double PureEnthalpy( double V, double T, double Mw, const double A[], 
					 const double G[], double Href, int EOS_Eqn, 
					 int Cv_Eqn )
{
	double RealPart, IdealPart;
	
	switch ( EOS_Eqn ) {
		case 2:		// EOS P-2
			RealPart = HResidualP2(V,T,A,Mw);
			break;
		case 3:		// EOS P-3
			RealPart = HResidualP3(V,T,A,Mw);
			break;
		case 4:		// EOS P-4
			RealPart = HResidualP4(V,T,A,Mw);
			break;
	}

	switch ( Cv_Eqn ) {
		case 0:		// Cv C-0
			IdealPart = HIdealC0(T,G,Mw);
			break;
		case 2:		// Cv C-2
			IdealPart = HIdealC2(T,G,Mw);
			break;
		case 3:		// Cv C-3
			IdealPart = HIdealC3(T,G,Mw);
			break;
		case 4:		// Cv C-4
			IdealPart = HIdealC4(T,G,Mw);
			break;
		case 5:		// Cv C-5
			IdealPart = HIdealC5(T,G,Mw);
			break;
		case 6:		// Cv C-6
			IdealPart = HIdealC6(T,G,Mw);
			break;
	}

	return IdealPart + RealPart - Href;

	
}

#line 1200
// Enthalpy and Entropy skeleton functions for pure components.
double PureEnthalpyP6( double V, double T, double Mw, 
					 const double AA[][8], const double rho_a[], 
					 const double G[], double Href, double Tcrit, 
					 int Cv_Eqn  )
{
	double RealPart, IdealPart;
	
			RealPart = HResidualP6(V,T,Tcrit,AA,rho_a,Mw);
			IdealPart = HIdealC6(T,G,Mw);

	return IdealPart + RealPart - Href;

}

// Enthalpy and Entropy skeleton functions for pure components.
double PureEnthalpyP6Q1( double V, double T, double Mw, 
					 const double AA[][7], const double G[], 
					 double Href, double Tcrit, int Cv_Eqn  )
{
	double RealPart, IdealPart;
	
			RealPart = HResidualP6Q1(V,T,Tcrit,AA,Mw);
			IdealPart = HIdealC2(T,G,Mw);

	return IdealPart + RealPart - Href;

}

// Enthalpy function for pure components.
// Using Polynomial coefficients for Cp
double PureEnthalpyPoly( double T, double Mw, 
					 const double A[], double Href)
{
	double EnthalpyinCalperMol, x, x0, c=1.0*1E-3;
	T /= 1.8;						// T to Kelvin

	x=c*T;
	x0=c*T0;
	EnthalpyinCalperMol = A[1]*(x-x0)/c
						 +A[2]/2.0*(pow(x,2)-pow(x0,2))/c
						 +A[3]/3.0*(pow(x,3)-pow(x0,3))/c
						 +A[4]/4.0*(pow(x,4)-pow(x0,4))/c
						 +A[5]/5.0*(pow(x,5)-pow(x0,5))/c
						 +A[6]/6.0*(pow(x,6)-pow(x0,6))/c
						 +A[7]/7.0*(pow(x,7)-pow(x0,7))/c;

	return (EnthalpyinCalperMol - Href)*Convcalmole;

}

double PureEntropy( double V, double T, double Mw, const double A[], 
					const double G[], double Sref, int EOS_Eqn, 
					int Cv_Eqn )
{
	double RealPart, IdealPart;
	
	switch ( EOS_Eqn ) {
		case 2:		// EOS P-2
			RealPart = SResidualP2(V,T,A,Mw);
			break;
		case 3:		// EOS P-3
			RealPart = SResidualP3(V,T,A,Mw);
			break;
		case 4:		// EOS P-4
			RealPart = SResidualP4(V,T,A,Mw);
			break;
	}

	switch ( Cv_Eqn ) {
		case 0:		// Cv C-0
			IdealPart = SIdealC0(V,T,G,Mw);
			break;
		case 2:		// Cv C-2
			IdealPart = SIdealC2(V,T,G,Mw);
			break;
		case 3:		// Cv C-3
			IdealPart = SIdealC3(V,T,G,Mw);
			break;
		case 4:		// Cv C-4
			IdealPart = SIdealC4(V,T,G,Mw);
			break;
		case 5:		// Cv C-5
			IdealPart = SIdealC5(V,T,G,Mw);
			break;
		case 6:		// Cv C-6
			IdealPart = SIdealC6(V,T,G,Mw);
			break;
	}

	return IdealPart + RealPart - Sref;

}

#line 1600
// Enthalpy and Entropy skeleton function for pure component (Water).
double PureEntropyP6( double V, double T, double Mw, 
					 const double AA[][8], const double rho_a[], 
					 const double G[], double Sref, double Tcrit, 
					 int Cv_Eqn )
{
	double RealPart, IdealPart;
	
			RealPart = SResidualP6(V,T,Tcrit,AA,rho_a,Mw);
			IdealPart = SIdealC6(V,T,G,Mw);

	
	return IdealPart + RealPart - Sref;

}

// Enthalpy and Entropy skeleton function for pure component.
double PureEntropyP6Q1( double V, double T, double Mw, 
					 const double AA[][7], const double G[], 
					 double Sref, double Tcrit, int Cv_Eqn )
{
	double RealPart, IdealPart;
	
			RealPart = SResidualP6Q1(V,T,Tcrit,AA,Mw);
			IdealPart = SIdealC2(V,T,G,Mw);

	
	return IdealPart + RealPart - Sref;

}

// Entropy function for pure components.
// Using Polynomial coefficients for Cp
double PureEntropyPoly( double P, double T, double Mw, 
					 const double A[], double Sref)
{
	double EntropyinCalperMolK, x, x0;
	T /= 1.8;						// T to Kelvin

	x=1.0*1E-3*T;
	x0=1.0*1E-3*T0;
	EntropyinCalperMolK = A[1]*log(T/T0)
						 +A[2]/1.0*(x-x0)
						 +A[3]/2.0*(pow(x,2)-pow(x0,2))
						 +A[4]/3.0*(pow(x,3)-pow(x0,3))
						 +A[5]/4.0*(pow(x,4)-pow(x0,4))
						 +A[6]/5.0*(pow(x,5)-pow(x0,5))
						 +A[7]/6.0*(pow(x,6)-pow(x0,6))
						 -Rcalmol*log(P/P0);

	return (EntropyinCalperMolK - Sref)*Convcalmole/1.8;

}

// Ideal Enthalpy and Entropy skeleton functions for pure components.
double IdealPEnthalpy( double T, double Mw,const double G[], int Cv_Eqn )
{
	double IdealPart;


	switch ( Cv_Eqn ) {
		case 0:		// Cv C-0
			IdealPart = HIdealC0(T,G,Mw);
			break;
		case 2:		// Cv C-2
			IdealPart = HIdealC2(T,G,Mw);
			break;
		case 3:		// Cv C-3
			IdealPart = HIdealC3(T,G,Mw);
			break;
		case 4:		// Cv C-4
			IdealPart = HIdealC4(T,G,Mw);
			break;
		case 5:		// Cv C-5
			IdealPart = HIdealC5(T,G,Mw);
			break;
		case 6:		// Cv C-6
			IdealPart = HIdealC6(T,G,Mw);
			break;
	}

	return IdealPart;
}

double IdealPEntropy( double V, double T, double Mw, const double G[], 
					int Cv_Eqn )
{
	double IdealPart;
	

	switch ( Cv_Eqn ) {
		case 0:		// Cv C-0
			IdealPart = SIdealC0(V,T,G,Mw);
			break;
		case 2:		// Cv C-2
			IdealPart = SIdealC2(V,T,G,Mw);
			break;
		case 3:		// Cv C-3
			IdealPart = SIdealC3(V,T,G,Mw);
			break;
		case 4:		// Cv C-4
			IdealPart = SIdealC4(V,T,G,Mw);
			break;
		case 5:		// Cv C-5
			IdealPart = SIdealC5(V,T,G,Mw);
			break;
		case 6:		// Cv C-6
			IdealPart = SIdealC6(V,T,G,Mw);
			break;
	}

	return IdealPart ;

}

//Returns the ideal part vithout the ln(V) term
double IdealPEntropyx( double T, double Mw, const double G[], 
					int Cv_Eqn )
{
	double IdealPart;
	

	switch ( Cv_Eqn ) {
		case 0:		// Cv C-0
			IdealPart = SIdealC0x(T,G,Mw);
			break;
		case 2:		// Cv C-2
			IdealPart = SIdealC2x(T,G,Mw);
			break;
		case 3:		// Cv C-3
			IdealPart = SIdealC3x(T,G,Mw);
			break;
		case 4:		// Cv C-4
			IdealPart = SIdealC4x(T,G,Mw);
			break;
		case 5:		// Cv C-5
			IdealPart = SIdealC5x(T,G,Mw);
			break;
		case 6:		// Cv C-6
			IdealPart = SIdealC6x(T,G,Mw);
			break;
	}

	return IdealPart ;

}

#line 2000
// Enthalpy and entropy functions for air.  Mw[] is the molecular weights
// of air, argon, nitrogen, and oxygen.  yi[] is mole fractions.  G1[] is
// for argon, G2[] for nitrogen, G3[] for oxygen.
double MixedEnthalpy( double V, double T, double Mw[], 
					  const double A[], double yi[],
			 		  const double G1[], const double G2[], const double G3[],
					  double Href )
{
	double RealPart, IdealPart;
	
	RealPart = HResidualP4(V,T,A,Mw[0]);

	IdealPart = yi[1]*HIdealC0(T,G1,Mw[1])
			  + yi[2]*HIdealC4(T,G2,Mw[2])
			  + yi[3]*HIdealC4(T,G3,Mw[3]);
	
	return IdealPart + RealPart - Href;

}

double MixedEntropy( double V, double T, double Mw[], 
					 const double A[], double yi[],
			 		 const double G1[], const double G2[], const double G3[],
					 double Sref )
{
	double RealPart, IdealPart;
	
	RealPart = SResidualP4(V,T,A,Mw[0]);

	IdealPart = yi[1]*SIdealC0(V,T,G1,Mw[1])
			  + yi[2]*SIdealC4(V,T,G2,Mw[2])
			  + yi[3]*SIdealC4(V,T,G3,Mw[3]);
	
	return IdealPart + RealPart - Sref;
}

// Saturation pressure skeleton functions for pure components.
double PureSatPressure( double T, double Mw, const double F[], 
					    double Tcrit, double Pcrit, int SatP_Eqn)
{
	double SatPressure;
	
	switch ( SatP_Eqn ) {
		case 6:		// Sat Pressure Equation S-6
			SatPressure = SatPressure_S6(T,F,Tcrit,Pcrit);
			break;
	}

	return SatPressure;
}


double PureSatLDensity( double T, double Mw, const double D[], 
					    double Tcrit, double rhocrit, int SatLrho_Eqn)
{
	double SatLiquidDensity;
	
	switch ( SatLrho_Eqn ) {
		case 5:		// Sat Liquid Density Equation D-5
			SatLiquidDensity = SLDensity_D5(T,D,Tcrit,rhocrit,Mw);
			break;

	}

	return SatLiquidDensity;
}


///////////////// Secant method root finder ////////////////////////////////  
// Inputs
//	  Target	Value of function where root (Arg1) is to be found.
//	  *Arg1		Desired variable and first argument of evaluated function.
//	  			MUST have an initial guess assigned when Secant is called.
//	  Arg2		Second argument of evaluated function.
//	  (*Fnct)	Pointer to function F(Arg1,Arg2) that is evaluated.
//	  MErr		Maximum error (stopping criteria)
//    MIter		Maximum iterations allowed (stopping criteria)
// Outputs
//    *Arg1		Solution is in same address as initial guess.
//	  Secant	Number of iterations.
////////////////////////////////////////////////////////////////////////////
int Secant( double Target, double *Arg1, double Arg2, 
		    double ( *Fnct )( double , double ), double MErr, int MIter )
{
	double ThisGuess = *Arg1 * 1.01, OldGuess;
	double ThisResult = (*Fnct)( ThisGuess, Arg2 ) - Target, OldResult;
	double delX, AproxErr;
	int iterations = 0;

	// Secant method
	do {
		OldGuess = ThisGuess; ThisGuess = *Arg1; OldResult = ThisResult;
		delX = OldGuess - ThisGuess;
		ThisResult = (*Fnct)( *Arg1, Arg2 ) - Target;
		*Arg1 = ThisGuess - ThisResult*delX/(OldResult - ThisResult);
		AproxErr = (ThisGuess - *Arg1)/(*Arg1);
		iterations++;
	} while( fabs(AproxErr) > MErr && iterations < MIter );
	
	return iterations;
}


